@@ -9,27 +9,34 @@
 #  cases as published by the Free Software Foundation.
 #
 
+import sys
 import json
 import os
 import logging
+import logging.handlers
 import fcntl
 from errno import EBADF
 from threading import Thread
 import multiprocessing
-from Queue import Queue
+try:
+    from queue import Queue
+except ImportError:
+    from Queue import Queue
 from datetime import datetime, timedelta
 import base64
 import hmac
 from hashlib import sha256
 from calendar import timegm
 
-from eventsapiconf import (LOG_FILE,
-                           WEBHOOKS_FILE,
-                           DEFAULT_CONFIG_FILE,
-                           CUSTOM_CONFIG_FILE,
-                           UUID_FILE,
-                           CERTS_DIR)
-import eventtypes
+sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
+
+from gfevents.eventsapiconf import (LOG_FILE,
+                                    WEBHOOKS_FILE,
+                                    DEFAULT_CONFIG_FILE,
+                                    CUSTOM_CONFIG_FILE,
+                                    UUID_FILE,
+                                    CERTS_DIR)
+from gfevents import eventtypes
 
 
 # Webhooks list
@@ -92,7 +99,7 @@ def setup_logger():
     logger.setLevel(logging.INFO)
 
     # create the logging file handler
-    fh = logging.FileHandler(LOG_FILE)
+    fh = logging.handlers.WatchedFileHandler(LOG_FILE)
 
     formatter = logging.Formatter("[%(asctime)s] %(levelname)s "
                                   "[%(module)s - %(lineno)s:%(funcName)s] "
@@ -384,7 +391,7 @@ class PidFile(object):
 def webhook_monitor(proc_queue, webhooks):
     queues = {}
     for url, data in webhooks.items():
-        if isinstance(data, str) or isinstance(data, unicode):
+        if isinstance(data, str):
             token = data
             secret = None
         else:
@@ -405,8 +412,8 @@ def webhook_monitor(proc_queue, webhooks):
 
 class WebhookThreadPool(object):
     def start(self):
-        # Seperate process to emit messages to webhooks
-        # which maintains one thread per webhook. Seperate
+        # Separate process to emit messages to webhooks
+        # which maintains one thread per webhook. Separate
         # process is required since on reload we need to stop
         # and start the thread pool. In Python Threads can't be stopped
         # so terminate the process and start again. Note: In transit
@@ -420,6 +427,7 @@ class WebhookThreadPool(object):
         # In transit messages are skipped, since webhooks monitor process
         # is terminated.
         self.proc.terminate()
+        self.proc.join()
         self.start()
 
     def send(self, event_type, event_ts, message):
