@@ -11,12 +11,11 @@
 #include <pthread.h>
 
 #include "glusterfs/globals.h"
-#include "glusterfs/glusterfs.h"
 #include "timer-wheel.h"
 
 glusterfs_ctx_t *global_ctx = NULL;
 glusterfs_ctx_t *
-glusterfs_ctx_new()
+glusterfs_ctx_new(void)
 {
     long namelen = 0;
     glusterfs_ctx_t *ctx = NULL;
@@ -47,13 +46,6 @@ glusterfs_ctx_new()
     ctx->cmd_args.vgtool = _gf_none;
 #endif
 
-    /* lock is never destroyed! */
-    if (LOCK_INIT(&ctx->lock)) {
-        free(ctx);
-        ctx = NULL;
-        goto out;
-    }
-
     GF_ATOMIC_INIT(ctx->stats.max_dict_pairs, 0);
     GF_ATOMIC_INIT(ctx->stats.total_pairs_used, 0);
     GF_ATOMIC_INIT(ctx->stats.total_dicts_used, 0);
@@ -63,17 +55,20 @@ glusterfs_ctx_new()
         namelen = _POSIX_HOST_NAME_MAX;
     ctx->hostname = MALLOC(namelen + 1);
     if (!ctx->hostname) {
-        free(ctx);
+        FREE(ctx);
         ctx = NULL;
         goto out;
     }
     if (gethostname(ctx->hostname, namelen + 1)) {
-        free(ctx->hostname);
-        free(ctx);
+        FREE(ctx->hostname);
+        FREE(ctx);
         ctx = NULL;
         goto out;
     }
 
+    LOCK_INIT(&ctx->lock);
+    LOCK_INIT(&ctx->volfile_lock);
+
     if (!global_ctx)
         global_ctx = ctx;
 out:
