@@ -10,7 +10,6 @@
 
 #include "glusterfs/iobuf.h"
 #include "glusterfs/statedump.h"
-#include <stdio.h>
 #include "glusterfs/libglusterfs-messages.h"
 
 /*
@@ -66,11 +65,6 @@ __iobuf_arena_init_iobufs(struct iobuf_arena *iobuf_arena)
     int offset = 0;
     int i = 0;
 
-    iobuf_arena->iobufs = GF_CALLOC(sizeof(*iobuf), iobuf_cnt,
-                                    gf_common_mt_iobuf);
-    if (!iobuf_arena->iobufs)
-        return;
-
     iobuf = iobuf_arena->iobufs;
     for (i = 0; i < iobuf_cnt; i++) {
         INIT_LIST_HEAD(&iobuf->list);
@@ -86,8 +80,6 @@ __iobuf_arena_init_iobufs(struct iobuf_arena *iobuf_arena)
         offset += iobuf_arena->page_size;
         iobuf++;
     }
-
-    return;
 }
 
 static void
@@ -97,12 +89,6 @@ __iobuf_arena_destroy_iobufs(struct iobuf_arena *iobuf_arena)
     struct iobuf *iobuf = NULL;
     int i = 0;
 
-    if (!iobuf_arena->iobufs) {
-        gf_msg_callingfn(THIS->name, GF_LOG_ERROR, 0, LG_MSG_IOBUFS_NOT_FOUND,
-                         "iobufs not found");
-        return;
-    }
-
     iobuf_cnt = iobuf_arena->page_count;
     iobuf = iobuf_arena->iobufs;
     for (i = 0; i < iobuf_cnt; i++) {
@@ -112,26 +98,16 @@ __iobuf_arena_destroy_iobufs(struct iobuf_arena *iobuf_arena)
         list_del_init(&iobuf->list);
         iobuf++;
     }
-
-    GF_FREE(iobuf_arena->iobufs);
-
-    return;
 }
 
 static void
-__iobuf_arena_destroy(struct iobuf_pool *iobuf_pool,
-                      struct iobuf_arena *iobuf_arena)
+__iobuf_arena_destroy(struct iobuf_arena *iobuf_arena)
 {
-    GF_VALIDATE_OR_GOTO("iobuf", iobuf_arena, out);
+    munmap(iobuf_arena->mem_base, iobuf_arena->arena_size);
 
     __iobuf_arena_destroy_iobufs(iobuf_arena);
 
-    if (iobuf_arena->mem_base && iobuf_arena->mem_base != MAP_FAILED)
-        munmap(iobuf_arena->mem_base, iobuf_arena->arena_size);
-
     GF_FREE(iobuf_arena);
-out:
-    return;
 }
 
 static struct iobuf_arena *
@@ -144,12 +120,13 @@ __iobuf_arena_alloc(struct iobuf_pool *iobuf_pool, size_t page_size,
 
     GF_VALIDATE_OR_GOTO("iobuf", iobuf_pool, out);
 
-    iobuf_arena = GF_CALLOC(sizeof(*iobuf_arena), 1, gf_common_mt_iobuf_arena);
+    iobuf_arena = GF_CALLOC(
+        1, sizeof(struct iobuf_arena) + sizeof(struct iobuf) * num_iobufs,
+        gf_common_mt_iobuf_arena);
     if (!iobuf_arena)
-        goto err;
+        goto out;
 
     INIT_LIST_HEAD(&iobuf_arena->list);
-    INIT_LIST_HEAD(&iobuf_arena->all_list);
     INIT_LIST_HEAD(&iobuf_arena->passive_list);
     INIT_LIST_HEAD(&iobuf_arena->active_list);
     iobuf_arena->iobuf_pool = iobuf_pool;
@@ -166,45 +143,32 @@ __iobuf_arena_alloc(struct iobuf_pool *iobuf_pool, size_t page_size,
                                  MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
     if (iobuf_arena->mem_base == MAP_FAILED) {
         gf_smsg(THIS->name, GF_LOG_WARNING, 0, LG_MSG_MAPPING_FAILED, NULL);
-        goto err;
+        GF_FREE(iobuf_arena);
+        goto out;
     }
 
-    list_add_tail(&iobuf_arena->all_list, &iobuf_pool->all_arenas);
-
     __iobuf_arena_init_iobufs(iobuf_arena);
-    if (!iobuf_arena->iobufs) {
-        gf_smsg(THIS->name, GF_LOG_ERROR, 0, LG_MSG_INIT_IOBUF_FAILED, NULL);
-        goto err;
-    }
 
     iobuf_pool->arena_cnt++;
 
     return iobuf_arena;
 
-err:
-    __iobuf_arena_destroy(iobuf_pool, iobuf_arena);
-
 out:
     return NULL;
 }
 
 static struct iobuf_arena *
-__iobuf_arena_unprune(struct iobuf_pool *iobuf_pool, const size_t page_size,
-                      const int index)
+__iobuf_arena_unprune(struct iobuf_pool *iobuf_pool, const int index)
 {
-    struct iobuf_arena *iobuf_arena = NULL;
     struct iobuf_arena *tmp = NULL;
 
-    GF_VALIDATE_OR_GOTO("iobuf", iobuf_pool, out);
-
     list_for_each_entry(tmp, &iobuf_pool->purge[index], list)
     {
         list_del_init(&tmp->list);
-        iobuf_arena = tmp;
-        break;
+        return tmp;
     }
-out:
-    return iobuf_arena;
+
+    return NULL;
 }
 
 static struct iobuf_arena *
@@ -213,7 +177,7 @@ __iobuf_pool_add_arena(struct iobuf_pool *iobuf_pool, const size_t page_size,
 {
     struct iobuf_arena *iobuf_arena = NULL;
 
-    iobuf_arena = __iobuf_arena_unprune(iobuf_pool, page_size, index);
+    iobuf_arena = __iobuf_arena_unprune(iobuf_pool, index);
 
     if (!iobuf_arena) {
         iobuf_arena = __iobuf_arena_alloc(iobuf_pool, page_size, num_pages);
@@ -247,14 +211,14 @@ iobuf_pool_destroy(struct iobuf_pool *iobuf_pool)
                 list_del_init(&iobuf_arena->list);
                 iobuf_pool->arena_cnt--;
 
-                __iobuf_arena_destroy(iobuf_pool, iobuf_arena);
+                __iobuf_arena_destroy(iobuf_arena);
             }
             list_for_each_entry_safe(iobuf_arena, tmp, &iobuf_pool->purge[i],
                                      list)
             {
                 list_del_init(&iobuf_arena->list);
                 iobuf_pool->arena_cnt--;
-                __iobuf_arena_destroy(iobuf_pool, iobuf_arena);
+                __iobuf_arena_destroy(iobuf_arena);
             }
             /* If there are no iobuf leaks, there should be no
              * arenas in the filled list. If at all there are any
@@ -266,7 +230,7 @@ iobuf_pool_destroy(struct iobuf_pool *iobuf_pool)
             {
                 list_del_init(&iobuf_arena->list);
                 iobuf_pool->arena_cnt--;
-                __iobuf_arena_destroy(iobuf_pool, iobuf_arena);
+                __iobuf_arena_destroy(iobuf_arena);
             }
             /* If there are no iobuf leaks, there shoould be
              * no standard allocated arenas, iobuf_put will free
@@ -322,7 +286,7 @@ iobuf_pool_new(void)
     iobuf_pool = GF_CALLOC(sizeof(*iobuf_pool), 1, gf_common_mt_iobuf_pool);
     if (!iobuf_pool)
         goto out;
-    INIT_LIST_HEAD(&iobuf_pool->all_arenas);
+
     pthread_mutex_init(&iobuf_pool->mutex, NULL);
     for (i = 0; i <= IOBUF_ARENA_MAX_INDEX; i++) {
         INIT_LIST_HEAD(&iobuf_pool->arenas[i]);
@@ -356,49 +320,23 @@ static void
 __iobuf_arena_prune(struct iobuf_pool *iobuf_pool,
                     struct iobuf_arena *iobuf_arena, const int index)
 {
+    list_del(&iobuf_arena->list);
+
     /* code flow comes here only if the arena is in purge list and we can
      * free the arena only if we have at least one arena in 'arenas' list
      * (ie, at least few iobufs free in arena), that way, there won't
-     * be spurious mmap/unmap of buffers
+     * be spurious mmap/unmap of buffers.
+     * If the list empty, add to the purge list and return.
      */
-    if (list_empty(&iobuf_pool->arenas[index]))
+    if (list_empty(&iobuf_pool->arenas[index])) {
+        list_add_tail(&iobuf_arena->list, &iobuf_pool->purge[index]);
         goto out;
+    }
 
     /* All cases matched, destroy */
-    list_del_init(&iobuf_arena->list);
-    list_del_init(&iobuf_arena->all_list);
     iobuf_pool->arena_cnt--;
 
-    __iobuf_arena_destroy(iobuf_pool, iobuf_arena);
-
-out:
-    return;
-}
-
-void
-iobuf_pool_prune(struct iobuf_pool *iobuf_pool)
-{
-    struct iobuf_arena *iobuf_arena = NULL;
-    struct iobuf_arena *tmp = NULL;
-    int i = 0;
-
-    GF_VALIDATE_OR_GOTO("iobuf", iobuf_pool, out);
-
-    pthread_mutex_lock(&iobuf_pool->mutex);
-    {
-        for (i = 0; i < IOBUF_ARENA_MAX_INDEX; i++) {
-            if (list_empty(&iobuf_pool->arenas[i])) {
-                continue;
-            }
-
-            list_for_each_entry_safe(iobuf_arena, tmp, &iobuf_pool->purge[i],
-                                     list)
-            {
-                __iobuf_arena_prune(iobuf_pool, iobuf_arena, i);
-            }
-        }
-    }
-    pthread_mutex_unlock(&iobuf_pool->mutex);
+    __iobuf_arena_destroy(iobuf_arena);
 
 out:
     return;
@@ -467,13 +405,19 @@ __iobuf_get(struct iobuf_pool *iobuf_pool, const size_t page_size,
     return iobuf;
 }
 
+static void
+__iobuf_free(struct iobuf *iobuf)
+{
+    LOCK_DESTROY(&iobuf->lock);
+    GF_FREE(iobuf);
+}
+
 static struct iobuf *
 iobuf_get_from_stdalloc(struct iobuf_pool *iobuf_pool, const size_t page_size)
 {
     struct iobuf *iobuf = NULL;
     struct iobuf_arena *iobuf_arena = NULL;
     struct iobuf_arena *trav = NULL;
-    int ret = -1;
 
     /* The first arena in the 'MAX-INDEX' will always be used for misc */
     list_for_each_entry(trav, &iobuf_pool->arenas[IOBUF_ARENA_MAX_INDEX], list)
@@ -482,65 +426,39 @@ iobuf_get_from_stdalloc(struct iobuf_pool *iobuf_pool, const size_t page_size)
         break;
     }
 
-    iobuf = GF_CALLOC(1, sizeof(*iobuf), gf_common_mt_iobuf);
-    if (!iobuf)
-        goto out;
+    iobuf = GF_MALLOC(sizeof(*iobuf) + ((page_size + GF_IOBUF_ALIGN_SIZE) - 1),
+                      gf_common_mt_iobuf);
+    if (caa_unlikely(!iobuf))
+        return NULL;
 
-    /* 4096 is the alignment */
-    iobuf->free_ptr = GF_CALLOC(1, ((page_size + GF_IOBUF_ALIGN_SIZE) - 1),
-                                gf_common_mt_char);
-    if (!iobuf->free_ptr)
-        goto out;
-
-    iobuf->ptr = GF_ALIGN_BUF(iobuf->free_ptr, GF_IOBUF_ALIGN_SIZE);
+    INIT_LIST_HEAD(&iobuf->list);
     iobuf->iobuf_arena = iobuf_arena;
-    iobuf->page_size = page_size;
     LOCK_INIT(&iobuf->lock);
-
     /* Hold a ref because you are allocating and using it */
     GF_ATOMIC_INIT(iobuf->ref, 1);
 
-    ret = 0;
-out:
-    if (ret && iobuf) {
-        GF_FREE(iobuf->free_ptr);
-        GF_FREE(iobuf);
-        iobuf = NULL;
-    }
+    iobuf->ptr = GF_ALIGN_BUF(iobuf->allocated_buffer, GF_IOBUF_ALIGN_SIZE);
+    iobuf->page_size = page_size;
 
     return iobuf;
 }
 
-static struct iobuf *
+struct iobuf *
 iobuf_get_from_small(const size_t page_size)
 {
     struct iobuf *iobuf = NULL;
-    int ret = -1;
 
-    iobuf = GF_MALLOC(sizeof(*iobuf), gf_common_mt_iobuf);
-    if (!iobuf)
-        goto out;
+    iobuf = GF_MALLOC(sizeof(*iobuf) + page_size, gf_common_mt_iobuf);
+    if (caa_unlikely(!iobuf))
+        return NULL;
 
-    iobuf->free_ptr = GF_MALLOC(page_size, gf_common_mt_iobuf_pool);
-    if (!iobuf->free_ptr)
-        goto out;
-
-    iobuf->ptr = iobuf->free_ptr;
-    iobuf->page_size = page_size;
     INIT_LIST_HEAD(&iobuf->list);
     iobuf->iobuf_arena = NULL;
     LOCK_INIT(&iobuf->lock);
     /* Hold a ref because you are allocating and using it */
     GF_ATOMIC_INIT(iobuf->ref, 1);
-
-    ret = 0;
-out:
-    if (ret && iobuf) {
-        GF_FREE(iobuf->free_ptr);
-        GF_FREE(iobuf);
-        iobuf = NULL;
-    }
-
+    iobuf->ptr = iobuf->allocated_buffer;
+    iobuf->page_size = page_size;
     return iobuf;
 }
 
@@ -621,17 +539,6 @@ iobuf_get_page_aligned(struct iobuf_pool *iobuf_pool, size_t page_size,
     iobuf = iobuf_get2(iobuf_pool, req_size);
     if (!iobuf)
         return NULL;
-    /* If std allocation was used, then free_ptr will be non-NULL. In this
-     * case, we do not want to modify the original free_ptr.
-     * On the other hand, if the buf was gotten through the available
-     * arenas, then we use iobuf->free_ptr to store the original
-     * pointer to the offset into the mmap'd block of memory and in turn
-     * reuse iobuf->ptr to hold the page-aligned address. And finally, in
-     * iobuf_put(), we copy iobuf->free_ptr into iobuf->ptr - back to where
-     * it was originally when __iobuf_get() returned this iobuf.
-     */
-    if (!iobuf->free_ptr)
-        iobuf->free_ptr = iobuf->ptr;
     iobuf->ptr = GF_ALIGN_BUF(iobuf->ptr, align_size);
 
     return iobuf;
@@ -658,8 +565,6 @@ __iobuf_put(struct iobuf *iobuf, struct iobuf_arena *iobuf_arena)
     struct iobuf_pool *iobuf_pool = NULL;
     int index = 0;
 
-    iobuf_pool = iobuf_arena->iobuf_pool;
-
     index = gf_iobuf_get_arena_index(iobuf_arena->page_size);
     if (index == -1) {
         gf_msg_debug("iobuf", 0,
@@ -668,12 +573,12 @@ __iobuf_put(struct iobuf *iobuf, struct iobuf_arena *iobuf_arena)
                      iobuf);
 
         /* free up properly without bothering about lists and all */
-        LOCK_DESTROY(&iobuf->lock);
-        GF_FREE(iobuf->free_ptr);
-        GF_FREE(iobuf);
+        __iobuf_free(iobuf);
         return;
     }
 
+    iobuf_pool = iobuf_arena->iobuf_pool;
+
     if (iobuf_arena->passive_cnt == 0) {
         list_del(&iobuf_arena->list);
         list_add_tail(&iobuf_arena->list, &iobuf_pool->arenas[index]);
@@ -682,36 +587,25 @@ __iobuf_put(struct iobuf *iobuf, struct iobuf_arena *iobuf_arena)
     list_del_init(&iobuf->list);
     iobuf_arena->active_cnt--;
 
-    if (iobuf->free_ptr) {
-        iobuf->ptr = iobuf->free_ptr;
-        iobuf->free_ptr = NULL;
-    }
-
     list_add(&iobuf->list, &iobuf_arena->passive_list);
     iobuf_arena->passive_cnt++;
 
     if (iobuf_arena->active_cnt == 0) {
-        list_del(&iobuf_arena->list);
-        list_add_tail(&iobuf_arena->list, &iobuf_pool->purge[index]);
-        GF_VALIDATE_OR_GOTO("iobuf", iobuf_pool, out);
         __iobuf_arena_prune(iobuf_pool, iobuf_arena, index);
     }
-out:
-    return;
 }
 
-void
+static void
 iobuf_put(struct iobuf *iobuf)
 {
     struct iobuf_arena *iobuf_arena = NULL;
     struct iobuf_pool *iobuf_pool = NULL;
 
-    GF_VALIDATE_OR_GOTO("iobuf", iobuf, out);
+    GF_ASSERT(iobuf);
 
     iobuf_arena = iobuf->iobuf_arena;
     if (!iobuf_arena) {
-        GF_FREE(iobuf->free_ptr);
-        GF_FREE(iobuf);
+        __iobuf_free(iobuf);
         return;
     }
 
@@ -727,9 +621,6 @@ iobuf_put(struct iobuf *iobuf)
         __iobuf_put(iobuf, iobuf_arena);
     }
     pthread_mutex_unlock(&iobuf_pool->mutex);
-
-out:
-    return;
 }
 
 void
@@ -759,7 +650,7 @@ out:
 }
 
 struct iobref *
-iobref_new()
+iobref_new(void)
 {
     struct iobref *iobref = NULL;
 
@@ -793,27 +684,25 @@ out:
     return iobref;
 }
 
-void
+static void
 iobref_destroy(struct iobref *iobref)
 {
     int i = 0;
     struct iobuf *iobuf = NULL;
 
-    GF_VALIDATE_OR_GOTO("iobuf", iobref, out);
+    GF_ASSERT(iobref);
 
-    for (i = 0; i < iobref->allocated; i++) {
+    for (i = 0; i < iobref->used; i++) {
         iobuf = iobref->iobrefs[i];
-
+        GF_ASSERT(iobuf);
         iobref->iobrefs[i] = NULL;
-        if (iobuf)
-            iobuf_unref(iobuf);
+        iobuf_unref(iobuf);
     }
 
+    LOCK_DESTROY(&iobref->lock);
+
     GF_FREE(iobref->iobrefs);
     GF_FREE(iobref);
-
-out:
-    return;
 }
 
 void
@@ -838,13 +727,9 @@ iobref_clear(struct iobref *iobref)
 
     GF_VALIDATE_OR_GOTO("iobuf", iobref, out);
 
-    for (; i < iobref->allocated; i++) {
-        if (iobref->iobrefs[i] != NULL) {
-            iobuf_unref(iobref->iobrefs[i]);
-        } else {
-            /** iobuf's are attached serially */
-            break;
-        }
+    for (; i < iobref->used; i++) {
+        GF_ASSERT(iobref->iobrefs[i]);
+        iobuf_unref(iobref->iobrefs[i]);
     }
 
     iobref_unref(iobref);
@@ -870,35 +755,21 @@ __iobref_grow(struct iobref *iobref)
     }
 }
 
-int
+static int
 __iobref_add(struct iobref *iobref, struct iobuf *iobuf)
 {
-    int i = 0;
-    int ret = -ENOMEM;
-
-    GF_VALIDATE_OR_GOTO("iobuf", iobref, out);
-    GF_VALIDATE_OR_GOTO("iobuf", iobuf, out);
+    GF_ASSERT(iobref);
+    GF_ASSERT(iobuf);
 
     if (iobref->used == iobref->allocated) {
         __iobref_grow(iobref);
 
-        if (iobref->used == iobref->allocated) {
-            ret = -ENOMEM;
-            goto out;
-        }
+        if (iobref->used == iobref->allocated)
+            return -ENOMEM;
     }
 
-    for (i = 0; i < iobref->allocated; i++) {
-        if (iobref->iobrefs[i] == NULL) {
-            iobref->iobrefs[i] = iobuf_ref(iobuf);
-            iobref->used++;
-            ret = 0;
-            break;
-        }
-    }
-
-out:
-    return ret;
+    iobref->iobrefs[iobref->used++] = iobuf_ref(iobuf);
+    return 0;
 }
 
 int
@@ -931,14 +802,11 @@ iobref_merge(struct iobref *to, struct iobref *from)
 
     LOCK(&from->lock);
     {
-        for (i = 0; i < from->allocated; i++) {
+        for (i = 0; i < from->used; i++) {
             iobuf = from->iobrefs[i];
-
-            if (!iobuf)
-                break;
+            GF_ASSERT(iobuf);
 
             ret = iobref_add(to, iobuf);
-
             if (ret < 0)
                 break;
         }
@@ -971,9 +839,9 @@ iobref_size(struct iobref *iobref)
 
     LOCK(&iobref->lock);
     {
-        for (i = 0; i < iobref->allocated; i++) {
-            if (iobref->iobrefs[i])
-                size += iobuf_size(iobref->iobrefs[i]);
+        for (i = 0; i < iobref->used; i++) {
+            GF_ASSERT(iobref->iobrefs[i]);
+            size += iobuf_size(iobref->iobrefs[i]);
         }
     }
     UNLOCK(&iobref->lock);
