@@ -192,6 +192,11 @@ xlator_volopt_dynload(char *xlator_type, void **dl_handle,
 
     GF_VALIDATE_OR_GOTO("xlator", xlator_type, out);
 
+#ifndef BUILD_GNFS
+    if (strcmp(xlator_type, "nfs/server") == 0)
+        return -1;
+#endif /* BUILD_GNFS */
+
     /* socket.so doesn't fall under the default xlator directory, hence we
      * need this check */
     if (!strstr(xlator_type, "rpc-transport"))
@@ -350,8 +355,6 @@ xlator_dynload_apis(xlator_t *xl)
         list_add_tail(&vol_opt->list, &xl->volume_options);
     }
 
-    xl->id = xlapi->xlator_id;
-    xl->flags = xlapi->flags;
     xl->identifier = xlapi->identifier;
     xl->category = xlapi->category;
 
@@ -718,9 +721,6 @@ xlator_notify(xlator_t *xl, int event, void *data, ...)
 int
 xlator_mem_acct_init(xlator_t *xl, int num_types)
 {
-    int i = 0;
-    int ret = 0;
-
     if (!xl)
         return -1;
 
@@ -730,8 +730,8 @@ xlator_mem_acct_init(xlator_t *xl, int num_types)
     if (!xl->ctx->mem_acct_enable)
         return 0;
 
-    xl->mem_acct = MALLOC(sizeof(struct mem_acct) +
-                          sizeof(struct mem_acct_rec) * num_types);
+    xl->mem_acct = CALLOC(
+        1, sizeof(struct mem_acct) + sizeof(struct mem_acct_rec) * num_types);
 
     if (!xl->mem_acct) {
         return -1;
@@ -740,33 +740,31 @@ xlator_mem_acct_init(xlator_t *xl, int num_types)
     xl->mem_acct->num_types = num_types;
     GF_ATOMIC_INIT(xl->mem_acct->refcnt, 1);
 
-    for (i = 0; i < num_types; i++) {
-        memset(&xl->mem_acct->rec[i], 0, sizeof(struct mem_acct_rec));
 #ifdef DEBUG
-        INIT_LIST_HEAD(&(xl->mem_acct->rec[i].obj_list));
-        ret = LOCK_INIT(&(xl->mem_acct->rec[i].lock));
-        if (ret) {
+    struct mem_acct_rec *rec = NULL;
+    int i;
+    for (i = 0; i < num_types; i++) {
+        rec = &xl->mem_acct->rec[i];
+        if (LOCK_INIT(&rec->lock) != 0) {
             fprintf(stderr, "Unable to lock..errno : %d", errno);
         }
-#endif
+        INIT_LIST_HEAD(&rec->obj_list);
     }
+#endif /* DEBUG */
 
     return 0;
 }
 
 void
-xlator_mem_acct_unref(struct mem_acct *mem_acct)
+xlator_mem_acct_destroy(struct mem_acct *mem_acct)
 {
-    uint32_t i;
-
-    if (GF_ATOMIC_DEC(mem_acct->refcnt) == 0) {
 #ifdef DEBUG
-        for (i = 0; i < mem_acct->num_types; i++) {
-            LOCK_DESTROY(&(mem_acct->rec[i].lock));
-        }
-#endif
-        FREE(mem_acct);
+    uint32_t i;
+    for (i = 0; i < mem_acct->num_types; i++) {
+        LOCK_DESTROY(&(mem_acct->rec[i].lock));
     }
+#endif
+    FREE(mem_acct);
 }
 
 void
@@ -797,7 +795,7 @@ xlator_list_destroy(xlator_list_t *list)
     return 0;
 }
 
-int
+static int
 xlator_memrec_free(xlator_t *xl)
 {
     struct mem_acct *mem_acct = NULL;
@@ -807,8 +805,8 @@ xlator_memrec_free(xlator_t *xl)
     }
     mem_acct = xl->mem_acct;
 
-    if (mem_acct) {
-        xlator_mem_acct_unref(mem_acct);
+    if (mem_acct && (GF_ATOMIC_DEC(mem_acct->refcnt) == 0)) {
+        xlator_mem_acct_destroy(mem_acct);
         xl->mem_acct = NULL;
     }
 
@@ -1074,34 +1072,30 @@ void
 loc_gfid(loc_t *loc, uuid_t gfid)
 {
     if (!gfid)
-        goto out;
-    gf_uuid_clear(gfid);
-
-    if (!loc)
-        goto out;
+        return;
+    else if (!loc)
+        gf_uuid_clear(gfid);
     else if (!gf_uuid_is_null(loc->gfid))
         gf_uuid_copy(gfid, loc->gfid);
     else if (loc->inode && (!gf_uuid_is_null(loc->inode->gfid)))
         gf_uuid_copy(gfid, loc->inode->gfid);
-out:
-    return;
+    else
+        gf_uuid_clear(gfid);
 }
 
 void
 loc_pargfid(loc_t *loc, uuid_t gfid)
 {
     if (!gfid)
-        goto out;
-    gf_uuid_clear(gfid);
-
-    if (!loc)
-        goto out;
+        return;
+    else if (!loc)
+        gf_uuid_clear(gfid);
     else if (!gf_uuid_is_null(loc->pargfid))
         gf_uuid_copy(gfid, loc->pargfid);
     else if (loc->parent && (!gf_uuid_is_null(loc->parent->gfid)))
         gf_uuid_copy(gfid, loc->parent->gfid);
-out:
-    return;
+    else
+        gf_uuid_clear(gfid);
 }
 
 char *
@@ -1244,46 +1238,6 @@ loc_is_root(loc_t *loc)
     return _gf_false;
 }
 
-int32_t
-loc_build_child(loc_t *child, loc_t *parent, char *name)
-{
-    int32_t ret = -1;
-
-    GF_VALIDATE_OR_GOTO("xlator", child, out);
-    GF_VALIDATE_OR_GOTO("xlator", parent, out);
-    GF_VALIDATE_OR_GOTO("xlator", name, out);
-
-    loc_gfid(parent, child->pargfid);
-
-    if (strcmp(parent->path, "/") == 0)
-        ret = gf_asprintf((char **)&child->path, "/%s", name);
-    else
-        ret = gf_asprintf((char **)&child->path, "%s/%s", parent->path, name);
-
-    if (ret < 0 || !child->path) {
-        ret = -1;
-        goto out;
-    }
-
-    child->name = strrchr(child->path, '/') + 1;
-
-    child->parent = inode_ref(parent->inode);
-    child->inode = inode_new(parent->inode->table);
-
-    if (!child->inode) {
-        ret = -1;
-        goto out;
-    }
-
-    ret = 0;
-
-out:
-    if ((ret < 0) && child)
-        loc_wipe(child);
-
-    return ret;
-}
-
 gf_boolean_t
 loc_is_nameless(loc_t *loc)
 {
@@ -1333,7 +1287,10 @@ is_gf_log_command(xlator_t *this, const char *name, char *value, size_t size)
     int ret = -1;
     int log_level = -1;
     gf_boolean_t syslog_flag = 0;
-    glusterfs_ctx_t *ctx = NULL;
+    glusterfs_ctx_t *ctx = this->ctx;
+
+    if (!ctx)
+        goto out;
 
     if (!strcmp("trusted.glusterfs.syslog", name)) {
         ret = gf_bin_to_string(key, sizeof(key), value, size);
@@ -1346,9 +1303,9 @@ is_gf_log_command(xlator_t *this, const char *name, char *value, size_t size)
             goto out;
         }
         if (syslog_flag)
-            gf_log_enable_syslog();
+            gf_log_enable_syslog(ctx);
         else
-            gf_log_disable_syslog();
+            gf_log_disable_syslog(ctx);
 
         goto out;
     }
@@ -1372,7 +1329,7 @@ is_gf_log_command(xlator_t *this, const char *name, char *value, size_t size)
         gf_smsg("glusterfs", gf_log_get_loglevel(), 0, LG_MSG_SET_LOG_LEVEL,
                 "new-value=%d", log_level, "old-value=%d",
                 gf_log_get_loglevel(), NULL);
-        gf_log_set_loglevel(this->ctx, log_level);
+        gf_log_set_loglevel(ctx, log_level);
         ret = 0;
         goto out;
     }
@@ -1387,9 +1344,6 @@ is_gf_log_command(xlator_t *this, const char *name, char *value, size_t size)
         goto out;
     }
 
-    ctx = this->ctx;
-    if (!ctx)
-        goto out;
     if (!ctx->active)
         goto out;
     trav = ctx->active->top;
@@ -1412,28 +1366,24 @@ out:
 int
 glusterd_check_log_level(const char *value)
 {
-    int log_level = -1;
-
     if (!strcasecmp(value, "CRITICAL")) {
-        log_level = GF_LOG_CRITICAL;
+        return GF_LOG_CRITICAL;
     } else if (!strcasecmp(value, "ERROR")) {
-        log_level = GF_LOG_ERROR;
+        return GF_LOG_ERROR;
     } else if (!strcasecmp(value, "WARNING")) {
-        log_level = GF_LOG_WARNING;
+        return GF_LOG_WARNING;
     } else if (!strcasecmp(value, "INFO")) {
-        log_level = GF_LOG_INFO;
+        return GF_LOG_INFO;
     } else if (!strcasecmp(value, "DEBUG")) {
-        log_level = GF_LOG_DEBUG;
+        return GF_LOG_DEBUG;
     } else if (!strcasecmp(value, "TRACE")) {
-        log_level = GF_LOG_TRACE;
+        return GF_LOG_TRACE;
     } else if (!strcasecmp(value, "NONE")) {
-        log_level = GF_LOG_NONE;
+        return GF_LOG_NONE;
     }
 
-    if (log_level == -1)
-        gf_smsg(THIS->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_INIT, NULL);
-
-    return log_level;
+    gf_smsg(THIS->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_INIT, NULL);
+    return -1;
 }
 
 int
