@@ -8,7 +8,7 @@
    cases as published by the Free Software Foundation.
 */
 
-#include "gf-event.h"
+#include <glusterfs/gf-event.h>
 #include "rpcsvc.h"
 
 void
@@ -18,6 +18,5 @@ rpcsvc_autoscale_threads(glusterfs_ctx_t *ctx, rpcsvc_t *rpc, int incr)
     int thread_count = pool->eventthreadcount;
 
     pool->auto_thread_count += incr;
-    (void)event_reconfigure_threads(pool, thread_count + incr);
-    rpcsvc_ownthread_reconf(rpc, pool->eventthreadcount);
+    (void)gf_event_reconfigure_threads(pool, thread_count + incr);
 }
