@@ -17,14 +17,10 @@
 
 #include <urcu/uatomic.h>
 
-#include <glusterfs/glusterfs.h>
-#include "afr.h"
+#include "libxlator.h"  // for gf_get_max_stime()
 #include <glusterfs/dict.h>
 #include <glusterfs/logging.h>
 #include <glusterfs/list.h>
-#include <glusterfs/byte-order.h>
-#include <glusterfs/defaults.h>
-#include <glusterfs/common-utils.h>
 #include <glusterfs/compat-errno.h>
 #include <glusterfs/compat.h>
 #include <glusterfs/quota-common-utils.h>
@@ -42,10 +38,9 @@
  * */
 
 int
-afr_handle_quota_size(call_frame_t *frame, xlator_t *this)
+afr_handle_quota_size(afr_local_t *local, xlator_t *this)
 {
     unsigned char *readable = NULL;
-    afr_local_t *local = NULL;
     afr_private_t *priv = NULL;
     struct afr_reply *replies = NULL;
     int i = 0;
@@ -59,7 +54,6 @@ afr_handle_quota_size(call_frame_t *frame, xlator_t *this)
     int readable_cnt = 0;
     int read_subvol = -1;
 
-    local = frame->local;
     priv = this->private;
     replies = local->replies;
 
@@ -115,7 +109,7 @@ afr_handle_quota_size(call_frame_t *frame, xlator_t *this)
 
 /* {{{ access */
 
-int
+static int
 afr_access_cbk(call_frame_t *frame, void *cookie, xlator_t *this, int op_ret,
                int op_errno, dict_t *xdata)
 {
@@ -136,7 +130,7 @@ afr_access_cbk(call_frame_t *frame, void *cookie, xlator_t *this, int op_ret,
     return 0;
 }
 
-int
+static int
 afr_access_wind(call_frame_t *frame, xlator_t *this, int subvol)
 {
     afr_private_t *priv = NULL;
@@ -188,7 +182,7 @@ out:
 
 /* {{{ stat */
 
-int
+static int
 afr_stat_cbk(call_frame_t *frame, void *cookie, xlator_t *this, int32_t op_ret,
              int32_t op_errno, struct iatt *buf, dict_t *xdata)
 {
@@ -209,7 +203,7 @@ afr_stat_cbk(call_frame_t *frame, void *cookie, xlator_t *this, int32_t op_ret,
     return 0;
 }
 
-int
+static int
 afr_stat_wind(call_frame_t *frame, xlator_t *this, int subvol)
 {
     afr_private_t *priv = NULL;
@@ -257,7 +251,7 @@ out:
 
 /* {{{ fstat */
 
-int
+static int
 afr_fstat_cbk(call_frame_t *frame, void *cookie, xlator_t *this, int32_t op_ret,
               int32_t op_errno, struct iatt *buf, dict_t *xdata)
 {
@@ -278,7 +272,7 @@ afr_fstat_cbk(call_frame_t *frame, void *cookie, xlator_t *this, int32_t op_ret,
     return 0;
 }
 
-int
+static int
 afr_fstat_wind(call_frame_t *frame, xlator_t *this, int subvol)
 {
     afr_private_t *priv = NULL;
@@ -329,7 +323,7 @@ out:
 
 /* {{{ readlink */
 
-int
+static int
 afr_readlink_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
                  int32_t op_ret, int32_t op_errno, const char *buf,
                  struct iatt *sbuf, dict_t *xdata)
@@ -350,7 +344,7 @@ afr_readlink_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
     return 0;
 }
 
-int
+static int
 afr_readlink_wind(call_frame_t *frame, xlator_t *this, int subvol)
 {
     afr_local_t *local = NULL;
@@ -408,7 +402,7 @@ struct _xattr_key {
     struct list_head list;
 };
 
-int
+static int
 __gather_xattr_keys(dict_t *dict, char *key, data_t *value, void *data)
 {
     struct list_head *list = data;
@@ -427,7 +421,7 @@ __gather_xattr_keys(dict_t *dict, char *key, data_t *value, void *data)
     return 0;
 }
 
-void
+static void
 afr_filter_xattrs(dict_t *dict)
 {
     struct list_head keys = {
@@ -459,7 +453,7 @@ afr_getxattr_ignorable_errnos(int32_t op_errno)
 
     return _gf_false;
 }
-int
+static int
 afr_getxattr_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
                  int32_t op_ret, int32_t op_errno, dict_t *dict, dict_t *xdata)
 {
@@ -483,7 +477,7 @@ afr_getxattr_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
     return 0;
 }
 
-int
+static int
 afr_getxattr_wind(call_frame_t *frame, xlator_t *this, int subvol)
 {
     afr_local_t *local = NULL;
@@ -505,7 +499,7 @@ afr_getxattr_wind(call_frame_t *frame, xlator_t *this, int subvol)
     return 0;
 }
 
-int32_t
+static int32_t
 afr_getxattr_unwind(call_frame_t *frame, int op_ret, int op_errno, dict_t *dict,
                     dict_t *xdata)
 
@@ -514,7 +508,7 @@ afr_getxattr_unwind(call_frame_t *frame, int op_ret, int op_errno, dict_t *dict,
     return 0;
 }
 
-int32_t
+static int32_t
 afr_fgetxattr_clrlk_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
                         int32_t op_ret, int32_t op_errno, dict_t *dict,
                         dict_t *xdata)
@@ -601,7 +595,7 @@ unlock:
     return ret;
 }
 
-int32_t
+static int32_t
 afr_getxattr_clrlk_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
                        int32_t op_ret, int32_t op_errno, dict_t *dict,
                        dict_t *xdata)
@@ -693,7 +687,7 @@ unlock:
 /**
  * node-uuid cbk uses next child querying mechanism
  */
-int32_t
+static int32_t
 afr_getxattr_node_uuid_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
                            int32_t op_ret, int32_t op_errno, dict_t *dict,
                            dict_t *xdata)
@@ -741,7 +735,7 @@ unwind:
 /**
  * list-node-uuids cbk returns the list of node_uuids for the subvolume.
  */
-int32_t
+static int32_t
 afr_getxattr_list_node_uuids_cbk(call_frame_t *frame, void *cookie,
                                  xlator_t *this, int32_t op_ret,
                                  int32_t op_errno, dict_t *dict, dict_t *xdata)
@@ -839,7 +833,7 @@ unlock:
     return ret;
 }
 
-int32_t
+static int32_t
 afr_getxattr_quota_size_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
                             int32_t op_ret, int32_t op_errno, dict_t *dict,
                             dict_t *xdata)
@@ -857,7 +851,7 @@ afr_getxattr_quota_size_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
     call_count = afr_frame_return(frame);
     if (call_count == 0) {
         local->inode = inode_ref(local->loc.inode);
-        read_subvol = afr_handle_quota_size(frame, this);
+        read_subvol = afr_handle_quota_size(local, this);
         if (read_subvol != -1) {
             op_ret = local->replies[read_subvol].op_ret;
             op_errno = local->replies[read_subvol].op_errno;
@@ -1047,7 +1041,7 @@ afr_fgetxattr_lockinfo_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
     return 0;
 }
 
-int32_t
+static int32_t
 afr_fgetxattr_pathinfo_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
                            int32_t op_ret, int32_t op_errno, dict_t *dict,
                            dict_t *xdata)
@@ -1171,7 +1165,7 @@ out:
     return ret;
 }
 
-int32_t
+static int32_t
 afr_getxattr_pathinfo_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
                           int32_t op_ret, int32_t op_errno, dict_t *dict,
                           dict_t *xdata)
@@ -1306,7 +1300,7 @@ afr_aggregate_stime_xattr(dict_t *this, char *key, data_t *value, void *data)
     return ret;
 }
 
-int32_t
+static int32_t
 afr_common_getxattr_stime_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
                               int32_t op_ret, int32_t op_errno, dict_t *dict,
                               dict_t *xdata)
@@ -1397,16 +1391,13 @@ out:
 }
 
 static void
-afr_getxattr_all_subvols(xlator_t *this, call_frame_t *frame, const char *name,
-                         loc_t *loc, fop_getxattr_cbk_t cbk)
+afr_getxattr_all_subvols(afr_private_t *priv, call_frame_t *frame,
+                         const char *name, loc_t *loc, fop_getxattr_cbk_t cbk)
 {
-    afr_private_t *priv = NULL;
     afr_local_t *local = NULL;
     int i = 0;
     int call_count = 0;
 
-    priv = this->private;
-
     local = frame->local;
     // local->call_count set in afr_local_init
     call_count = local->call_count;
@@ -1430,7 +1421,7 @@ afr_getxattr_all_subvols(xlator_t *this, call_frame_t *frame, const char *name,
     return;
 }
 
-int
+static int
 afr_marker_populate_args(call_frame_t *frame, int type, int *gauge,
                          xlator_t **subvols)
 {
@@ -1559,7 +1550,7 @@ afr_getxattr(call_frame_t *frame, xlator_t *this, loc_t *loc, const char *name,
      * Special xattrs which need responses from all subvols
      */
     if (afr_is_special_xattr(name, &cbk, 0)) {
-        afr_getxattr_all_subvols(this, frame, name, loc, cbk);
+        afr_getxattr_all_subvols(priv, frame, name, loc, cbk);
         return 0;
     }
 
@@ -1586,7 +1577,7 @@ out:
 
 /* {{{ fgetxattr */
 
-int32_t
+static int32_t
 afr_fgetxattr_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
                   int32_t op_ret, int32_t op_errno, dict_t *dict, dict_t *xdata)
 {
@@ -1610,7 +1601,7 @@ afr_fgetxattr_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
     return 0;
 }
 
-int
+static int
 afr_fgetxattr_wind(call_frame_t *frame, xlator_t *this, int subvol)
 {
     afr_local_t *local = NULL;
@@ -1633,16 +1624,13 @@ afr_fgetxattr_wind(call_frame_t *frame, xlator_t *this, int subvol)
 }
 
 static void
-afr_fgetxattr_all_subvols(xlator_t *this, call_frame_t *frame,
+afr_fgetxattr_all_subvols(afr_private_t *priv, call_frame_t *frame,
                           fop_fgetxattr_cbk_t cbk)
 {
-    afr_private_t *priv = NULL;
     afr_local_t *local = NULL;
     int i = 0;
     int call_count = 0;
 
-    priv = this->private;
-
     local = frame->local;
     // local->call_count set in afr_local_init
     call_count = local->call_count;
@@ -1694,7 +1682,7 @@ afr_fgetxattr(call_frame_t *frame, xlator_t *this, fd_t *fd, const char *name,
      * collect information from all children.
      */
     if (afr_is_special_xattr(name, &cbk, 1)) {
-        afr_fgetxattr_all_subvols(this, frame, cbk);
+        afr_fgetxattr_all_subvols(this->private, frame, cbk);
         return 0;
     }
 
@@ -1714,7 +1702,7 @@ out:
 
 /* {{{ readv */
 
-int
+static int
 afr_readv_cbk(call_frame_t *frame, void *cookie, xlator_t *this, int32_t op_ret,
               int32_t op_errno, struct iovec *vector, int32_t count,
               struct iatt *buf, struct iobref *iobref, dict_t *xdata)
@@ -1736,7 +1724,7 @@ afr_readv_cbk(call_frame_t *frame, void *cookie, xlator_t *this, int32_t op_ret,
     return 0;
 }
 
-int
+static int
 afr_readv_wind(call_frame_t *frame, xlator_t *this, int subvol)
 {
     afr_local_t *local = NULL;
@@ -1793,7 +1781,7 @@ out:
 
 /* {{{ seek */
 
-int
+static int
 afr_seek_cbk(call_frame_t *frame, void *cookie, xlator_t *this, int32_t op_ret,
              int32_t op_errno, off_t offset, dict_t *xdata)
 {
@@ -1813,7 +1801,7 @@ afr_seek_cbk(call_frame_t *frame, void *cookie, xlator_t *this, int32_t op_ret,
     return 0;
 }
 
-int
+static int
 afr_seek_wind(call_frame_t *frame, xlator_t *this, int subvol)
 {
     afr_local_t *local = NULL;
