@@ -8,7 +8,6 @@
   cases as published by the Free Software Foundation.
 */
 
-#include "afr.h"
 #include "afr-transaction.h"
 #include "afr-messages.h"
 
@@ -45,7 +44,7 @@ afr_read_txn_wind(call_frame_t *frame, xlator_t *this, int subvol)
     local->readfn(frame, this, subvol);
 }
 
-int
+static int
 afr_read_txn_next_subvol(call_frame_t *frame, xlator_t *this)
 {
     afr_local_t *local = NULL;
@@ -231,7 +230,7 @@ out:
     return ret;
 }
 
-void
+static void
 afr_ta_read_txn_synctask(call_frame_t *frame, xlator_t *this)
 {
     call_frame_t *ta_frame = NULL;
@@ -264,7 +263,7 @@ out:
     afr_read_txn_wind(frame, this, -1);
 }
 
-int
+static int
 afr_read_txn_refresh_done(call_frame_t *frame, xlator_t *this, int err)
 {
     afr_private_t *priv = NULL;
@@ -341,22 +340,20 @@ afr_read_txn_continue(call_frame_t *frame, xlator_t *this, int subvol)
    the same frame
 */
 
-void
-afr_read_txn_wipe(call_frame_t *frame, xlator_t *this)
+static void
+afr_read_txn_wipe(call_frame_t *frame, unsigned int child_count)
 {
     afr_local_t *local = NULL;
-    afr_private_t *priv = NULL;
     int i = 0;
 
     local = frame->local;
-    priv = this->private;
 
     local->readfn = NULL;
 
     if (local->inode)
         inode_unref(local->inode);
 
-    for (i = 0; i < priv->child_count; i++) {
+    for (i = 0; i < child_count; i++) {
         local->read_attempted[i] = 0;
         local->readable[i] = 0;
     }
@@ -406,14 +403,14 @@ afr_read_txn(call_frame_t *frame, xlator_t *this, inode_t *inode,
     data = alloca0(priv->child_count);
     metadata = alloca0(priv->child_count);
 
-    afr_read_txn_wipe(frame, this);
+    afr_read_txn_wipe(frame, priv->child_count);
 
     local->readfn = readfn;
     local->inode = inode_ref(inode);
     local->is_read_txn = _gf_true;
     local->transaction.type = type;
 
-    if (priv->quorum_count && !afr_has_quorum(local->child_up, this, NULL)) {
+    if (priv->quorum_count && !afr_has_quorum(local->child_up, priv, NULL)) {
         local->op_ret = -1;
         local->op_errno = afr_quorum_errno(priv);
         goto read;
