@@ -11,7 +11,7 @@
 #ifndef __DHT_MEM_TYPES_H__
 #define __DHT_MEM_TYPES_H__
 
-#include "mem-types.h"
+#include <glusterfs/mem-types.h>
 
 enum gf_dht_mem_types_ {
     gf_dht_mt_dht_du_t = gf_common_mt_end + 1,
@@ -20,23 +20,17 @@ enum gf_dht_mem_types_ {
     gf_dht_mt_int32_t,
     gf_dht_mt_xlator_t,
     gf_dht_mt_dht_layout_t,
-    gf_switch_mt_dht_conf_t,
-    gf_switch_mt_dht_du_t,
     gf_switch_mt_switch_sched_array,
     gf_switch_mt_switch_struct,
     gf_dht_mt_subvol_time,
     gf_dht_mt_loc_t,
     gf_defrag_info_mt,
     gf_dht_mt_inode_ctx_t,
-    gf_dht_mt_ctx_stat_time_t,
     gf_dht_mt_dirent_t,
     gf_dht_mt_container_t,
     gf_dht_mt_octx_t,
     gf_dht_mt_miginfo_t,
-    gf_tier_mt_bricklist_t,
-    gf_tier_mt_ipc_ctr_params_t,
     gf_dht_mt_fd_ctx_t,
-    gf_tier_mt_qfile_array_t,
     gf_dht_ret_cache_t,
     gf_dht_nodeuuids_t,
     gf_dht_mt_end
