@@ -13,10 +13,7 @@
  */
 #include "dht-common.h"
 #include "dht-lock.h"
-#include <glusterfs/defaults.h>
 
-int
-dht_rename_unlock(call_frame_t *frame, xlator_t *this);
 int32_t
 dht_rename_lock_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
                     int32_t op_ret, int32_t op_errno, dict_t *xdata);
@@ -58,8 +55,6 @@ dht_rename_dir_unlock_dst(call_frame_t *frame, xlator_t *this)
 {
     dht_local_t *local = NULL;
     int op_ret = -1;
-    char src_gfid[GF_UUID_BUF_SIZE] = {0};
-    char dst_gfid[GF_UUID_BUF_SIZE] = {0};
 
     local = frame->local;
 
@@ -67,10 +62,11 @@ dht_rename_dir_unlock_dst(call_frame_t *frame, xlator_t *this)
     dht_unlock_entrylk_wrapper(frame, &local->lock[1].ns.directory_ns);
 
     /* Unlock inodelk */
-    op_ret = dht_unlock_inodelk(frame, local->lock[1].ns.parent_layout.locks,
-                                local->lock[1].ns.parent_layout.lk_count,
+    op_ret = dht_unlock_inodelk(frame, &local->lock[1].ns.parent_layout,
                                 dht_rename_unlock_cbk);
     if (op_ret < 0) {
+        char src_gfid[GF_UUID_BUF_SIZE];
+        char dst_gfid[GF_UUID_BUF_SIZE] = {0};
         uuid_utoa_r(local->loc.inode->gfid, src_gfid);
 
         if (local->loc2.inode)
@@ -196,7 +192,7 @@ unwind:
     return 0;
 }
 
-int
+static int
 dht_rename_hashed_dir_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
                           int32_t op_ret, int32_t op_errno, struct iatt *stbuf,
                           struct iatt *preoldparent, struct iatt *postoldparent,
@@ -262,7 +258,7 @@ unwind:
     return 0;
 }
 
-int
+static int
 dht_rename_dir_do(call_frame_t *frame, xlator_t *this)
 {
     dht_local_t *local = NULL;
@@ -284,7 +280,7 @@ err:
     return 0;
 }
 
-int
+static int
 dht_rename_readdir_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
                        int op_ret, int op_errno, gf_dirent_t *entries,
                        dict_t *xdata)
@@ -312,7 +308,7 @@ dht_rename_readdir_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
     return 0;
 }
 
-int
+static int
 dht_rename_opendir_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
                        int op_ret, int op_errno, fd_t *fd, dict_t *xdata)
 {
@@ -348,7 +344,7 @@ err:
     return 0;
 }
 
-int
+static int
 dht_rename_dir_lock2_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
                          int32_t op_ret, int32_t op_errno, dict_t *xdata)
 {
@@ -407,7 +403,7 @@ err:
     return 0;
 }
 
-int
+static int
 dht_rename_dir_lock1_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
                          int32_t op_ret, int32_t op_errno, dict_t *xdata)
 {
@@ -703,18 +699,18 @@ dht_rename_unlock(call_frame_t *frame, xlator_t *this)
 {
     dht_local_t *local = NULL;
     int op_ret = -1;
-    char src_gfid[GF_UUID_BUF_SIZE] = {0};
-    char dst_gfid[GF_UUID_BUF_SIZE] = {0};
-    dht_ilock_wrap_t inodelk_wrapper = {
-        0,
-    };
+    dht_lock_wrap_t inodelk_wrapper;
 
     local = frame->local;
+
+    memset(&inodelk_wrapper, 0, sizeof(dht_lock_wrap_t));
     inodelk_wrapper.locks = local->rename_inodelk_backward_compatible;
     inodelk_wrapper.lk_count = local->rename_inodelk_bc_count;
 
     op_ret = dht_unlock_inodelk_wrapper(frame, &inodelk_wrapper);
     if (op_ret < 0) {
+        char src_gfid[GF_UUID_BUF_SIZE];
+        char dst_gfid[GF_UUID_BUF_SIZE] = {0};
         uuid_utoa_r(local->loc.inode->gfid, src_gfid);
 
         if (local->loc2.inode)
@@ -760,7 +756,7 @@ dht_rename_done(call_frame_t *frame, xlator_t *this)
     return 0;
 }
 
-int
+static int
 dht_rename_unlink_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
                       int32_t op_ret, int32_t op_errno, struct iatt *preparent,
                       struct iatt *postparent, dict_t *xdata)
@@ -1005,7 +1001,7 @@ unwind:
     return 0;
 }
 
-int
+static int
 dht_rename_links_create_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
                             int32_t op_ret, int32_t op_errno, inode_t *inode,
                             struct iatt *stbuf, struct iatt *preparent,
@@ -1233,7 +1229,7 @@ cleanup:
     return 0;
 }
 
-int
+static int
 dht_rename_linkto_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
                       int32_t op_ret, int32_t op_errno, inode_t *inode,
                       struct iatt *stbuf, struct iatt *preparent,
@@ -1459,7 +1455,7 @@ cleanup:
     return ret;
 }
 
-int
+static int
 dht_rename_lookup_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
                       int op_ret, int op_errno, inode_t *inode,
                       struct iatt *stbuf, dict_t *xattr,
@@ -1659,7 +1655,7 @@ err:
     return 0;
 }
 
-int32_t
+static int32_t
 dht_rename_file_protect_namespace(call_frame_t *frame, void *cookie,
                                   xlator_t *this, int32_t op_ret,
                                   int32_t op_errno, dict_t *xdata)
