@@ -8,7 +8,11 @@
   cases as published by the Free Software Foundation.
 */
 
-#include <glusterfs/byte-order.h>
+#ifdef __FreeBSD__
+#include <sys/endian.h>
+#else
+#include <endian.h>
+#endif
 #include <glusterfs/hashfn.h>
 
 #include "ec-mem-types.h"
@@ -185,7 +189,7 @@ ec_lock_conflict(ec_lock_link_t *l1, ec_lock_link_t *l2)
     return ec_is_range_conflict(l1, l2);
 }
 
-uint32_t
+static uint32_t
 ec_select_first_by_read_policy(ec_t *ec, ec_fop_data_t *fop)
 {
     if (ec->read_policy == EC_ROUND_ROBIN) {
@@ -227,10 +231,10 @@ ec_child_next(ec_t *ec, ec_fop_data_t *fop, uint32_t idx)
     return idx;
 }
 
-int32_t
+static int32_t
 ec_heal_report(call_frame_t *frame, void *cookie, xlator_t *this,
                int32_t op_ret, int32_t op_errno, uintptr_t mask, uintptr_t good,
-               uintptr_t bad, dict_t *xdata)
+               uintptr_t bad, uint32_t pending, dict_t *xdata)
 {
     if (op_ret < 0) {
         gf_msg(this->name, GF_LOG_DEBUG, op_errno, EC_MSG_HEAL_FAIL,
@@ -249,13 +253,11 @@ ec_heal_report(call_frame_t *frame, void *cookie, xlator_t *this,
 }
 
 static uintptr_t
-ec_fop_needs_name_heal(ec_fop_data_t *fop)
+ec_fop_needs_name_heal(ec_t *ec, ec_fop_data_t *fop)
 {
-    ec_t *ec = NULL;
     ec_cbk_data_t *cbk = NULL;
     ec_cbk_data_t *enoent_cbk = NULL;
 
-    ec = fop->xl->private;
     if (fop->id != GF_FOP_LOOKUP)
         return 0;
 
@@ -276,11 +278,9 @@ ec_fop_needs_name_heal(ec_fop_data_t *fop)
     return ec->xl_up & ~enoent_cbk->mask;
 }
 
-int32_t
-ec_fop_needs_heal(ec_fop_data_t *fop)
+static int32_t
+ec_fop_needs_heal(ec_t *ec, ec_fop_data_t *fop)
 {
-    ec_t *ec = fop->xl->private;
-
     if (fop->lock_count == 0) {
         /*
          * if fop->lock_count is zero that means it saw version mismatch
@@ -296,14 +296,14 @@ ec_fop_needs_heal(ec_fop_data_t *fop)
     return (ec->xl_up & ~(fop->remaining | fop->good)) != 0;
 }
 
-void
+static void
 ec_check_status(ec_fop_data_t *fop)
 {
     ec_t *ec = fop->xl->private;
     int32_t partial = 0;
     char str1[32], str2[32], str3[32], str4[32], str5[32];
 
-    if (!ec_fop_needs_name_heal(fop) && !ec_fop_needs_heal(fop)) {
+    if (!ec_fop_needs_name_heal(ec, fop) && !ec_fop_needs_heal(ec, fop)) {
         return;
     }
 
@@ -316,17 +316,19 @@ ec_check_status(ec_fop_data_t *fop)
         }
     }
 
-    gf_msg(fop->xl->name, GF_LOG_WARNING, 0, EC_MSG_OP_FAIL_ON_SUBVOLS,
-           "Operation failed on %d of %d subvolumes.(up=%s, mask=%s, "
-           "remaining=%s, good=%s, bad=%s, %s)",
-           gf_bits_count(ec->xl_up & ~(fop->remaining | fop->good)), ec->nodes,
-           ec_bin(str1, sizeof(str1), ec->xl_up, ec->nodes),
-           ec_bin(str2, sizeof(str2), fop->mask, ec->nodes),
-           ec_bin(str3, sizeof(str3), fop->remaining, ec->nodes),
-           ec_bin(str4, sizeof(str4), fop->good, ec->nodes),
-           ec_bin(str5, sizeof(str5), ec->xl_up & ~(fop->remaining | fop->good),
-                  ec->nodes),
-           ec_msg_str(fop));
+    gf_msg(
+        fop->xl->name, GF_LOG_WARNING, 0, EC_MSG_OP_FAIL_ON_SUBVOLS,
+        "Operation failed on %d of %d subvolumes.(up=%s, mask=%s, "
+        "remaining=%s, good=%s, bad=%s,"
+        "(Least significant bit represents first client/brick of subvol), %s)",
+        gf_bits_count(ec->xl_up & ~(fop->remaining | fop->good)), ec->nodes,
+        ec_bin(str1, sizeof(str1), ec->xl_up, ec->nodes),
+        ec_bin(str2, sizeof(str2), fop->mask, ec->nodes),
+        ec_bin(str3, sizeof(str3), fop->remaining, ec->nodes),
+        ec_bin(str4, sizeof(str4), fop->good, ec->nodes),
+        ec_bin(str5, sizeof(str5), ec->xl_up & ~(fop->remaining | fop->good),
+               ec->nodes),
+        ec_msg_str(fop));
     if (fop->use_fd) {
         if (fop->fd != NULL) {
             ec_fheal(NULL, fop->xl, -1, EC_MINIMUM_ONE, ec_heal_report, NULL,
@@ -445,7 +447,7 @@ ec_sleep(ec_fop_data_t *fop)
     UNLOCK(&fop->lock);
 }
 
-int32_t
+static int32_t
 ec_check_complete(ec_fop_data_t *fop, ec_resume_f resume)
 {
     int32_t error = -1;
@@ -516,7 +518,7 @@ ec_resume_parent(ec_fop_data_t *fop)
     }
 }
 
-gf_boolean_t
+static gf_boolean_t
 ec_is_recoverable_error(int32_t op_errno)
 {
     switch (op_errno) {
@@ -612,36 +614,71 @@ ec_msg_str(ec_fop_data_t *fop)
 {
     loc_t *loc1 = NULL;
     loc_t *loc2 = NULL;
-    char gfid1[64] = {0};
-    char gfid2[64] = {0};
+    ec_fop_data_t *parent = fop->parent;
 
     if (fop->errstr)
         return fop->errstr;
-
     if (!fop->use_fd) {
         loc1 = &fop->loc[0];
         loc2 = &fop->loc[1];
 
         if (fop->id == GF_FOP_RENAME) {
+            char gfid2[64];
             gf_asprintf(&fop->errstr,
                         "FOP : '%s' failed on '%s' and '%s' with gfids "
-                        "%s and %s respectively",
+                        "%s and %s respectively. Parent FOP: %s",
                         ec_fop_name(fop->id), loc1->path, loc2->path,
-                        uuid_utoa_r(loc1->gfid, gfid1),
-                        uuid_utoa_r(loc2->gfid, gfid2));
+                        uuid_utoa(loc1->gfid), uuid_utoa_r(loc2->gfid, gfid2),
+                        parent ? ec_fop_name(parent->id) : "No Parent");
         } else {
-            gf_asprintf(&fop->errstr, "FOP : '%s' failed on '%s' with gfid %s",
-                        ec_fop_name(fop->id), loc1->path,
-                        uuid_utoa_r(loc1->gfid, gfid1));
+            gf_asprintf(
+                &fop->errstr,
+                "FOP : '%s' failed on '%s' with gfid %s. Parent FOP: %s",
+                ec_fop_name(fop->id), loc1->path, uuid_utoa(loc1->gfid),
+                parent ? ec_fop_name(parent->id) : "No Parent");
         }
     } else {
-        gf_asprintf(&fop->errstr, "FOP : '%s' failed on gfid %s",
-                    ec_fop_name(fop->id),
-                    uuid_utoa_r(fop->fd->inode->gfid, gfid1));
+        gf_asprintf(&fop->errstr,
+                    "FOP : '%s' failed on gfid %s. Parent FOP: %s",
+                    ec_fop_name(fop->id), uuid_utoa(fop->fd->inode->gfid),
+                    parent ? ec_fop_name(parent->id) : "No Parent");
     }
     return fop->errstr;
 }
 
+static void
+ec_log_insufficient_vol(ec_fop_data_t *fop, ec_t *ec, int32_t have,
+                        uint32_t need, int32_t loglevel)
+{
+    char str1[32], str2[32], str3[32];
+
+    gf_msg(ec->xl->name, loglevel, 0, EC_MSG_CHILDS_INSUFFICIENT,
+           "Insufficient available children for this request: "
+           "Have : %d, Need : %u : Child UP : %s "
+           "Mask: %s, Healing : %s : %s ",
+           have, need, ec_bin(str1, sizeof(str1), ec->xl_up, ec->nodes),
+           ec_bin(str2, sizeof(str2), fop->mask, ec->nodes),
+           ec_bin(str3, sizeof(str3), fop->healing, ec->nodes),
+           ec_msg_str(fop));
+}
+
+static gf_boolean_t
+ec_is_data_fop(glusterfs_fop_t fop)
+{
+    switch (fop) {
+        case GF_FOP_WRITE:
+        case GF_FOP_TRUNCATE:
+        case GF_FOP_FTRUNCATE:
+        case GF_FOP_FALLOCATE:
+        case GF_FOP_DISCARD:
+        case GF_FOP_ZEROFILL:
+            return _gf_true;
+        default:
+            return _gf_false;
+    }
+    return _gf_false;
+}
+
 static int32_t
 ec_child_select(ec_fop_data_t *fop)
 {
@@ -699,11 +736,7 @@ ec_child_select(ec_fop_data_t *fop)
     ec_trace("SELECT", fop, "");
 
     if ((num < fop->minimum) && (num < ec->fragments)) {
-        gf_msg(ec->xl->name, GF_LOG_ERROR, 0, EC_MSG_CHILDS_INSUFFICIENT,
-               "Insufficient available children "
-               "for this request (have %d, need "
-               "%d). %s",
-               num, fop->minimum, ec_msg_str(fop));
+        ec_log_insufficient_vol(fop, ec, num, fop->minimum, GF_LOG_ERROR);
         return 0;
     }
 
@@ -711,11 +744,8 @@ ec_child_select(ec_fop_data_t *fop)
         (fop->locks[0].update[EC_DATA_TXN] ||
          fop->locks[0].update[EC_METADATA_TXN])) {
         if (ec->quorum_count && (num < ec->quorum_count)) {
-            gf_msg(ec->xl->name, GF_LOG_ERROR, 0, EC_MSG_CHILDS_INSUFFICIENT,
-                   "Insufficient available children "
-                   "for this request (have %d, need "
-                   "%d). %s",
-                   num, ec->quorum_count, ec_msg_str(fop));
+            ec_log_insufficient_vol(fop, ec, num, ec->quorum_count,
+                                    GF_LOG_ERROR);
             return 0;
         }
     }
@@ -750,7 +780,7 @@ ec_dispatch_next(ec_fop_data_t *fop, uint32_t idx)
     }
 }
 
-void
+static void
 ec_dispatch_mask(ec_fop_data_t *fop, uintptr_t mask)
 {
     ec_t *ec = fop->xl->private;
@@ -779,7 +809,7 @@ ec_dispatch_mask(ec_fop_data_t *fop, uintptr_t mask)
     }
 }
 
-void
+static void
 ec_dispatch_start(ec_fop_data_t *fop)
 {
     fop->answer = NULL;
@@ -902,7 +932,7 @@ ec_succeed_all(ec_fop_data_t *fop)
     }
 }
 
-ec_lock_t *
+static ec_lock_t *
 ec_lock_allocate(ec_fop_data_t *fop, loc_t *loc)
 {
     ec_t *ec = fop->xl->private;
@@ -938,7 +968,7 @@ ec_lock_allocate(ec_fop_data_t *fop, loc_t *loc)
     return lock;
 }
 
-void
+static void
 ec_lock_destroy(ec_lock_t *lock)
 {
     loc_wipe(&lock->loc);
@@ -949,7 +979,7 @@ ec_lock_destroy(ec_lock_t *lock)
     mem_put(lock);
 }
 
-int32_t
+static int
 ec_lock_compare(ec_lock_t *lock1, ec_lock_t *lock2)
 {
     return gf_uuid_compare(lock1->loc.gfid, lock2->loc.gfid);
@@ -1128,7 +1158,7 @@ ec_lock_prepare_fd(ec_fop_data_t *fop, fd_t *fd, uint32_t flags, off_t fl_start,
     loc_wipe(&loc);
 }
 
-gf_boolean_t
+static gf_boolean_t
 ec_config_check(xlator_t *xl, ec_config_t *config)
 {
     ec_t *ec;
@@ -1174,7 +1204,7 @@ ec_config_check(xlator_t *xl, ec_config_t *config)
     return _gf_true;
 }
 
-gf_boolean_t
+static gf_boolean_t
 ec_set_dirty_flag(ec_lock_link_t *link, ec_inode_t *ctx, uint64_t *dirty)
 {
     gf_boolean_t set_dirty = _gf_false;
@@ -1196,7 +1226,7 @@ ec_set_dirty_flag(ec_lock_link_t *link, ec_inode_t *ctx, uint64_t *dirty)
     return set_dirty;
 }
 
-int32_t
+static int32_t
 ec_prepare_update_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
                       int32_t op_ret, int32_t op_errno, dict_t *dict,
                       dict_t *xdata)
@@ -1399,7 +1429,7 @@ ec_set_xattrop_flags_and_params(ec_lock_t *lock, ec_lock_link_t *link,
     return oldflags ^ newflags;
 }
 
-void
+static void
 ec_get_size_version(ec_lock_link_t *link)
 {
     loc_t loc;
@@ -1423,8 +1453,6 @@ ec_get_size_version(ec_lock_link_t *link)
         !ec_is_data_fop(fop->id))
         link->optimistic_changelog = _gf_true;
 
-    memset(&loc, 0, sizeof(loc));
-
     LOCK(&lock->loc.inode->lock);
 
     set_dirty = ec_set_dirty_flag(link, ctx, dirty);
@@ -1530,6 +1558,8 @@ unlock:
         ec_xattrop(fop->frame, fop->xl, fop->mask, fop->minimum,
                    ec_prepare_update_cbk, link, &loc, GF_XATTROP_ADD_ARRAY64,
                    dict, xdata);
+
+        loc_wipe(&loc);
     } else {
         ec_fxattrop(fop->frame, fop->xl, fop->mask, fop->minimum,
                     ec_prepare_update_cbk, link, lock->fd,
@@ -1542,8 +1572,6 @@ out:
     fop->frame->root->uid = fop->uid;
     fop->frame->root->gid = fop->gid;
 
-    loc_wipe(&loc);
-
     if (dict != NULL) {
         dict_unref(dict);
     }
@@ -1676,7 +1704,7 @@ unlock:
     UNLOCK(&inode->lock);
 }
 
-int32_t
+static int32_t
 ec_get_real_size_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
                      int32_t op_ret, int32_t op_errno, inode_t *inode,
                      struct iatt *buf, dict_t *xdata, struct iatt *postparent)
@@ -1703,7 +1731,7 @@ ec_get_real_size_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
  * Any error processing this request is ignored. In the worst case, an invalid
  * or not up to date value in the iatt could cause some cache invalidation.
  */
-void
+static void
 ec_get_real_size(ec_lock_link_t *link)
 {
     ec_fop_data_t *fop;
@@ -1825,7 +1853,7 @@ ec_lock_apply(ec_lock_link_t *link)
     ec_get_real_size(link);
 }
 
-gf_boolean_t
+static gf_boolean_t
 ec_lock_acquire(ec_lock_link_t *link);
 
 static void
@@ -1850,7 +1878,7 @@ ec_lock_resume_shared(struct list_head *list)
     }
 }
 
-void
+static void
 ec_lock_acquired(ec_lock_link_t *link)
 {
     struct list_head list;
@@ -1888,7 +1916,7 @@ ec_lock_acquired(ec_lock_link_t *link)
     ec_lock_resume_shared(&list);
 }
 
-int32_t
+static int32_t
 ec_locked(call_frame_t *frame, void *cookie, xlator_t *this, int32_t op_ret,
           int32_t op_errno, dict_t *xdata)
 {
@@ -1917,7 +1945,7 @@ ec_locked(call_frame_t *frame, void *cookie, xlator_t *this, int32_t op_ret,
     return 0;
 }
 
-gf_boolean_t
+static gf_boolean_t
 ec_lock_acquire(ec_lock_link_t *link)
 {
     ec_lock_t *lock;
@@ -2176,7 +2204,7 @@ ec_lock(ec_fop_data_t *fop)
     ec_resume(fop, 0);
 }
 
-void
+static void
 ec_lock_unfreeze(ec_lock_link_t *link)
 {
     struct list_head list;
@@ -2231,7 +2259,7 @@ ec_lock_unfreeze(ec_lock_link_t *link)
     }
 }
 
-int32_t
+static int32_t
 ec_unlocked(call_frame_t *frame, void *cookie, xlator_t *this, int32_t op_ret,
             int32_t op_errno, dict_t *xdata)
 {
@@ -2250,7 +2278,7 @@ ec_unlocked(call_frame_t *frame, void *cookie, xlator_t *this, int32_t op_ret,
     return 0;
 }
 
-void
+static void
 ec_unlock_lock(ec_lock_link_t *link)
 {
     ec_lock_t *lock;
@@ -2277,7 +2305,7 @@ ec_unlock_lock(ec_lock_link_t *link)
     }
 }
 
-void
+static void
 ec_inode_bad_inc(inode_t *inode, xlator_t *xl)
 {
     ec_inode_t *ctx = NULL;
@@ -2294,7 +2322,7 @@ unlock:
     UNLOCK(&inode->lock);
 }
 
-int32_t
+static int32_t
 ec_update_size_version_done(call_frame_t *frame, void *cookie, xlator_t *this,
                             int32_t op_ret, int32_t op_errno, dict_t *xattr,
                             dict_t *xdata)
@@ -2353,7 +2381,7 @@ ec_update_size_version_done(call_frame_t *frame, void *cookie, xlator_t *this,
     return 0;
 }
 
-void
+static void
 ec_update_size_version(ec_lock_link_t *link, uint64_t *version, uint64_t size,
                        uint64_t *dirty)
 {
@@ -2448,7 +2476,7 @@ out:
     }
 }
 
-gf_boolean_t
+static gf_boolean_t
 ec_update_info(ec_lock_link_t *link)
 {
     ec_lock_t *lock;
@@ -2515,7 +2543,7 @@ ec_update_info(ec_lock_link_t *link)
     return _gf_false;
 }
 
-void
+static void
 ec_unlock_now(ec_lock_link_t *link)
 {
     ec_lock_t *lock;
@@ -2584,10 +2612,10 @@ done:
     }
 }
 
-void
+static void
 ec_unlock_timer_add(ec_lock_link_t *link);
 
-void
+static void
 ec_unlock_timer_del(ec_lock_link_t *link)
 {
     ec_lock_t *lock;
@@ -2665,7 +2693,7 @@ ec_unlock_timer_del(ec_lock_link_t *link)
     }
 }
 
-void
+static void
 ec_unlock_timer_cbk(void *data)
 {
     ec_unlock_timer_del(data);
@@ -2722,12 +2750,13 @@ ec_lock_delay_create(ec_lock_link_t *link)
     return _gf_true;
 }
 
-void
+static void
 ec_unlock_timer_add(ec_lock_link_t *link)
 {
     ec_fop_data_t *fop = link->fop;
     ec_lock_t *lock = link->lock;
     gf_boolean_t now = _gf_false;
+    ec_t *ec = fop->xl->private;
 
     LOCK(&lock->loc.inode->lock);
 
@@ -2749,7 +2778,7 @@ ec_unlock_timer_add(ec_lock_link_t *link)
 
     /* If the fop detects that a heal is needed, we mark the lock to be
      * released as soon as possible. */
-    lock->release |= ec_fop_needs_heal(fop);
+    lock->release |= ec_fop_needs_heal(ec, fop);
 
     if (lock->refs_owners > 1) {
         ec_trace("UNLOCK_SKIP", fop, "lock=%p", lock);
@@ -2766,8 +2795,6 @@ ec_unlock_timer_add(ec_lock_link_t *link)
          * owner, so the waiting list should be empty. */
         GF_ASSERT(list_empty(&lock->owners) && list_empty(&lock->waiting));
 
-        ec_t *ec = fop->xl->private;
-
         /* If everything goes as expected this fop will be put to sleep until
          * the timer callback is executed. */
         ec_sleep(fop);
@@ -2958,7 +2985,7 @@ ec_lock_reuse(ec_fop_data_t *fop)
     }
 }
 
-void
+static void
 __ec_manager(ec_fop_data_t *fop, int32_t error)
 {
     ec_t *ec = fop->xl->private;
