@@ -15,14 +15,14 @@
 #include "changelog-rpc-common.h"
 #include "changelog-lib-messages.h"
 
-#include "syscall.h"
+#include <glusterfs/syscall.h>
 
 /**
  * Reverse socket: actual data transfer handler. Connection
  * initiator is PROBER, data transfer is REBORP.
  */
 
-struct rpcsvc_program *gf_changelog_reborp_programs[];
+static struct rpcsvc_program *gf_changelog_reborp_programs[];
 
 void *
 gf_changelog_connection_janitor(void *arg)
@@ -55,9 +55,8 @@ gf_changelog_connection_janitor(void *arg)
         ev = &entry->event;
 
         gf_smsg(this->name, GF_LOG_INFO, 0,
-                CHANGELOG_LIB_MSG_CLEANING_BRICK_ENTRY_INFO,
-                "Cleaning brick entry for brick", "brick=%s", entry->brick,
-                NULL);
+                CHANGELOG_LIB_MSG_CLEANING_BRICK_ENTRY_INFO, "brick=%s",
+                entry->brick, NULL);
 
         /* 0x0: disable rpc-clnt */
         rpc_clnt_disable(RPC_PROBER(entry));
@@ -71,21 +70,19 @@ gf_changelog_connection_janitor(void *arg)
         while (!list_empty(&ev->events)) {
             event = list_first_entry(&ev->events, struct gf_event, list);
             gf_smsg(this->name, GF_LOG_INFO, 0,
-                    CHANGELOG_LIB_MSG_DRAINING_EVENT_INFO, "Draining event",
-                    "seq=%lu", event->seq, "payload=%d", event->count, NULL);
+                    CHANGELOG_LIB_MSG_DRAINING_EVENT_INFO, "seq=%lu",
+                    event->seq, "payload=%d", event->count, NULL);
 
             GF_FREE(event);
             drained++;
         }
 
         gf_smsg(this->name, GF_LOG_INFO, 0,
-                CHANGELOG_LIB_MSG_DRAINING_EVENT_INFO, "Drained events",
-                "num=%lu", drained, NULL);
+                CHANGELOG_LIB_MSG_DRAINED_EVENT_INFO, "num=%lu", drained, NULL);
 
         /* 0x3: freeup brick entry */
         gf_smsg(this->name, GF_LOG_INFO, 0,
-                CHANGELOG_LIB_MSG_FREEING_ENTRY_INFO, "freeing entry",
-                "entry=%p", entry, NULL);
+                CHANGELOG_LIB_MSG_FREEING_ENTRY_INFO, "entry=%p", entry, NULL);
         LOCK_DESTROY(&entry->statelock);
         GF_FREE(entry);
     }
@@ -112,9 +109,7 @@ gf_changelog_reborp_rpcsvc_notify(rpcsvc_t *rpc, void *mydata,
             ret = sys_unlink(RPC_SOCK(entry));
             if (ret != 0)
                 gf_smsg(this->name, GF_LOG_WARNING, errno,
-                        CHANGELOG_LIB_MSG_UNLINK_FAILED,
-                        "failed to unlink "
-                        "reverse socket",
+                        CHANGELOG_LIB_MSG_UNLINK_FAILED, "name=reverse socket",
                         "path=%s", RPC_SOCK(entry), NULL);
             if (entry->connected)
                 GF_CHANGELOG_INVOKE_CBK(this, entry->connected, entry->brick,
@@ -353,7 +348,9 @@ gf_changelog_event_handler(rpcsvc_request_t *req, xlator_t *this,
     }
 
     gf_msg_debug(this->name, 0,
-                 "seq: %lu [%s] (time: %lu.%lu), (vec: %d, len: %zd)",
+                 "seq: %" PRIu64 " [%s] (time: %" PRIu64 ".%" PRIu64
+                 "), "
+                 "(vec: %d, len: %zd)",
                  rpc_req.seq, entry->brick, rpc_req.tv_sec, rpc_req.tv_usec,
                  payloadcnt, payloadlen);
 
@@ -389,11 +386,10 @@ gf_changelog_reborp_handle_event(rpcsvc_request_t *req)
     return gf_changelog_event_handler(req, this, entry);
 }
 
-rpcsvc_actor_t gf_changelog_reborp_actors[CHANGELOG_REV_PROC_MAX] = {
+static rpcsvc_actor_t gf_changelog_reborp_actors[CHANGELOG_REV_PROC_MAX] = {
     [CHANGELOG_REV_PROC_EVENT] = {"CHANGELOG EVENT HANDLER",
-                                  CHANGELOG_REV_PROC_EVENT,
-                                  gf_changelog_reborp_handle_event, NULL, 0,
-                                  DRC_NA},
+                                  gf_changelog_reborp_handle_event, NULL,
+                                  CHANGELOG_REV_PROC_EVENT, DRC_NA, 0},
 };
 
 /**
@@ -402,7 +398,7 @@ rpcsvc_actor_t gf_changelog_reborp_actors[CHANGELOG_REV_PROC_MAX] = {
  * and that's required to invoke the callback with the appropriate
  * brick path and it's private data.
  */
-struct rpcsvc_program gf_changelog_reborp_prog = {
+static struct rpcsvc_program gf_changelog_reborp_prog = {
     .progname = "LIBGFCHANGELOG REBORP",
     .prognum = CHANGELOG_REV_RPC_PROCNUM,
     .progver = CHANGELOG_REV_RPC_PROCVER,
@@ -411,7 +407,7 @@ struct rpcsvc_program gf_changelog_reborp_prog = {
     .synctask = _gf_false,
 };
 
-struct rpcsvc_program *gf_changelog_reborp_programs[] = {
+static struct rpcsvc_program *gf_changelog_reborp_programs[] = {
     &gf_changelog_reborp_prog,
     NULL,
 };
