@@ -8,9 +8,6 @@
  *   cases as published by the Free Software Foundation.
  */
 
-#include <glusterfs/glusterfs.h>
-#include <glusterfs/xlator.h>
-#include <glusterfs/defaults.h>
 #include "cloudsync.h"
 #include "cloudsync-common.h"
 #include <glusterfs/call-stub.h>
