@@ -9,11 +9,10 @@
 */
 #include "index.h"
 #include <glusterfs/options.h>
-#include "glusterfs3-xdr.h"
+#include "glusterfs4-xdr.h"
 #include <glusterfs/syscall.h>
 #include <glusterfs/statedump.h>
 #include <glusterfs/syncop.h>
-#include <glusterfs/common-utils.h>
 #include "index-messages.h"
 #include <ftw.h>
 #include <libgen.h> /* for dirname() */
@@ -39,7 +38,7 @@ static char *index_subdirs[XATTROP_TYPE_END] = {
     [DIRTY] = DIRTY_SUBDIR,
     [ENTRY_CHANGES] = ENTRY_CHANGES_SUBDIR};
 
-int
+static int
 index_get_type_from_vgfid(index_priv_t *priv, uuid_t vgfid)
 {
     int i = 0;
@@ -51,7 +50,7 @@ index_get_type_from_vgfid(index_priv_t *priv, uuid_t vgfid)
     return -1;
 }
 
-gf_boolean_t
+static gf_boolean_t
 index_is_virtual_gfid(index_priv_t *priv, uuid_t vgfid)
 {
     if (index_get_type_from_vgfid(priv, vgfid) < 0)
@@ -59,8 +58,8 @@ index_is_virtual_gfid(index_priv_t *priv, uuid_t vgfid)
     return _gf_true;
 }
 
-static int
-__index_inode_ctx_get(inode_t *inode, xlator_t *this, index_inode_ctx_t **ctx)
+static index_inode_ctx_t *
+__index_inode_ctx_get(inode_t *inode, xlator_t *this)
 {
     int ret = 0;
     index_inode_ctx_t *ictx = NULL;
@@ -72,8 +71,7 @@ __index_inode_ctx_get(inode_t *inode, xlator_t *this, index_inode_ctx_t **ctx)
         goto out;
     }
     ictx = GF_CALLOC(1, sizeof(*ictx), gf_index_inode_ctx_t);
-    if (!ictx) {
-        ret = -1;
+    if (ictx == NULL) {
         goto out;
     }
 
@@ -82,39 +80,36 @@ __index_inode_ctx_get(inode_t *inode, xlator_t *this, index_inode_ctx_t **ctx)
     if (ret) {
         GF_FREE(ictx);
         ictx = NULL;
-        goto out;
     }
+
 out:
-    if (ictx)
-        *ctx = ictx;
-    return ret;
+    return ictx;
 }
 
-static int
-index_inode_ctx_get(inode_t *inode, xlator_t *this, index_inode_ctx_t **ctx)
+static index_inode_ctx_t *
+index_inode_ctx_get(inode_t *inode, xlator_t *this)
 {
-    int ret = 0;
+    index_inode_ctx_t *ctx = NULL;
 
     LOCK(&inode->lock);
     {
-        ret = __index_inode_ctx_get(inode, this, ctx);
+        ctx = __index_inode_ctx_get(inode, this);
     }
     UNLOCK(&inode->lock);
 
-    return ret;
+    return ctx;
 }
 
 static gf_boolean_t
 index_is_subdir_of_entry_changes(xlator_t *this, inode_t *inode)
 {
     index_inode_ctx_t *ctx = NULL;
-    int ret = 0;
 
     if (!inode)
         return _gf_false;
 
-    ret = index_inode_ctx_get(inode, this, &ctx);
-    if ((ret == 0) && !gf_uuid_is_null(ctx->virtual_pargfid))
+    ctx = index_inode_ctx_get(inode, this);
+    if (ctx && !gf_uuid_is_null(ctx->virtual_pargfid))
         return _gf_true;
     return _gf_false;
 }
@@ -131,7 +126,7 @@ index_get_type_from_vgfid_xattr(const char *name)
     return -1;
 }
 
-gf_boolean_t
+static gf_boolean_t
 index_is_fop_on_internal_inode(xlator_t *this, inode_t *inode, uuid_t gfid)
 {
     index_priv_t *priv = this->private;
@@ -160,7 +155,7 @@ index_is_vgfid_xattr(const char *name)
     return _gf_true;
 }
 
-call_stub_t *
+static call_stub_t *
 __index_dequeue(struct list_head *callstubs)
 {
     call_stub_t *stub = NULL;
@@ -194,7 +189,7 @@ worker_enqueue(xlator_t *this, call_stub_t *stub)
     pthread_mutex_unlock(&priv->mutex);
 }
 
-void *
+static void *
 index_worker(void *data)
 {
     index_priv_t *priv = NULL;
@@ -242,13 +237,13 @@ index_worker(void *data)
     return NULL;
 }
 
-static void
+static int
 make_index_dir_path(char *base, const char *subdir, char *index_dir, size_t len)
 {
-    snprintf(index_dir, len, "%s/%s", base, subdir);
+    return snprintf(index_dir, len, "%s/%s", base, subdir);
 }
 
-int
+static int
 index_dir_create(xlator_t *this, const char *subdir)
 {
     int ret = 0;
@@ -261,8 +256,8 @@ index_dir_create(xlator_t *this, const char *subdir)
     size_t pathlen = 0;
 
     priv = this->private;
-    make_index_dir_path(priv->index_basepath, subdir, fullpath,
-                        sizeof(fullpath));
+    pathlen = make_index_dir_path(priv->index_basepath, subdir, fullpath,
+                                  sizeof(fullpath));
     ret = sys_stat(fullpath, &st);
     if (!ret) {
         if (!S_ISDIR(st.st_mode))
@@ -270,7 +265,6 @@ index_dir_create(xlator_t *this, const char *subdir)
         goto out;
     }
 
-    pathlen = strlen(fullpath);
     if ((pathlen > 1) && fullpath[pathlen - 1] == '/')
         fullpath[pathlen - 1] = '\0';
     dir = strchr(fullpath, '/');
@@ -304,7 +298,7 @@ out:
     return ret;
 }
 
-void
+static void
 index_get_index(index_priv_t *priv, uuid_t index)
 {
     LOCK(&priv->lock);
@@ -314,7 +308,7 @@ index_get_index(index_priv_t *priv, uuid_t index)
     UNLOCK(&priv->lock);
 }
 
-void
+static void
 index_generate_index(index_priv_t *priv, uuid_t index)
 {
     LOCK(&priv->lock);
@@ -333,17 +327,21 @@ static void
 make_index_path(char *base, const char *subdir, uuid_t index, char *index_path,
                 size_t len)
 {
-    make_index_dir_path(base, subdir, index_path, len);
-    snprintf(index_path + strlen(index_path), len - strlen(index_path),
-             "/%s-%s", subdir, uuid_utoa(index));
+    int index_path_len;
+
+    index_path_len = make_index_dir_path(base, subdir, index_path, len);
+    snprintf(index_path + index_path_len, len - index_path_len, "/%s-%s",
+             subdir, uuid_utoa(index));
 }
 
 static void
 make_gfid_path(char *base, const char *subdir, uuid_t gfid, char *gfid_path,
                size_t len)
 {
-    make_index_dir_path(base, subdir, gfid_path, len);
-    snprintf(gfid_path + strlen(gfid_path), len - strlen(gfid_path), "/%s",
+    int gfid_path_len;
+
+    gfid_path_len = make_index_dir_path(base, subdir, gfid_path, len);
+    snprintf(gfid_path + gfid_path_len, len - gfid_path_len, "/%s",
              uuid_utoa(gfid));
 }
 
@@ -351,17 +349,16 @@ static void
 make_file_path(char *base, const char *subdir, const char *filename,
                char *file_path, size_t len)
 {
-    make_index_dir_path(base, subdir, file_path, len);
-    snprintf(file_path + strlen(file_path), len - strlen(file_path), "/%s",
-             filename);
+    int file_path_len;
+
+    file_path_len = make_index_dir_path(base, subdir, file_path, len);
+    snprintf(file_path + file_path_len, len - file_path_len, "/%s", filename);
 }
 
 static int
 is_index_file_current(char *filename, uuid_t priv_index, char *subdir)
 {
-    char current_index[GF_UUID_BUF_SIZE + 16] = {
-        0,
-    };
+    char current_index[GF_UUID_BUF_SIZE + 16];
 
     snprintf(current_index, sizeof current_index, "%s-%s", subdir,
              uuid_utoa(priv_index));
@@ -449,7 +446,7 @@ index_update_link_count_cache(index_priv_t *priv, index_xattrop_type_t type,
     }
 }
 
-char *
+static char *
 index_get_subdir_from_type(index_xattrop_type_t type)
 {
     if (type < XATTROP || type >= XATTROP_TYPE_END)
@@ -457,7 +454,7 @@ index_get_subdir_from_type(index_xattrop_type_t type)
     return index_subdirs[type];
 }
 
-char *
+static char *
 index_get_subdir_from_vgfid(index_priv_t *priv, uuid_t vgfid)
 {
     return index_get_subdir_from_type(index_get_type_from_vgfid(priv, vgfid));
@@ -480,6 +477,7 @@ index_fill_readdir(fd_t *fd, index_fd_ctx_t *fctx, DIR *dir, off_t off,
     int32_t this_size = -1;
     gf_dirent_t *this_entry = NULL;
     xlator_t *this = NULL;
+    size_t entry_dname_len;
 
     this = THIS;
     if (!off) {
@@ -523,18 +521,32 @@ index_fill_readdir(fd_t *fd, index_fd_ctx_t *fctx, DIR *dir, off_t off,
             break;
         }
 
-        if (!strncmp(entry->d_name, XATTROP_SUBDIR "-",
-                     strlen(XATTROP_SUBDIR "-"))) {
-            check_delete_stale_index_file(this, entry->d_name, XATTROP_SUBDIR);
-            continue;
-        } else if (!strncmp(entry->d_name, DIRTY_SUBDIR "-",
-                            strlen(DIRTY_SUBDIR "-"))) {
-            check_delete_stale_index_file(this, entry->d_name, DIRTY_SUBDIR);
-            continue;
+        entry_dname_len = strlen(entry->d_name);
+
+        /* Skip entry name comparison unless the length is equal to either:
+         * SLEN('dirty-') + '-' + UUID_CANONICAL_FORM_LEN (36)
+         * or
+         * SLEN('xattrop') + '-' + UUID_CANONICAL_FORM_LEN (36)
+         */
+        if (entry_dname_len ==
+                (SLEN(XATTROP_SUBDIR) + 1 + UUID_CANONICAL_FORM_LEN) ||
+            entry_dname_len ==
+                (SLEN(DIRTY_SUBDIR) + 1 + UUID_CANONICAL_FORM_LEN)) {
+            if (!strncmp(entry->d_name, XATTROP_SUBDIR "-",
+                         strlen(XATTROP_SUBDIR "-"))) {
+                check_delete_stale_index_file(this, entry->d_name,
+                                              XATTROP_SUBDIR);
+                continue;
+            } else if (!strncmp(entry->d_name, DIRTY_SUBDIR "-",
+                                strlen(DIRTY_SUBDIR "-"))) {
+                check_delete_stale_index_file(this, entry->d_name,
+                                              DIRTY_SUBDIR);
+                continue;
+            }
         }
 
-        this_size = max(sizeof(gf_dirent_t), sizeof(gfs3_dirplist)) +
-                    strlen(entry->d_name) + 1;
+        this_size = max(sizeof(gf_dirent_t), sizeof(gfx_dirplist)) +
+                    entry_dname_len + 1;
 
         if (this_size + filled > size) {
             seekdir(dir, in_case);
@@ -554,14 +566,6 @@ index_fill_readdir(fd_t *fd, index_fd_ctx_t *fctx, DIR *dir, off_t off,
             break;
         }
 
-        this_entry = gf_dirent_for_name(entry->d_name);
-
-        if (!this_entry) {
-            gf_msg(THIS->name, GF_LOG_ERROR, errno,
-                   INDEX_MSG_INDEX_READDIR_FAILED,
-                   "could not create gf_dirent for entry %s", entry->d_name);
-            goto out;
-        }
         /*
          * we store the offset of next entry here, which is
          * probably not intended, but code using syncop_readdir()
@@ -569,8 +573,16 @@ index_fill_readdir(fd_t *fd, index_fd_ctx_t *fctx, DIR *dir, off_t off,
          * for directory read resumption.
          */
         last_off = (u_long)telldir(dir);
-        this_entry->d_off = last_off;
-        this_entry->d_ino = entry->d_ino;
+
+        this_entry = gf_dirent_for_name2(entry->d_name, entry_dname_len,
+                                         entry->d_ino, last_off, 0, NULL);
+
+        if (!this_entry) {
+            gf_msg(THIS->name, GF_LOG_ERROR, errno,
+                   INDEX_MSG_INDEX_READDIR_FAILED,
+                   "could not create gf_dirent for entry %s", entry->d_name);
+            goto out;
+        }
 
         list_add_tail(&this_entry->list, &entries->list);
 
@@ -590,7 +602,7 @@ out:
     return count;
 }
 
-int
+static int
 index_link_to_base(xlator_t *this, char *fpath, const char *subdir)
 {
     int ret = 0;
@@ -651,7 +663,7 @@ out:
     return -op_errno;
 }
 
-int
+static int
 index_add(xlator_t *this, uuid_t gfid, const char *subdir,
           index_xattrop_type_t type)
 {
@@ -681,7 +693,7 @@ out:
     return ret;
 }
 
-int
+static int
 index_del(xlator_t *this, uuid_t gfid, const char *subdir, int type)
 {
     int32_t op_errno __attribute__((unused)) = 0;
@@ -694,8 +706,8 @@ index_del(xlator_t *this, uuid_t gfid, const char *subdir, int type)
     uuid_t uuid;
 
     priv = this->private;
-    GF_ASSERT_AND_GOTO_WITH_ERROR(this->name, !gf_uuid_is_null(gfid), out,
-                                  op_errno, EINVAL);
+    GF_ASSERT_AND_GOTO_WITH_ERROR(!gf_uuid_is_null(gfid), out, op_errno,
+                                  EINVAL);
     make_gfid_path(priv->index_basepath, subdir, gfid, gfid_path,
                    sizeof(gfid_path));
 
@@ -781,7 +793,7 @@ index_find_xattr_type(dict_t *d, char *k, data_t *v)
     return idx;
 }
 
-int
+static int
 index_fill_zero_array(dict_t *d, char *k, data_t *v, void *adata)
 {
     int idx = -1;
@@ -832,7 +844,7 @@ _check_key_is_zero_filled(dict_t *d, char *k, data_t *v, void *tmp)
     return 0;
 }
 
-int
+static int
 index_entry_create(xlator_t *this, inode_t *inode, char *filename)
 {
     int ret = -1;
@@ -845,12 +857,12 @@ index_entry_create(xlator_t *this, inode_t *inode, char *filename)
 
     priv = this->private;
 
-    GF_ASSERT_AND_GOTO_WITH_ERROR(this->name, !gf_uuid_is_null(inode->gfid),
-                                  out, op_errno, EINVAL);
-    GF_ASSERT_AND_GOTO_WITH_ERROR(this->name, filename, out, op_errno, EINVAL);
+    GF_ASSERT_AND_GOTO_WITH_ERROR(!gf_uuid_is_null(inode->gfid), out, op_errno,
+                                  EINVAL);
+    GF_ASSERT_AND_GOTO_WITH_ERROR(filename, out, op_errno, EINVAL);
 
-    ret = index_inode_ctx_get(inode, this, &ctx);
-    if (ret) {
+    ctx = index_inode_ctx_get(inode, this);
+    if (ctx == NULL) {
         op_errno = EINVAL;
         gf_msg(this->name, GF_LOG_ERROR, op_errno,
                INDEX_MSG_INODE_CTX_GET_SET_FAILED,
@@ -894,10 +906,9 @@ out:
     return ret;
 }
 
-int
+static int
 index_entry_delete(xlator_t *this, uuid_t pgfid, char *filename)
 {
-    int ret = 0;
     int op_errno = 0;
     char pgfid_path[PATH_MAX] = {0};
     char entry_path[PATH_MAX] = {0};
@@ -906,9 +917,9 @@ index_entry_delete(xlator_t *this, uuid_t pgfid, char *filename)
 
     priv = this->private;
 
-    GF_ASSERT_AND_GOTO_WITH_ERROR(this->name, !gf_uuid_is_null(pgfid), out,
-                                  op_errno, EINVAL);
-    GF_ASSERT_AND_GOTO_WITH_ERROR(this->name, filename, out, op_errno, EINVAL);
+    GF_ASSERT_AND_GOTO_WITH_ERROR(!gf_uuid_is_null(pgfid), out, op_errno,
+                                  EINVAL);
+    GF_ASSERT_AND_GOTO_WITH_ERROR(filename, out, op_errno, EINVAL);
 
     make_gfid_path(priv->index_basepath, ENTRY_CHANGES_SUBDIR, pgfid,
                    pgfid_path, sizeof(pgfid_path));
@@ -928,18 +939,13 @@ index_entry_delete(xlator_t *this, uuid_t pgfid, char *filename)
         goto out;
     }
 
-    ret = sys_unlink(entry_path);
-    if (ret && (errno != ENOENT)) {
+    if (!gf_unlink(entry_path))
         op_errno = errno;
-        gf_msg(this->name, GF_LOG_ERROR, op_errno, INDEX_MSG_INDEX_DEL_FAILED,
-               "%s: failed to delete from index/entry-changes", entry_path);
-    }
-
 out:
     return -op_errno;
 }
 
-int
+static int
 index_entry_action(xlator_t *this, inode_t *inode, dict_t *xdata, char *key)
 {
     int ret = 0;
@@ -960,7 +966,7 @@ out:
     return ret;
 }
 
-void
+static void
 _index_action(xlator_t *this, inode_t *inode, int *zfilled)
 {
     int ret = 0;
@@ -968,14 +974,14 @@ _index_action(xlator_t *this, inode_t *inode, int *zfilled)
     index_inode_ctx_t *ctx = NULL;
     char *subdir = NULL;
 
-    ret = index_inode_ctx_get(inode, this, &ctx);
-    if (ret) {
+    ctx = index_inode_ctx_get(inode, this);
+    if (ctx == NULL) {
         gf_msg(this->name, GF_LOG_ERROR, EINVAL,
                INDEX_MSG_INODE_CTX_GET_SET_FAILED,
                "Not able to get"
                " inode context for %s.",
                uuid_utoa(inode->gfid));
-        goto out;
+        return;
     }
 
     for (i = 0; i < XATTROP_TYPE_END; i++) {
@@ -994,8 +1000,6 @@ _index_action(xlator_t *this, inode_t *inode, int *zfilled)
                 ctx->state[i] = IN;
         }
     }
-out:
-    return;
 }
 
 static void
@@ -1021,7 +1025,7 @@ index_init_state(xlator_t *this, inode_t *inode, index_inode_ctx_t *ctx,
     return;
 }
 
-void
+static void
 xattrop_index_action(xlator_t *this, index_local_t *local, dict_t *xattr,
                      dict_match_t match, void *match_data)
 {
@@ -1059,7 +1063,10 @@ xattrop_index_action(xlator_t *this, index_local_t *local, dict_t *xattr,
         goto out;
 
     subdir = index_get_subdir_from_type(ENTRY_CHANGES);
-    ret = index_inode_ctx_get(inode, this, &ctx);
+    ctx = index_inode_ctx_get(inode, this);
+    if (ctx == NULL)
+        goto out;
+
     if (ctx->state[ENTRY_CHANGES] == UNKNOWN)
         index_init_state(this, inode, ctx, subdir);
     if (ctx->state[ENTRY_CHANGES] == IN) {
@@ -1092,13 +1099,14 @@ index_xattrop_track(xlator_t *this, gf_xattrop_flags_t flags, dict_t *dict)
     return _gf_false;
 }
 
-int
+static int
 index_inode_path(xlator_t *this, inode_t *inode, char *dirpath, size_t len)
 {
     char *subdir = NULL;
     int ret = 0;
     index_priv_t *priv = NULL;
     index_inode_ctx_t *ictx = NULL;
+    int dir_path_len;
 
     priv = this->private;
     if (!index_is_fop_on_internal_inode(this, inode, NULL)) {
@@ -1112,18 +1120,19 @@ index_inode_path(xlator_t *this, inode_t *inode, char *dirpath, size_t len)
             ret = -EINVAL;
             goto out;
         }
-        make_index_dir_path(priv->index_basepath, subdir, dirpath, len);
+        (void)make_index_dir_path(priv->index_basepath, subdir, dirpath, len);
     } else {
-        ret = index_inode_ctx_get(inode, this, &ictx);
-        if (ret)
+        ictx = index_inode_ctx_get(inode, this);
+        if (ictx == NULL) {
+            ret = -1;
             goto out;
-        if (gf_uuid_is_null(ictx->virtual_pargfid)) {
+        } else if (gf_uuid_is_null(ictx->virtual_pargfid)) {
             ret = -EINVAL;
             goto out;
         }
-        make_index_dir_path(priv->index_basepath, ENTRY_CHANGES_SUBDIR, dirpath,
-                            len);
-        if (len <= strlen(dirpath) + 1 /*'/'*/ + SLEN(UUID0_STR)) {
+        dir_path_len = make_index_dir_path(priv->index_basepath,
+                                           ENTRY_CHANGES_SUBDIR, dirpath, len);
+        if (len <= dir_path_len + 1 /*'/'*/ + SLEN(UUID0_STR)) {
             ret = -EINVAL;
             goto out;
         }
@@ -1134,17 +1143,15 @@ out:
     return ret;
 }
 
-int
+static int
 __index_fd_ctx_get(fd_t *fd, xlator_t *this, index_fd_ctx_t **ctx)
 {
     int ret = 0;
     index_fd_ctx_t *fctx = NULL;
-    uint64_t tmpctx = 0;
     char dirpath[PATH_MAX] = {0};
 
-    ret = __fd_ctx_get(fd, this, &tmpctx);
-    if (!ret) {
-        fctx = (index_fd_ctx_t *)(long)tmpctx;
+    fctx = __fd_ctx_get_ptr(fd, this);
+    if (fctx) {
         *ctx = fctx;
         goto out;
     }
@@ -1181,7 +1188,7 @@ out:
     return ret;
 }
 
-int
+static int
 index_fd_ctx_get(fd_t *fd, xlator_t *this, index_fd_ctx_t **ctx)
 {
     int ret = 0;
@@ -1195,18 +1202,17 @@ index_fd_ctx_get(fd_t *fd, xlator_t *this, index_fd_ctx_t **ctx)
 
 // new - Not NULL means start a fop
 // new - NULL means done processing the fop
-void
+static void
 index_queue_process(xlator_t *this, inode_t *inode, call_stub_t *new)
 {
     call_stub_t *stub = NULL;
     index_inode_ctx_t *ctx = NULL;
-    int ret = 0;
     call_frame_t *frame = NULL;
 
     LOCK(&inode->lock);
     {
-        ret = __index_inode_ctx_get(inode, this, &ctx);
-        if (ret)
+        ctx = __index_inode_ctx_get(inode, this);
+        if (!ctx)
             goto unlock;
 
         if (new) {
@@ -1227,7 +1233,7 @@ index_queue_process(xlator_t *this, inode_t *inode, call_stub_t *new)
 unlock:
     UNLOCK(&inode->lock);
 
-    if (ret && new) {
+    if (!ctx && new) {
         frame = new->frame;
         if (new->fop == GF_FOP_XATTROP) {
             INDEX_STACK_UNWIND(xattrop, frame, -1, ENOMEM, NULL, NULL);
@@ -1264,7 +1270,7 @@ out:
     return 0;
 }
 
-int32_t
+static int32_t
 index_xattrop_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
                   int32_t op_ret, int32_t op_errno, dict_t *xattr,
                   dict_t *xdata)
@@ -1276,7 +1282,7 @@ index_xattrop_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
     return 0;
 }
 
-int32_t
+static int32_t
 index_xattrop64_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
                     int32_t op_ret, int32_t op_errno, dict_t *xattr,
                     dict_t *xdata)
@@ -1287,7 +1293,7 @@ index_xattrop64_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
                        is_xattr_in_watchlist, priv->pending_watchlist);
 }
 
-void
+static void
 index_xattrop_do(call_frame_t *frame, xlator_t *this, loc_t *loc, fd_t *fd,
                  gf_xattrop_flags_t optype, dict_t *xattr, dict_t *xdata)
 {
@@ -1341,7 +1347,7 @@ index_xattrop_do(call_frame_t *frame, xlator_t *this, loc_t *loc, fd_t *fd,
                    FIRST_CHILD(this)->fops->fxattrop, fd, optype, xattr, xdata);
 }
 
-int
+static int
 index_xattrop_wrapper(call_frame_t *frame, xlator_t *this, loc_t *loc,
                       gf_xattrop_flags_t optype, dict_t *xattr, dict_t *xdata)
 {
@@ -1349,7 +1355,7 @@ index_xattrop_wrapper(call_frame_t *frame, xlator_t *this, loc_t *loc,
     return 0;
 }
 
-int
+static int
 index_fxattrop_wrapper(call_frame_t *frame, xlator_t *this, fd_t *fd,
                        gf_xattrop_flags_t optype, dict_t *xattr, dict_t *xdata)
 {
@@ -1427,8 +1433,8 @@ out:
     return 0;
 }
 
-uint64_t
-index_entry_count(xlator_t *this, char *subdir)
+static uint64_t
+index_entry_count(xlator_t *this, const char *subdir, const int subdir_len)
 {
     uint64_t count = 0;
     index_priv_t *priv = NULL;
@@ -1445,8 +1451,8 @@ index_entry_count(xlator_t *this, char *subdir)
 
     priv = this->private;
 
-    make_index_dir_path(priv->index_basepath, subdir, index_dir,
-                        sizeof(index_dir));
+    (void)make_index_dir_path(priv->index_basepath, subdir, index_dir,
+                              sizeof(index_dir));
 
     dirp = sys_opendir(index_dir);
     if (!dirp)
@@ -1461,7 +1467,7 @@ index_entry_count(xlator_t *this, char *subdir)
         if (strcmp(entry->d_name, ".") == 0 || strcmp(entry->d_name, "..") == 0)
             continue;
 
-        if (!strncmp(entry->d_name, subdir, strlen(subdir)))
+        if (!strncmp(entry->d_name, subdir, subdir_len))
             continue;
 
         count++;
@@ -1472,7 +1478,7 @@ index_entry_count(xlator_t *this, char *subdir)
     return count;
 }
 
-int32_t
+static int32_t
 index_getxattr_wrapper(call_frame_t *frame, xlator_t *this, loc_t *loc,
                        const char *name, dict_t *xdata)
 {
@@ -1507,7 +1513,7 @@ index_getxattr_wrapper(call_frame_t *frame, xlator_t *this, loc_t *loc,
     /* TODO: Need to check what kind of link-counts are needed for
      * ENTRY-CHANGES before refactor of this block with array*/
     if (strcmp(name, GF_XATTROP_INDEX_COUNT) == 0) {
-        count = index_entry_count(this, XATTROP_SUBDIR);
+        count = index_entry_count(this, XATTROP_SUBDIR, SLEN(XATTROP_SUBDIR));
 
         ret = dict_set_uint64(xattr, (char *)name, count);
         if (ret) {
@@ -1518,7 +1524,7 @@ index_getxattr_wrapper(call_frame_t *frame, xlator_t *this, loc_t *loc,
             goto done;
         }
     } else if (strcmp(name, GF_XATTROP_DIRTY_COUNT) == 0) {
-        count = index_entry_count(this, DIRTY_SUBDIR);
+        count = index_entry_count(this, DIRTY_SUBDIR, SLEN(DIRTY_SUBDIR));
 
         ret = dict_set_uint64(xattr, (char *)name, count);
         if (ret) {
@@ -1554,8 +1560,8 @@ index_save_pargfid_for_entry_changes(xlator_t *this, loc_t *loc, char *path)
     if (gf_uuid_compare(loc->pargfid, priv->internal_vgfid[ENTRY_CHANGES]))
         return 0;
 
-    ret = index_inode_ctx_get(loc->inode, this, &ctx);
-    if (ret) {
+    ctx = index_inode_ctx_get(loc->inode, this);
+    if (ctx == NULL) {
         gf_msg(this->name, GF_LOG_ERROR, EINVAL,
                INDEX_MSG_INODE_CTX_GET_SET_FAILED,
                "Unable to get inode context for %s", path);
@@ -1573,7 +1579,7 @@ index_save_pargfid_for_entry_changes(xlator_t *this, loc_t *loc, char *path)
     return 0;
 }
 
-int32_t
+static int32_t
 index_lookup_wrapper(call_frame_t *frame, xlator_t *this, loc_t *loc,
                      dict_t *xattr_req)
 {
@@ -1617,7 +1623,8 @@ index_lookup_wrapper(call_frame_t *frame, xlator_t *this, loc_t *loc,
 
     } else if (index_is_virtual_gfid(priv, loc->gfid)) {
         subdir = index_get_subdir_from_vgfid(priv, loc->gfid);
-        make_index_dir_path(priv->index_basepath, subdir, path, sizeof(path));
+        (void)make_index_dir_path(priv->index_basepath, subdir, path,
+                                  sizeof(path));
         is_dir = _gf_true;
 
         if ((xattr_req) && (dict_get(xattr_req, GF_INDEX_IA_TYPE_GET_REQ))) {
@@ -1689,7 +1696,7 @@ done:
     return 0;
 }
 
-int
+static int
 index_get_gfid_type(void *opaque)
 {
     gf_dirent_t *entry = NULL;
@@ -1701,7 +1708,8 @@ index_get_gfid_type(void *opaque)
 
     list_for_each_entry(entry, &args->entries->list, list)
     {
-        if (strcmp(entry->d_name, ".") == 0 || strcmp(entry->d_name, "..") == 0)
+        /* skip . and .. */
+        if (inode_dir_or_parentdir(entry))
             continue;
 
         loc_wipe(&loc);
@@ -1731,7 +1739,7 @@ index_get_gfid_type(void *opaque)
     return 0;
 }
 
-int32_t
+static int32_t
 index_readdir_wrapper(call_frame_t *frame, xlator_t *this, fd_t *fd,
                       size_t size, off_t off, dict_t *xdata)
 {
@@ -1782,7 +1790,7 @@ done:
     return 0;
 }
 
-int
+static int
 deletion_handler(const char *fpath, const struct stat *sb, int typeflag,
                  struct FTW *ftwbuf)
 {
@@ -1838,7 +1846,7 @@ index_get_parent_iatt(struct iatt *parent, char *path, loc_t *loc,
     return;
 }
 
-int
+static int
 index_rmdir_wrapper(call_frame_t *frame, xlator_t *this, loc_t *loc, int flag,
                     dict_t *xdata)
 {
@@ -1861,8 +1869,8 @@ index_rmdir_wrapper(call_frame_t *frame, xlator_t *this, loc_t *loc, int flag,
 
     type = index_get_type_from_vgfid(priv, loc->pargfid);
     subdir = index_get_subdir_from_vgfid(priv, loc->pargfid);
-    make_index_dir_path(priv->index_basepath, subdir, index_dir,
-                        sizeof(index_dir));
+    (void)make_index_dir_path(priv->index_basepath, subdir, index_dir,
+                              sizeof(index_dir));
 
     index_get_parent_iatt(&preparent, index_dir, loc, &op_ret, &op_errno);
     if (op_ret < 0)
@@ -1895,7 +1903,7 @@ done:
     return 0;
 }
 
-int
+static int
 index_unlink_wrapper(call_frame_t *frame, xlator_t *this, loc_t *loc, int flag,
                      dict_t *xdata)
 {
@@ -1926,13 +1934,16 @@ index_unlink_wrapper(call_frame_t *frame, xlator_t *this, loc_t *loc, int flag,
         goto done;
 
     if (type <= XATTROP_TYPE_UNSET) {
-        ret = index_inode_ctx_get(loc->parent, this, &ictx);
-        if ((ret == 0) && gf_uuid_is_null(ictx->virtual_pargfid)) {
-            ret = -EINVAL;
-        }
-        if (ret == 0) {
-            ret = index_entry_delete(this, ictx->virtual_pargfid,
-                                     (char *)loc->name);
+        ictx = index_inode_ctx_get(loc->parent, this);
+        if (ictx) {
+            if (gf_uuid_is_null(ictx->virtual_pargfid)) {
+                ret = -EINVAL;
+            } else {
+                ret = index_entry_delete(this, ictx->virtual_pargfid,
+                                         (char *)loc->name);
+            }
+        } else {  // ictx is NUILL
+            ret = -1;
         }
     } else if (type == ENTRY_CHANGES) {
         make_file_path(priv->index_basepath, ENTRY_CHANGES_SUBDIR,
@@ -2007,8 +2018,8 @@ index_fetch_link_count(xlator_t *this, index_xattrop_type_t type)
     };
 
     subdir = index_get_subdir_from_type(type);
-    make_index_dir_path(priv->index_basepath, subdir, index_dir,
-                        sizeof(index_dir));
+    (void)make_index_dir_path(priv->index_basepath, subdir, index_dir,
+                              sizeof(index_dir));
 
     dirp = sys_opendir(index_dir);
     if (!dirp)
@@ -2048,7 +2059,7 @@ out:
     return count;
 }
 
-dict_t *
+static dict_t *
 index_fill_link_count(xlator_t *this, dict_t *xdata)
 {
     int ret = -1;
@@ -2082,7 +2093,7 @@ out:
     return xdata;
 }
 
-int32_t
+static int32_t
 index_lookup_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
                  int32_t op_ret, int32_t op_errno, inode_t *inode,
                  struct iatt *buf, dict_t *xdata, struct iatt *postparent)
@@ -2138,7 +2149,7 @@ normal:
     return 0;
 }
 
-int32_t
+static int32_t
 index_fstat_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
                 int32_t op_ret, int32_t op_errno, struct iatt *buf,
                 dict_t *xdata)
@@ -2251,7 +2262,7 @@ out:
     return 0;
 }
 
-int
+static int
 index_make_xattrop_watchlist(xlator_t *this, index_priv_t *priv,
                              char *watchlist, index_xattrop_type_t type)
 {
@@ -2276,7 +2287,7 @@ index_make_xattrop_watchlist(xlator_t *this, index_priv_t *priv,
         goto out;
     }
 
-    dummy = int_to_data(1);
+    dummy = data_from_int32(1);
     if (!dummy) {
         ret = -1;
         goto out;
@@ -2573,14 +2584,12 @@ int32_t
 index_releasedir(xlator_t *this, fd_t *fd)
 {
     index_fd_ctx_t *fctx = NULL;
-    uint64_t ctx = 0;
     int ret = 0;
 
-    ret = fd_ctx_del(fd, this, &ctx);
-    if (ret < 0)
+    fctx = fd_ctx_del_ptr(fd, this);
+    if (!fctx)
         goto out;
 
-    fctx = (index_fd_ctx_t *)(long)ctx;
     if (fctx->dir) {
         ret = sys_closedir(fctx->dir);
         if (ret)
@@ -2597,16 +2606,12 @@ int32_t
 index_release(xlator_t *this, fd_t *fd)
 {
     index_fd_ctx_t *fctx = NULL;
-    uint64_t ctx = 0;
-    int ret = 0;
 
-    ret = fd_ctx_del(fd, this, &ctx);
-    if (ret < 0)
-        goto out;
+    fctx = fd_ctx_del_ptr(fd, this);
+    if (fctx) {
+        GF_FREE(fctx);
+    }
 
-    fctx = (index_fd_ctx_t *)(long)ctx;
-    GF_FREE(fctx);
-out:
     return 0;
 }
 
