@@ -33,7 +33,7 @@ __allocate_domain(const char *volume)
 {
     pl_dom_list_t *dom = NULL;
 
-    dom = GF_CALLOC(1, sizeof(*dom), gf_locks_mt_pl_dom_list_t);
+    dom = GF_MALLOC(sizeof(*dom), gf_locks_mt_pl_dom_list_t);
     if (!dom)
         goto out;
 
@@ -535,11 +535,11 @@ new_posix_lock(struct gf_flock *flock, client_t *client, pid_t client_pid,
     lock->fd_num = fd_to_fdnum(fd);
     lock->fd = fd;
     lock->client_pid = client_pid;
-    lock->owner = *owner;
+    lk_owner_copy(&lock->owner, owner);
     lock->lk_flags = lk_flags;
 
     lock->blocking = blocking;
-    memcpy(&lock->user_flock, flock, sizeof(lock->user_flock));
+    gf_flock_copy(&lock->user_flock, flock);
 
     INIT_LIST_HEAD(&lock->list);
 
@@ -569,6 +569,9 @@ __copy_lock(posix_lock_t *src)
 
     dst = GF_MALLOC(sizeof(posix_lock_t), gf_locks_mt_posix_lock_t);
     if (dst != NULL) {
+        /* TODO: replace this memcpy with copying individual
+         * variables, and then efficiently copy the owner and
+         * user_owner structures using lk_owner_copy(), gf_flock_copy()*/
         memcpy(dst, src, sizeof(posix_lock_t));
         dst->client_uid = gf_strdup(src->client_uid);
         if (dst->client_uid == NULL) {
@@ -590,7 +593,7 @@ posix_lock_to_flock(posix_lock_t *lock, struct gf_flock *flock)
     flock->l_pid = lock->user_flock.l_pid;
     flock->l_type = lock->fl_type;
     flock->l_start = lock->fl_start;
-    flock->l_owner = lock->owner;
+    lk_owner_copy(&flock->l_owner, &lock->owner);
 
     if (lock->fl_end == LLONG_MAX)
         flock->l_len = 0;
@@ -1136,7 +1139,7 @@ pl_getlk(pl_inode_t *pl_inode, posix_lock_t *lock)
 }
 
 gf_boolean_t
-pl_does_monkey_want_stuck_lock()
+pl_does_monkey_want_stuck_lock(void)
 {
     long int monkey_unlock_rand = 0;
     long int monkey_unlock_rand_rem = 0;
