@@ -8,11 +8,7 @@
    cases as published by the Free Software Foundation.
 */
 #include <glusterfs/dict.h>
-#include <glusterfs/xlator.h>
-#include <glusterfs/defaults.h>
-#include "libxlator.h"
-#include <glusterfs/common-utils.h>
-#include <glusterfs/byte-order.h>
+#include <glusterfs/compat-errno.h>
 #include "marker-quota.h"
 #include "marker-quota-helper.h"
 #include <glusterfs/syncop.h>
@@ -319,13 +315,13 @@ quota_dict_set_size_meta(xlator_t *this, dict_t *dict, const quota_meta_t *meta)
     if (value == NULL) {
         goto out;
     }
-    value[0].size = hton64(meta->size);
-    value[0].file_count = hton64(meta->file_count);
-    value[0].dir_count = hton64(meta->dir_count);
+    value[0].size = htobe64(meta->size);
+    value[0].file_count = htobe64(meta->file_count);
+    value[0].dir_count = htobe64(meta->dir_count);
 
     value[1].size = 0;
     value[1].file_count = 0;
-    value[1].dir_count = hton64(1);
+    value[1].dir_count = htobe64(1);
 
     GET_SIZE_KEY(this, size_key, ret);
     if (ret < 0)
@@ -384,9 +380,6 @@ mq_are_xattrs_set(xlator_t *this, loc_t *loc, gf_boolean_t *contri_set,
     quota_meta_t meta = {
         0,
     };
-    struct iatt stbuf = {
-        0,
-    };
     dict_t *dict = NULL;
     dict_t *rsp_dict = NULL;
 
@@ -400,7 +393,7 @@ mq_are_xattrs_set(xlator_t *this, loc_t *loc, gf_boolean_t *contri_set,
     if (ret < 0)
         goto out;
 
-    ret = syncop_lookup(FIRST_CHILD(this), loc, &stbuf, NULL, dict, &rsp_dict);
+    ret = syncop_lookup(FIRST_CHILD(this), loc, NULL, NULL, dict, &rsp_dict);
     if (ret < 0) {
         gf_log_callingfn(
             this->name,
@@ -531,9 +524,6 @@ mq_get_dirty(xlator_t *this, loc_t *loc, int32_t *dirty)
     int8_t value = 0;
     dict_t *dict = NULL;
     dict_t *rsp_dict = NULL;
-    struct iatt stbuf = {
-        0,
-    };
 
     dict = dict_new();
     if (dict == NULL) {
@@ -547,7 +537,7 @@ mq_get_dirty(xlator_t *this, loc_t *loc, int32_t *dirty)
         goto out;
     }
 
-    ret = syncop_lookup(FIRST_CHILD(this), loc, &stbuf, NULL, dict, &rsp_dict);
+    ret = syncop_lookup(FIRST_CHILD(this), loc, NULL, NULL, dict, &rsp_dict);
     if (ret < 0) {
         gf_log_callingfn(
             this->name,
@@ -1120,8 +1110,8 @@ mq_synctask1(xlator_t *this, synctask_fn_t task, gf_boolean_t spawn, loc_t *loc,
     }
 
     if (spawn) {
-        ret = synctask_new1(this->ctx->env, 1024 * 16, task,
-                            mq_synctask_cleanup, NULL, args);
+        ret = synctask_new1(this->ctx->env, 0, task, mq_synctask_cleanup, NULL,
+                            args);
         if (ret) {
             gf_log(this->name, GF_LOG_ERROR,
                    "Failed to spawn "
@@ -1913,8 +1903,8 @@ mq_update_dirty_inode_task(void *opaque)
         list_for_each_entry(entry, &entries.list, list)
         {
             offset = entry->d_off;
-
-            if (!strcmp(entry->d_name, ".") || !strcmp(entry->d_name, ".."))
+            /* skip . and .. */
+            if (inode_dir_or_parentdir(entry))
                 continue;
 
             memset(&contri, 0, sizeof(contri));
