@@ -7,15 +7,12 @@
    later), or the GNU General Public License, version 2 (GPLv2), in all
    cases as published by the Free Software Foundation.
 */
-#include <glusterfs/xlator.h>
-#include <glusterfs/defaults.h>
 #include "libxlator.h"
 #include "marker.h"
 #include "marker-mem-types.h"
 #include "marker-quota.h"
 #include "marker-quota-helper.h"
 #include "marker-common.h"
-#include <glusterfs/byte-order.h>
 #include <glusterfs/syncop.h>
 #include <glusterfs/syscall.h>
 
@@ -2887,7 +2884,7 @@ __has_quota_xattrs(dict_t *xattrs)
     return _gf_false;
 }
 
-int32_t
+static int32_t
 marker_lookup_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
                   int32_t op_ret, int32_t op_errno, inode_t *inode,
                   struct iatt *buf, dict_t *dict, struct iatt *postparent)
@@ -2966,19 +2963,25 @@ out:
     return 0;
 }
 
-int32_t
+static int32_t
 marker_lookup(call_frame_t *frame, xlator_t *this, loc_t *loc,
               dict_t *xattr_req)
 {
     int32_t ret = 0;
     marker_local_t *local = NULL;
     marker_conf_t *priv = NULL;
+    gf_boolean_t unref = _gf_false;  // do we need to unref the dict
 
     priv = this->private;
 
+    if (priv->version <= 0)
+        goto wind;
+
     xattr_req = xattr_req ? dict_ref(xattr_req) : dict_new();
-    if (!xattr_req)
+    if (!xattr_req) {
         goto err;
+    }
+    unref = _gf_true;
 
     ret = marker_key_replace_with_ver(this, xattr_req);
     if (ret < 0)
@@ -3004,7 +3007,8 @@ wind:
     STACK_WIND(frame, marker_lookup_cbk, FIRST_CHILD(this),
                FIRST_CHILD(this)->fops->lookup, loc, xattr_req);
 
-    dict_unref(xattr_req);
+    if (unref)
+        dict_unref(xattr_req);
 
     return 0;
 err:
@@ -3082,8 +3086,8 @@ marker_readdirp_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
 
     list_for_each_entry(entry, &entries->list, list)
     {
-        if ((strcmp(entry->d_name, ".") == 0) ||
-            (strcmp(entry->d_name, "..") == 0) || entry->inode == NULL)
+        /* skip . and .. */
+        if (entry->inode == NULL || inode_dir_or_parentdir(entry))
             continue;
 
         if (no_found_entry)
