@@ -122,7 +122,7 @@ shard_unlock_entrylk(call_frame_t *frame, xlator_t *this);
         __bs = GF_MALLOC(sizeof(uint64_t), gf_shard_mt_uint64_t);              \
         if (!__bs)                                                             \
             goto label;                                                        \
-        *__bs = hton64(block_size);                                            \
+        *__bs = htobe64(block_size);                                           \
         __ret = dict_set_bin(xattr_req, GF_XATTR_SHARD_BLOCK_SIZE, __bs,       \
                              sizeof(*__bs));                                   \
         if (__ret) {                                                           \
@@ -207,6 +207,16 @@ typedef enum {
 
 /* rm = "remove me" */
 
+typedef struct shard_unlink_thread {
+    pthread_mutex_t mutex;
+    pthread_cond_t cond;
+    pthread_t thread;
+    xlator_t *this;
+    gf_boolean_t running;
+    gf_boolean_t rerun;
+    gf_boolean_t stop;
+} shard_unlink_thread_t;
+
 typedef struct shard_priv {
     uint64_t block_size;
     uuid_t dot_shard_gfid;
@@ -214,12 +224,13 @@ typedef struct shard_priv {
     inode_t *dot_shard_inode;
     inode_t *dot_shard_rm_inode;
     gf_lock_t lock;
-    int inode_count;
     struct list_head ilist_head;
+    int inode_count;
     uint32_t deletion_rate;
     shard_bg_deletion_state_t bg_del_state;
     gf_boolean_t first_lookup_done;
     uint64_t lru_limit;
+    shard_unlink_thread_t thread_info;
 } shard_priv_t;
 
 typedef struct {
@@ -254,9 +265,9 @@ typedef int32_t (*shard_post_update_size_fop_handler_t)(call_frame_t *frame,
 typedef struct shard_local {
     int op_ret;
     int op_errno;
-    int first_block;
-    int last_block;
-    int num_blocks;
+    uint64_t first_block;
+    uint64_t last_block;
+    uint64_t num_blocks;
     int call_count;
     int eexist_count;
     int create_count;
@@ -268,6 +279,7 @@ typedef struct shard_local {
     uint64_t block_size;
     uint64_t dst_block_size;
     int32_t datasync;
+    glusterfs_fop_t fop;
     off_t offset;
     size_t total_size;
     size_t written_size;
@@ -285,7 +297,6 @@ typedef struct shard_local {
     dict_t *xattr_req;
     dict_t *xattr_rsp;
     inode_t **inode_list;
-    glusterfs_fop_t fop;
     struct iatt prebuf;
     struct iatt postbuf;
     struct iatt preoldparent;
@@ -298,6 +309,11 @@ typedef struct shard_local {
     gf_dirent_t entries_head;
     gf_boolean_t is_set_fsid;
     gf_boolean_t list_inited;
+    gf_boolean_t first_lookup_done;
+    gf_boolean_t lookup_shards_barriered;
+    gf_boolean_t unlink_shards_barriered;
+    gf_boolean_t resolve_not;
+    gf_boolean_t cleanup_required;
     shard_post_fop_handler_t handler;
     shard_post_lookup_shards_fop_handler_t pls_fop_handler;
     shard_post_resolve_fop_handler_t post_res_handler;
@@ -306,37 +322,32 @@ typedef struct shard_local {
     shard_inodelk_t int_inodelk;
     shard_entrylk_t int_entrylk;
     inode_t *resolver_base_inode;
-    gf_boolean_t first_lookup_done;
     syncbarrier_t barrier;
-    gf_boolean_t lookup_shards_barriered;
-    gf_boolean_t unlink_shards_barriered;
-    gf_boolean_t resolve_not;
     loc_t newloc;
     call_frame_t *main_frame;
     call_frame_t *inodelk_frame;
     call_frame_t *entrylk_frame;
-    uint32_t deletion_rate;
-    gf_boolean_t cleanup_required;
-    uuid_t base_gfid;
     char *name;
+    uint32_t deletion_rate;
+    uuid_t base_gfid;
 } shard_local_t;
 
 typedef struct shard_inode_ctx {
     uint64_t block_size; /* The block size with which this inode is
                             sharded */
     struct iatt stat;
-    gf_boolean_t refresh;
     /* The following members of inode ctx will be applicable only to the
      * individual shards' ctx and never the base file ctx.
      */
     struct list_head ilist;
     uuid_t base_gfid;
     int block_num;
+    gf_boolean_t refresh;
     gf_boolean_t refreshed;
     struct list_head to_fsync_list;
     int fsync_needed;
-    inode_t *inode;
     int fsync_count;
+    inode_t *inode;
     inode_t *base_inode;
 } shard_inode_ctx_t;
 
