@@ -8,13 +8,12 @@
    cases as published by the Free Software Foundation.
 */
 
-#include "xlator.h"
-#include "defaults.h"
+#include <glusterfs/defaults.h>
 
 #include "meta-mem-types.h"
 #include "meta.h"
 
-#include "compat-errno.h"
+#include <glusterfs/compat-errno.h>
 
 int
 meta_default_fgetxattr(call_frame_t *frame, xlator_t *this, fd_t *fd,
@@ -64,7 +63,7 @@ meta_default_fstat(call_frame_t *frame, xlator_t *this, fd_t *fd, dict_t *xdata)
 {
     struct iatt iatt = {};
 
-    meta_iatt_fill(&iatt, fd->inode, fd->inode->ia_type);
+    meta_iatt_fill(this, &iatt, fd->inode, fd->inode->ia_type);
 
     META_STACK_UNWIND(fstat, frame, 0, 0, &iatt, xdata);
 
@@ -119,7 +118,7 @@ meta_default_readv(call_frame_t *frame, xlator_t *this, fd_t *fd, size_t size,
     struct iobuf *iobuf = NULL;
     struct iobref *iobref = NULL;
     off_t copy_offset = 0;
-    size_t copy_size = 0;
+    int copy_size = 0;
     struct iatt iatt = {};
 
     meta_fd = meta_fd_get(fd, this);
@@ -127,7 +126,7 @@ meta_default_readv(call_frame_t *frame, xlator_t *this, fd_t *fd, size_t size,
         return default_readv_failure_cbk(frame, ENODATA);
 
     if (!meta_fd->size)
-        meta_file_fill(this, fd);
+        meta_file_fill(this, meta_fd, fd);
 
     iobuf = iobuf_get2(this->ctx->iobuf_pool, size);
     if (!iobuf)
@@ -145,11 +144,11 @@ meta_default_readv(call_frame_t *frame, xlator_t *this, fd_t *fd, size_t size,
         return default_readv_failure_cbk(frame, ENOMEM);
     }
 
+    iov.iov_base = iobuf_ptr(iobuf);
+
     /* iobref would have taken a ref */
     iobuf_unref(iobuf);
 
-    iov.iov_base = iobuf_ptr(iobuf);
-
     copy_offset = min(meta_fd->size, offset);
     copy_size = min(size, (meta_fd->size - copy_offset));
 
@@ -241,9 +240,10 @@ meta_default_readlink(call_frame_t *frame, xlator_t *this, loc_t *loc,
     struct meta_ops *ops = NULL;
     strfd_t *strfd = NULL;
     struct iatt iatt = {};
+    int len = -1;
 
     ops = meta_ops_get(loc->inode, this);
-    if (!ops->link_fill) {
+    if (!ops || !ops->link_fill) {
         META_STACK_UNWIND(readlink, frame, -1, EPERM, 0, 0, 0);
         return 0;
     }
@@ -256,12 +256,12 @@ meta_default_readlink(call_frame_t *frame, xlator_t *this, loc_t *loc,
 
     ops->link_fill(this, loc->inode, strfd);
 
-    meta_iatt_fill(&iatt, loc->inode, IA_IFLNK);
+    meta_iatt_fill(this, &iatt, loc->inode, IA_IFLNK);
 
-    if (strfd->data)
-        META_STACK_UNWIND(readlink, frame, strlen(strfd->data), 0, strfd->data,
-                          &iatt, xdata);
-    else
+    if (strfd->data) {
+        len = strlen(strfd->data);
+        META_STACK_UNWIND(readlink, frame, len, 0, strfd->data, &iatt, xdata);
+    } else
         META_STACK_UNWIND(readlink, frame, -1, ENODATA, 0, 0, 0);
 
     strfd_close(strfd);
@@ -282,7 +282,7 @@ meta_default_ftruncate(call_frame_t *frame, xlator_t *this, fd_t *fd,
 {
     struct iatt iatt = {};
 
-    meta_iatt_fill(&iatt, fd->inode, IA_IFREG);
+    meta_iatt_fill(this, &iatt, fd->inode, IA_IFREG);
 
     META_STACK_UNWIND(ftruncate, frame, 0, 0, &iatt, &iatt, xdata);
 
@@ -388,6 +388,7 @@ meta_default_readdir(call_frame_t *frame, xlator_t *this, fd_t *fd, size_t size,
     struct meta_dirent *dirents = NULL;
     struct meta_dirent *end = NULL;
     struct meta_ops *ops = NULL;
+    size_t dirents_name_len;
 
     INIT_LIST_HEAD(&head.list);
 
@@ -399,7 +400,7 @@ meta_default_readdir(call_frame_t *frame, xlator_t *this, fd_t *fd, size_t size,
     if (!meta_fd)
         goto err;
 
-    meta_dir_fill(this, fd);
+    meta_dir_fill(this, meta_fd, ops, fd);
 
     fixed_dirents = ops->fixed_dirents;
     fixed_size = fixed_dirents_len(fixed_dirents);
@@ -417,43 +418,17 @@ meta_default_readdir(call_frame_t *frame, xlator_t *this, fd_t *fd, size_t size,
         }
 
         while (dirents < end) {
-            this_size = sizeof(gf_dirent_t) + strlen(dirents->name) + 1;
+            dirents_name_len = strlen(dirents->name);
+            this_size = gf_dirent_len(dirents_name_len);
             if (this_size + filled_size > size)
                 goto unwind;
 
-            list = gf_dirent_for_name(dirents->name);
+            list = gf_dirent_for_name2(
+                dirents->name, dirents_name_len, i + 42, i + 1,
+                gf_d_type_from_ia_type(dirents->type), NULL);
             if (!list)
                 break;
 
-            list->d_off = i + 1;
-            list->d_ino = i + 42;
-            switch (dirents->type) {
-                case IA_IFDIR:
-                    list->d_type = DT_DIR;
-                    break;
-                case IA_IFCHR:
-                    list->d_type = DT_CHR;
-                    break;
-                case IA_IFBLK:
-                    list->d_type = DT_BLK;
-                    break;
-                case IA_IFIFO:
-                    list->d_type = DT_FIFO;
-                    break;
-                case IA_IFLNK:
-                    list->d_type = DT_LNK;
-                    break;
-                case IA_IFREG:
-                    list->d_type = DT_REG;
-                    break;
-                case IA_IFSOCK:
-                    list->d_type = DT_SOCK;
-                    break;
-                case IA_INVAL:
-                    list->d_type = DT_UNKNOWN;
-                    break;
-            }
-
             list_add_tail(&list->list, &head.list);
             ret++;
             i++;
@@ -493,7 +468,7 @@ meta_default_truncate(call_frame_t *frame, xlator_t *this, loc_t *loc,
 {
     struct iatt iatt = {};
 
-    meta_iatt_fill(&iatt, loc->inode, IA_IFREG);
+    meta_iatt_fill(this, &iatt, loc->inode, IA_IFREG);
 
     META_STACK_UNWIND(truncate, frame, 0, 0, &iatt, &iatt, xdata);
 
@@ -506,7 +481,7 @@ meta_default_stat(call_frame_t *frame, xlator_t *this, loc_t *loc,
 {
     struct iatt iatt = {};
 
-    meta_iatt_fill(&iatt, loc->inode, loc->inode->ia_type);
+    meta_iatt_fill(this, &iatt, loc->inode, loc->inode->ia_type);
 
     META_STACK_UNWIND(stat, frame, 0, 0, &iatt, xdata);
 
@@ -552,7 +527,7 @@ hook:
 
         dirent->hook(frame, this, loc, xdata);
 
-        meta_iatt_fill(&iatt, loc->inode, dirent->type);
+        meta_iatt_fill(this, &iatt, loc->inode, dirent->type);
 
         META_STACK_UNWIND(lookup, frame, 0, 0, loc->inode, &iatt, xdata,
                           &parent);
