@@ -8,16 +8,12 @@
    cases as published by the Free Software Foundation.
 */
 
-#include <glusterfs/glusterfs.h>
 #include <glusterfs/dict.h>
-#include <glusterfs/xlator.h>
 #include <glusterfs/logging.h>
 #include <glusterfs/run.h>
-#include <glusterfs/defaults.h>
 #include <glusterfs/syscall.h>
 #include <glusterfs/compat.h>
 #include <glusterfs/compat-errno.h>
-#include "glusterd.h"
 #include "glusterd-sm.h"
 #include "glusterd-op-sm.h"
 #include "glusterd-utils.h"
@@ -249,8 +245,8 @@ glusterd_hooks_set_volume_args(dict_t *dict, runner_t *runner)
 
     glusterd_hooks_add_custom_args(dict, runner);
     if (flag == 1) {
-        ret = dict_get_str_sizen(this->options, "transport.address-family",
-                                 &inet_family);
+        ret = dict_get_str(this->options, "transport.address-family",
+                           &inet_family);
         if (!ret) {
             runner_argprintf(runner, "transport.address-family=%s",
                              inet_family);
