@@ -7,10 +7,6 @@
    later), or the GNU General Public License, version 2 (GPLv2), in all
    cases as published by the Free Software Foundation.
 */
-#include <glusterfs/common-utils.h>
-#include "cli1-xdr.h"
-#include "xdr-generic.h"
-#include "glusterd.h"
 #include "glusterd-op-sm.h"
 #include "glusterd-store.h"
 #include "glusterd-utils.h"
@@ -55,15 +51,12 @@ glusterd_mgmt_v3_is_type_valid(char *type)
 /* Initialize the global mgmt_v3 lock list(dict) when
  * glusterd is spawned */
 int32_t
-glusterd_mgmt_v3_lock_init()
+glusterd_mgmt_v3_lock_init(void)
 {
     int32_t ret = -1;
-    xlator_t *this = NULL;
     glusterd_conf_t *priv = NULL;
 
-    this = THIS;
-    GF_ASSERT(this);
-    priv = this->private;
+    priv = THIS->private;
     GF_ASSERT(priv);
 
     priv->mgmt_v3_lock = dict_new();
@@ -78,14 +71,11 @@ out:
 /* Destroy the global mgmt_v3 lock list(dict) when
  * glusterd cleanup is performed */
 void
-glusterd_mgmt_v3_lock_fini()
+glusterd_mgmt_v3_lock_fini(void)
 {
-    xlator_t *this = NULL;
     glusterd_conf_t *priv = NULL;
 
-    this = THIS;
-    GF_ASSERT(this);
-    priv = this->private;
+    priv = THIS->private;
     GF_ASSERT(priv);
 
     if (priv->mgmt_v3_lock)
@@ -95,15 +85,12 @@ glusterd_mgmt_v3_lock_fini()
 /* Initialize the global mgmt_v3_timer lock list(dict) when
  * glusterd is spawned */
 int32_t
-glusterd_mgmt_v3_lock_timer_init()
+glusterd_mgmt_v3_lock_timer_init(void)
 {
     int32_t ret = -1;
-    xlator_t *this = NULL;
+    xlator_t *this = THIS;
     glusterd_conf_t *priv = NULL;
 
-    this = THIS;
-    GF_VALIDATE_OR_GOTO("glusterd", this, out);
-
     priv = this->private;
     GF_VALIDATE_OR_GOTO(this->name, priv, out);
 
@@ -119,14 +106,11 @@ out:
 /* Destroy the global mgmt_v3_timer lock list(dict) when
  * glusterd cleanup is performed */
 void
-glusterd_mgmt_v3_lock_timer_fini()
+glusterd_mgmt_v3_lock_timer_fini(void)
 {
-    xlator_t *this = NULL;
+    xlator_t *this = THIS;
     glusterd_conf_t *priv = NULL;
 
-    this = THIS;
-    GF_VALIDATE_OR_GOTO("glusterd", this, out);
-
     priv = this->private;
     GF_VALIDATE_OR_GOTO(this->name, priv, out);
 
@@ -142,10 +126,8 @@ glusterd_get_mgmt_v3_lock_owner(char *key, uuid_t *uuid)
     int32_t ret = -1;
     glusterd_mgmt_v3_lock_obj *lock_obj = NULL;
     glusterd_conf_t *priv = NULL;
-    xlator_t *this = NULL;
+    xlator_t *this = THIS;
 
-    this = THIS;
-    GF_ASSERT(this);
     priv = this->private;
     GF_ASSERT(priv);
 
@@ -177,10 +159,8 @@ glusterd_release_multiple_locks_per_entity(dict_t *dict, uuid_t uuid,
     int32_t i = -1;
     int32_t op_ret = 0;
     int32_t ret = -1;
-    xlator_t *this = NULL;
+    xlator_t *this = THIS;
 
-    this = THIS;
-    GF_ASSERT(this);
     GF_ASSERT(dict);
     GF_ASSERT(type);
 
@@ -232,10 +212,8 @@ glusterd_acquire_multiple_locks_per_entity(dict_t *dict, uuid_t uuid,
     int32_t i = -1;
     int32_t ret = -1;
     int32_t locked_count = 0;
-    xlator_t *this = NULL;
+    xlator_t *this = THIS;
 
-    this = THIS;
-    GF_ASSERT(this);
     GF_ASSERT(dict);
     GF_ASSERT(type);
 
@@ -296,10 +274,8 @@ glusterd_mgmt_v3_unlock_entity(dict_t *dict, uuid_t uuid, char *type,
     int32_t count = -1;
     int32_t ret = -1;
     gf_boolean_t hold_locks = _gf_false;
-    xlator_t *this = NULL;
+    xlator_t *this = THIS;
 
-    this = THIS;
-    GF_ASSERT(this);
     GF_ASSERT(dict);
     GF_ASSERT(type);
 
@@ -366,10 +342,8 @@ glusterd_mgmt_v3_lock_entity(dict_t *dict, uuid_t uuid, uint32_t *op_errno,
     int32_t count = -1;
     int32_t ret = -1;
     gf_boolean_t hold_locks = _gf_false;
-    xlator_t *this = NULL;
+    xlator_t *this = THIS;
 
-    this = THIS;
-    GF_ASSERT(this);
     GF_ASSERT(dict);
     GF_ASSERT(type);
 
@@ -430,10 +404,7 @@ glusterd_multiple_mgmt_v3_unlock(dict_t *dict, uuid_t uuid)
     int32_t i = -1;
     int32_t ret = -1;
     int32_t op_ret = 0;
-    xlator_t *this = NULL;
-
-    this = THIS;
-    GF_ASSERT(this);
+    xlator_t *this = THIS;
 
     if (!dict) {
         gf_msg(this->name, GF_LOG_ERROR, 0, GD_MSG_DICT_EMPTY, "dict is null.");
@@ -466,10 +437,7 @@ glusterd_multiple_mgmt_v3_lock(dict_t *dict, uuid_t uuid, uint32_t *op_errno)
     int32_t i = -1;
     int32_t ret = -1;
     int32_t locked_count = 0;
-    xlator_t *this = NULL;
-
-    this = THIS;
-    GF_ASSERT(this);
+    xlator_t *this = THIS;
 
     if (!dict) {
         gf_msg(this->name, GF_LOG_ERROR, 0, GD_MSG_DICT_EMPTY, "dict is null.");
@@ -520,19 +488,13 @@ glusterd_mgmt_v3_lock(const char *name, uuid_t uuid, uint32_t *op_errno,
     char key[PATH_MAX] = "";
     int32_t ret = -1;
     glusterd_mgmt_v3_lock_obj *lock_obj = NULL;
-    glusterd_mgmt_v3_lock_timer *mgmt_lock_timer = NULL;
+    gf_timer_t *mgmt_lock_timer = NULL;
     glusterd_conf_t *priv = NULL;
     gf_boolean_t is_valid = _gf_true;
     uuid_t owner = {0};
-    xlator_t *this = NULL;
-    char *bt = NULL;
+    xlator_t *this = THIS;
     struct timespec delay = {0};
     char *key_dup = NULL;
-    glusterfs_ctx_t *mgmt_lock_timer_ctx = NULL;
-    xlator_t *mgmt_lock_timer_xl = NULL;
-
-    this = THIS;
-    GF_ASSERT(this);
 
     priv = this->private;
     GF_ASSERT(priv);
@@ -600,40 +562,25 @@ glusterd_mgmt_v3_lock(const char *name, uuid_t uuid, uint32_t *op_errno,
         goto out;
     }
 
-    mgmt_lock_timer = GF_CALLOC(1, sizeof(glusterd_mgmt_v3_lock_timer),
-                                gf_common_mt_mgmt_v3_lock_timer_t);
-
-    if (!mgmt_lock_timer) {
-        ret = -1;
-        goto out;
-    }
-
-    mgmt_lock_timer->xl = THIS;
-    /*changing to default timeout value*/
-    priv->mgmt_v3_lock_timeout = GF_LOCK_TIMER;
-
-    ret = -1;
-    mgmt_lock_timer_xl = mgmt_lock_timer->xl;
-    if (!mgmt_lock_timer_xl) {
-        GF_FREE(mgmt_lock_timer);
-        goto out;
-    }
-
-    mgmt_lock_timer_ctx = mgmt_lock_timer_xl->ctx;
-    if (!mgmt_lock_timer_ctx) {
-        GF_FREE(mgmt_lock_timer);
-        goto out;
-    }
-
     key_dup = gf_strdup(key);
     delay.tv_sec = priv->mgmt_v3_lock_timeout;
     delay.tv_nsec = 0;
 
-    mgmt_lock_timer->timer = gf_timer_call_after(
-        mgmt_lock_timer_ctx, delay, gd_mgmt_v3_unlock_timer_cbk, key_dup);
+    /* Changing to default timeout value. */
+    priv->mgmt_v3_lock_timeout = GF_LOCK_TIMER;
 
-    ret = dict_set_bin(priv->mgmt_v3_lock_timer, key, mgmt_lock_timer,
-                       sizeof(glusterd_mgmt_v3_lock_timer));
+    mgmt_lock_timer = gf_timer_call_after(this->ctx, delay,
+                                          gd_mgmt_v3_unlock_timer_cbk, key_dup);
+
+    /* Timer object is allocated dynamically but freed with
+       gf_timer_call_cancel(). So it should not be managed by
+       dict to avoid double free and treated as static here.
+       Also note that just the pointer to timer is stored in
+       the dict, not the timer object as a whole (the latter
+       causes copying and most likely a crash in timer code). */
+    /* coverity[SUSPICIOUS_SIZEOF] */
+    ret = dict_set_static_bin(priv->mgmt_v3_lock_timer, key, mgmt_lock_timer,
+                              sizeof(mgmt_lock_timer));
     if (ret) {
         gf_msg(this->name, GF_LOG_ERROR, 0, GD_MSG_DICT_SET_FAILED,
                "Unable to set timer in mgmt_v3 lock");
@@ -642,6 +589,9 @@ glusterd_mgmt_v3_lock(const char *name, uuid_t uuid, uint32_t *op_errno,
         goto out;
     }
 
+#ifdef DEBUG
+    char *bt = NULL;
+
     /* Saving the backtrace into the pre-allocated buffer, ctx->btbuf*/
     if ((bt = gf_backtrace_save(NULL))) {
         snprintf(key, sizeof(key), "debug.last-success-bt-%s", key_dup);
@@ -654,6 +604,8 @@ glusterd_mgmt_v3_lock(const char *name, uuid_t uuid, uint32_t *op_errno,
         ret = 0;
     }
 
+#endif
+
     gf_msg_debug(this->name, 0, "Lock for %s successfully held by %s", key_dup,
                  uuid_utoa(uuid));
 
@@ -669,20 +621,12 @@ out:
 void
 gd_mgmt_v3_unlock_timer_cbk(void *data)
 {
-    xlator_t *this = NULL;
+    xlator_t *this = THIS;
     glusterd_conf_t *conf = NULL;
-    glusterd_mgmt_v3_lock_timer *mgmt_lock_timer = NULL;
+    gf_timer_t *mgmt_lock_timer = NULL;
     char *key = NULL;
     int keylen;
-    char bt_key[PATH_MAX] = "";
-    int bt_key_len = 0;
     int32_t ret = -1;
-    glusterfs_ctx_t *mgmt_lock_timer_ctx = NULL;
-    xlator_t *mgmt_lock_timer_xl = NULL;
-    gf_timer_t *timer = NULL;
-
-    this = THIS;
-    GF_VALIDATE_OR_GOTO("glusterd", this, out);
 
     conf = this->private;
     GF_VALIDATE_OR_GOTO(this->name, conf, out);
@@ -693,6 +637,10 @@ gd_mgmt_v3_unlock_timer_cbk(void *data)
     keylen = strlen(key);
     dict_deln(conf->mgmt_v3_lock, key, keylen);
 
+#ifdef DEBUG
+    char bt_key[PATH_MAX] = "";
+    int bt_key_len = 0;
+
     bt_key_len = snprintf(bt_key, PATH_MAX, "debug.last-success-bt-%s", key);
     if (bt_key_len != SLEN("debug.last-success-bt-") + keylen) {
         gf_msg(this->name, GF_LOG_ERROR, 0, GD_MSG_CREATE_KEY_FAIL,
@@ -702,6 +650,7 @@ gd_mgmt_v3_unlock_timer_cbk(void *data)
     }
 
     dict_deln(conf->mgmt_v3_lock, bt_key, bt_key_len);
+#endif
 
     ret = dict_get_bin(conf->mgmt_v3_lock_timer, key,
                        (void **)&mgmt_lock_timer);
@@ -711,27 +660,18 @@ gd_mgmt_v3_unlock_timer_cbk(void *data)
     }
 
 out:
-    if (mgmt_lock_timer && mgmt_lock_timer->timer) {
-        mgmt_lock_timer_xl = mgmt_lock_timer->xl;
-        GF_VALIDATE_OR_GOTO(this->name, mgmt_lock_timer_xl, ret_function);
-
-        mgmt_lock_timer_ctx = mgmt_lock_timer_xl->ctx;
-        GF_VALIDATE_OR_GOTO(this->name, mgmt_lock_timer_ctx, ret_function);
-
-        timer = mgmt_lock_timer->timer;
-        GF_FREE(timer->data);
-        gf_timer_call_cancel(mgmt_lock_timer_ctx, mgmt_lock_timer->timer);
+    if (mgmt_lock_timer) {
+        GF_ASSERT(mgmt_lock_timer->xl && mgmt_lock_timer->xl->ctx);
+        GF_FREE(mgmt_lock_timer->data);
+        gf_timer_call_cancel(mgmt_lock_timer->xl->ctx, mgmt_lock_timer);
+#ifdef DEBUG
         dict_deln(conf->mgmt_v3_lock_timer, bt_key, bt_key_len);
-        mgmt_lock_timer->timer = NULL;
+#endif
         gf_log(this->name, GF_LOG_INFO,
                "unlock timer is cancelled for volume_type"
                " %s",
                key);
     }
-
-ret_function:
-
-    return;
 }
 
 int32_t
@@ -744,15 +684,9 @@ glusterd_mgmt_v3_unlock(const char *name, uuid_t uuid, char *type)
     gf_boolean_t is_valid = _gf_true;
     glusterd_conf_t *priv = NULL;
     glusterd_volinfo_t *volinfo = NULL;
-    glusterd_mgmt_v3_lock_timer *mgmt_lock_timer = NULL;
+    gf_timer_t *mgmt_lock_timer = NULL;
     uuid_t owner = {0};
-    xlator_t *this = NULL;
-    glusterfs_ctx_t *mgmt_lock_timer_ctx = NULL;
-    xlator_t *mgmt_lock_timer_xl = NULL;
-    gf_timer_t *timer = NULL;
-
-    this = THIS;
-    GF_ASSERT(this);
+    xlator_t *this = THIS;
 
     priv = this->private;
     GF_ASSERT(priv);
@@ -821,6 +755,7 @@ glusterd_mgmt_v3_unlock(const char *name, uuid_t uuid, char *type)
 
     (void)snprintf(key_dup, sizeof(key_dup), "%s", key);
 
+#ifdef DEBUG
     /* Remove the backtrace key as well */
     ret = snprintf(key, sizeof(key), "debug.last-success-bt-%s", key_dup);
     if (ret != SLEN("debug.last-success-bt-") + keylen) {
@@ -831,23 +766,16 @@ glusterd_mgmt_v3_unlock(const char *name, uuid_t uuid, char *type)
         goto out;
     }
     dict_deln(priv->mgmt_v3_lock, key, ret);
+#endif
 
     gf_msg_debug(this->name, 0, "Lock for %s %s successfully released", type,
                  name);
 
-    /* Release owner reference which was held during lock */
-    if (mgmt_lock_timer && mgmt_lock_timer->timer) {
-        ret = -1;
-        mgmt_lock_timer_xl = mgmt_lock_timer->xl;
-        GF_VALIDATE_OR_GOTO(this->name, mgmt_lock_timer_xl, out);
-
-        mgmt_lock_timer_ctx = mgmt_lock_timer_xl->ctx;
-        GF_VALIDATE_OR_GOTO(this->name, mgmt_lock_timer_ctx, out);
-        ret = 0;
-
-        timer = mgmt_lock_timer->timer;
-        GF_FREE(timer->data);
-        gf_timer_call_cancel(mgmt_lock_timer_ctx, mgmt_lock_timer->timer);
+    /* Release owner reference which was held during lock. */
+    if (mgmt_lock_timer) {
+        GF_ASSERT(mgmt_lock_timer->xl && mgmt_lock_timer->xl->ctx);
+        GF_FREE(mgmt_lock_timer->data);
+        gf_timer_call_cancel(mgmt_lock_timer->xl->ctx, mgmt_lock_timer);
         dict_deln(priv->mgmt_v3_lock_timer, key_dup, keylen);
     }
     ret = glusterd_volinfo_find(name, &volinfo);
