@@ -8,13 +8,8 @@
    cases as published by the Free Software Foundation.
 */
 /* rpc related syncops */
-#include "rpc-clnt.h"
-#include "protocol-common.h"
-#include "xdr-generic.h"
-#include "glusterd1-xdr.h"
 #include "glusterd-syncop.h"
 
-#include "glusterd.h"
 #include "glusterd-utils.h"
 #include "glusterd-locks.h"
 #include "glusterd-mgmt.h"
@@ -414,7 +409,7 @@ gd_mgmt_v3_post_commit_fn(glusterd_op_t op, dict_t *dict, char **op_errstr,
 
     switch (op) {
         case GD_OP_ADD_BRICK:
-            ret = glusterd_post_commit_add_brick(dict, op_errstr);
+            ret = glusterd_post_commit_brick_operation(dict, op_errstr);
             if (ret) {
                 gf_msg(this->name, GF_LOG_ERROR, 0, GD_MSG_POST_COMMIT_OP_FAIL,
                        "Add-brick post commit failed.");
@@ -422,7 +417,7 @@ gd_mgmt_v3_post_commit_fn(glusterd_op_t op, dict_t *dict, char **op_errstr,
             }
             break;
         case GD_OP_REPLACE_BRICK:
-            ret = glusterd_post_commit_replace_brick(dict, op_errstr);
+            ret = glusterd_post_commit_brick_operation(dict, op_errstr);
             if (ret) {
                 gf_msg(this->name, GF_LOG_ERROR, 0, GD_MSG_POST_COMMIT_OP_FAIL,
                        "Replace-brick post commit failed.");
@@ -468,7 +463,7 @@ gd_mgmt_v3_post_validate_fn(glusterd_op_t op, int32_t op_ret, dict_t *dict,
             break;
         }
         case GD_OP_ADD_BRICK: {
-            ret = dict_get_strn(dict, "volname", SLEN("volname"), &volname);
+            ret = dict_get_str(dict, "volname", &volname);
             if (ret) {
                 gf_msg("glusterd", GF_LOG_ERROR, 0, GD_MSG_DICT_GET_FAILED,
                        "Unable to get"
@@ -493,7 +488,7 @@ gd_mgmt_v3_post_validate_fn(glusterd_op_t op, int32_t op_ret, dict_t *dict,
             break;
         }
         case GD_OP_START_VOLUME: {
-            ret = dict_get_strn(dict, "volname", SLEN("volname"), &volname);
+            ret = dict_get_str(dict, "volname", &volname);
             if (ret) {
                 gf_msg("glusterd", GF_LOG_ERROR, 0, GD_MSG_DICT_GET_FAILED,
                        "Unable to get"
@@ -512,7 +507,7 @@ gd_mgmt_v3_post_validate_fn(glusterd_op_t op, int32_t op_ret, dict_t *dict,
             break;
         }
         case GD_OP_STOP_VOLUME: {
-            ret = dict_get_strn(dict, "volname", SLEN("volname"), &volname);
+            ret = dict_get_str(dict, "volname", &volname);
             if (ret) {
                 gf_msg("glusterd", GF_LOG_ERROR, 0, GD_MSG_DICT_GET_FAILED,
                        "Unable to get"
@@ -715,7 +710,7 @@ glusterd_mgmt_v3_initiate_lockdown(glusterd_op_t op, dict_t *dict,
         if (!peerinfo->connected)
             continue;
         if (op != GD_OP_SYNC_VOLUME &&
-            peerinfo->state.state != GD_FRIEND_STATE_BEFRIENDED)
+            peerinfo->state != GD_FRIEND_STATE_BEFRIENDED)
             continue;
 
         gd_mgmt_v3_lock(op, dict, peerinfo, &args, MY_UUID, peer_uuid);
@@ -990,13 +985,6 @@ glusterd_mgmt_v3_pre_validate(glusterd_op_t op, dict_t *req_dict,
     GF_ASSERT(op_errstr);
     GF_VALIDATE_OR_GOTO(this->name, op_errno, out);
 
-    rsp_dict = dict_new();
-    if (!rsp_dict) {
-        gf_msg(this->name, GF_LOG_ERROR, 0, GD_MSG_DICT_CREATE_FAIL,
-               "Failed to create response dictionary");
-        goto out;
-    }
-
     if (op == GD_OP_PROFILE_VOLUME || op == GD_OP_STOP_VOLUME ||
         op == GD_OP_REBALANCE || op == GD_OP_REMOVE_BRICK) {
         ret = glusterd_validate_quorum(this, op, req_dict, op_errstr);
@@ -1007,6 +995,13 @@ glusterd_mgmt_v3_pre_validate(glusterd_op_t op, dict_t *req_dict,
         }
     }
 
+    rsp_dict = dict_new();
+    if (!rsp_dict) {
+        gf_msg(this->name, GF_LOG_ERROR, 0, GD_MSG_DICT_CREATE_FAIL,
+               "Failed to create response dictionary");
+        goto out;
+    }
+
     /* Pre Validation on local node */
     ret = gd_mgmt_v3_pre_validate_fn(op, req_dict, op_errstr, rsp_dict,
                                      op_errno);
@@ -1063,7 +1058,7 @@ glusterd_mgmt_v3_pre_validate(glusterd_op_t op, dict_t *req_dict,
         if (!peerinfo->connected)
             continue;
         if (op != GD_OP_SYNC_VOLUME &&
-            peerinfo->state.state != GD_FRIEND_STATE_BEFRIENDED)
+            peerinfo->state != GD_FRIEND_STATE_BEFRIENDED)
             continue;
 
         gd_mgmt_v3_pre_validate_req(op, req_dict, peerinfo, &args, MY_UUID,
@@ -1096,6 +1091,8 @@ glusterd_mgmt_v3_pre_validate(glusterd_op_t op, dict_t *req_dict,
                  gd_op_list[op], peer_cnt, ret);
 out:
     gd_syncargs_fini(&args);
+    if (rsp_dict)
+        dict_unref(rsp_dict);
     return ret;
 }
 
@@ -1129,7 +1126,7 @@ glusterd_mgmt_v3_build_payload(dict_t **req, char **op_errstr, dict_t *dict,
         case GD_OP_REPLACE_BRICK:
         case GD_OP_RESET_BRICK:
         case GD_OP_PROFILE_VOLUME: {
-            ret = dict_get_strn(dict, "volname", SLEN("volname"), &volname);
+            ret = dict_get_str(dict, "volname", &volname);
             if (ret) {
                 gf_msg(this->name, GF_LOG_CRITICAL, -ret,
                        GD_MSG_DICT_GET_FAILED,
@@ -1151,7 +1148,7 @@ glusterd_mgmt_v3_build_payload(dict_t **req, char **op_errstr, dict_t *dict,
                 ret = -1;
                 goto out;
             }
-            ret = dict_get_strn(dict, "volname", SLEN("volname"), &volname);
+            ret = dict_get_str(dict, "volname", &volname);
             if (ret) {
                 gf_msg(this->name, GF_LOG_CRITICAL, -ret,
                        GD_MSG_DICT_GET_FAILED,
@@ -1175,6 +1172,8 @@ glusterd_mgmt_v3_build_payload(dict_t **req, char **op_errstr, dict_t *dict,
     *req = req_dict;
     ret = 0;
 out:
+    if (ret && req_dict)
+        dict_unref(req_dict);
     return ret;
 }
 
@@ -1408,7 +1407,7 @@ glusterd_mgmt_v3_brick_op(glusterd_op_t op, dict_t *op_ctx, dict_t *req_dict,
         if (!peerinfo->connected)
             continue;
         if (op != GD_OP_SYNC_VOLUME &&
-            peerinfo->state.state != GD_FRIEND_STATE_BEFRIENDED)
+            peerinfo->state != GD_FRIEND_STATE_BEFRIENDED)
             continue;
 
         gd_mgmt_v3_brick_op_req(op, req_dict, peerinfo, &args, MY_UUID,
@@ -1686,7 +1685,7 @@ glusterd_mgmt_v3_commit(glusterd_op_t op, dict_t *op_ctx, dict_t *req_dict,
             continue;
 
         if (op != GD_OP_SYNC_VOLUME &&
-            peerinfo->state.state != GD_FRIEND_STATE_BEFRIENDED)
+            peerinfo->state != GD_FRIEND_STATE_BEFRIENDED)
             continue;
 
         gd_mgmt_v3_commit_req(op, req_dict, peerinfo, &args, MY_UUID,
@@ -1927,9 +1926,6 @@ glusterd_mgmt_v3_post_commit(glusterd_op_t op, dict_t *op_ctx, dict_t *req_dict,
         goto out;
     }
 
-    dict_unref(rsp_dict);
-    rsp_dict = NULL;
-
     /* Sending post commit req to other nodes in the cluster */
     ret = gd_syncargs_init(&args, op_ctx);
     if (ret)
@@ -1948,7 +1944,7 @@ glusterd_mgmt_v3_post_commit(glusterd_op_t op, dict_t *op_ctx, dict_t *req_dict,
             continue;
 
         if (op != GD_OP_SYNC_VOLUME &&
-            peerinfo->state.state != GD_FRIEND_STATE_BEFRIENDED)
+            peerinfo->state != GD_FRIEND_STATE_BEFRIENDED)
             continue;
 
         gd_mgmt_v3_post_commit_req(op, req_dict, peerinfo, &args, MY_UUID,
@@ -1980,6 +1976,10 @@ glusterd_mgmt_v3_post_commit(glusterd_op_t op, dict_t *op_ctx, dict_t *req_dict,
                  "peers. Returning %d",
                  gd_op_list[op], peer_cnt, ret);
 out:
+    if (rsp_dict) {
+        dict_unref(rsp_dict);
+        rsp_dict = NULL;
+    }
     glusterd_op_modify_op_ctx(op, op_ctx);
     gd_syncargs_fini(&args);
     return ret;
@@ -2170,7 +2170,7 @@ glusterd_mgmt_v3_post_validate(glusterd_op_t op, int32_t op_ret, dict_t *dict,
         if (!peerinfo->connected)
             continue;
         if (op != GD_OP_SYNC_VOLUME &&
-            peerinfo->state.state != GD_FRIEND_STATE_BEFRIENDED)
+            peerinfo->state != GD_FRIEND_STATE_BEFRIENDED)
             continue;
 
         gd_mgmt_v3_post_validate_req(op, op_ret, req_dict, peerinfo, &args,
@@ -2353,7 +2353,7 @@ glusterd_mgmt_v3_release_peer_locks(glusterd_op_t op, dict_t *dict,
         if (!peerinfo->connected)
             continue;
         if (op != GD_OP_SYNC_VOLUME &&
-            peerinfo->state.state != GD_FRIEND_STATE_BEFRIENDED)
+            peerinfo->state != GD_FRIEND_STATE_BEFRIENDED)
             continue;
 
         gd_mgmt_v3_unlock(op, dict, peerinfo, &args, MY_UUID, peer_uuid);
@@ -2733,7 +2733,7 @@ glusterd_set_barrier_value(dict_t *dict, char *option)
      * As of now only snapshot of single volume is supported,
      * Hence volname1 is directly fetched
      */
-    ret = dict_get_strn(dict, "volname1", SLEN("volname1"), &volname);
+    ret = dict_get_str(dict, "volname1", &volname);
     if (ret) {
         gf_msg(this->name, GF_LOG_ERROR, 0, GD_MSG_DICT_GET_FAILED,
                "Volname not present in "
@@ -2831,8 +2831,7 @@ glusterd_mgmt_v3_initiate_snap_phases(rpcsvc_request_t *req, glusterd_op_t op,
     }
 
     /* Marking the operation as complete synctasked */
-    ret = dict_set_int32n(dict, "is_synctasked", SLEN("is_synctasked"),
-                          _gf_true);
+    ret = dict_set_int32_sizen(dict, "is_synctasked", _gf_true);
     if (ret) {
         gf_msg(this->name, GF_LOG_ERROR, 0, GD_MSG_DICT_SET_FAILED,
                "Failed to set synctasked flag.");
@@ -2923,7 +2922,7 @@ glusterd_mgmt_v3_initiate_snap_phases(rpcsvc_request_t *req, glusterd_op_t op,
        above and along with it the originator glusterd also goes down?
        Who will initiate the cleanup?
     */
-    ret = dict_set_int32n(req_dict, "cleanup", SLEN("cleanup"), 1);
+    ret = dict_set_int32_sizen(req_dict, "cleanup", 1);
     if (ret) {
         gf_msg(this->name, GF_LOG_ERROR, 0, GD_MSG_DICT_SET_FAILED,
                "failed to set dict");
