@@ -18,10 +18,7 @@ void
 glusterd_nfssvc_build(glusterd_svc_t *svc);
 
 int
-glusterd_nfssvc_init(glusterd_svc_t *svc);
-
-int
-glusterd_nfssvc_reconfigure();
+glusterd_nfssvc_reconfigure(void);
 
 #endif /* BUILD_GNFS */
 #endif
