@@ -10,21 +10,20 @@
 #ifndef _GLUSTERD_OP_SM_H_
 #define _GLUSTERD_OP_SM_H_
 
-#include <pthread.h>
 #include <glusterfs/compat-uuid.h>
 
-#include <glusterfs/glusterfs.h>
 #include <glusterfs/xlator.h>
 #include <glusterfs/logging.h>
-#include <glusterfs/call-stub.h>
-#include <glusterfs/byte-order.h>
 #include "glusterd.h"
-#include "protocol-common.h"
 #include "glusterd-hooks.h"
 
 #define GD_OP_PROTECTED (0x02)
 #define GD_OP_UNPROTECTED (0x04)
 
+#define GLUSTERD_SHRD_STRG_HOOK_SCRIPT                                         \
+    "/hooks/1/set/post/"                                                       \
+    "S32gluster_enable_shared_storage.sh"
+
 typedef enum glusterd_op_sm_state_ {
     GD_OP_STATE_DEFAULT = 0,
     GD_OP_STATE_LOCK_SENT,
@@ -85,7 +84,6 @@ struct glusterd_op_info_ {
     int32_t op_count;
     /* op is an enum, glusterd_op_t or glusterd_op_sm_state_info_t */
     int op;
-    struct cds_list_head op_peers;
     void *op_ctx;
     rpcsvc_request_t *req;
     int32_t op_ret;
@@ -174,10 +172,10 @@ glusterd_op_sm_inject_event(glusterd_op_sm_event_type_t event_type,
                             uuid_t *txn_id, void *ctx);
 
 int
-glusterd_op_sm_init();
+glusterd_op_sm_init(void);
 
 int
-glusterd_op_sm();
+glusterd_op_sm(void);
 
 int32_t
 glusterd_op_set_ctx(void *ctx);
@@ -201,10 +199,10 @@ glusterd_op_txn_begin(rpcsvc_request_t *req, glusterd_op_t op, void *ctx,
                       char *err_str, size_t err_len);
 
 int32_t
-glusterd_op_txn_complete();
+glusterd_op_txn_complete(uuid_t *txn_id);
 
 void *
-glusterd_op_get_ctx();
+glusterd_op_get_ctx(void);
 
 int32_t
 glusterd_op_set_req(rpcsvc_request_t *req);
@@ -214,10 +212,10 @@ glusterd_op_send_cli_response(glusterd_op_t op, int32_t op_ret,
                               int32_t op_errno, rpcsvc_request_t *req,
                               void *ctx, char *op_errstr);
 int32_t
-glusterd_op_get_op();
+glusterd_op_get_op(void);
 
 int32_t
-glusterd_op_clear_op();
+glusterd_op_clear_op(void);
 
 int32_t
 glusterd_op_free_ctx(glusterd_op_t op, void *ctx);
@@ -243,10 +241,6 @@ glusterd_brick_op_build_payload(glusterd_op_t op,
 int
 glusterd_node_op_build_payload(glusterd_op_t op, gd1_mgmt_brick_op_req **req,
                                dict_t *dict);
-int32_t
-glusterd_handle_brick_rsp(void *pending_entry, glusterd_op_t op,
-                          dict_t *rsp_dict, dict_t *ctx_dict, char **op_errstr,
-                          gd_node_type type);
 
 dict_t *
 glusterd_op_init_commit_rsp_dict(glusterd_op_t op);
@@ -254,17 +248,6 @@ glusterd_op_init_commit_rsp_dict(glusterd_op_t op);
 void
 glusterd_op_modify_op_ctx(glusterd_op_t op, void *op_ctx);
 
-int
-glusterd_set_detach_bricks(dict_t *dict, glusterd_volinfo_t *volinfo);
-
-int32_t
-glusterd_volume_stats_read_perf(char *brick_path, int32_t blk_size,
-                                int32_t blk_count, double *throughput,
-                                double *time);
-int32_t
-glusterd_volume_stats_write_perf(char *brick_path, int32_t blk_size,
-                                 int32_t blk_count, double *throughput,
-                                 double *time);
 gf_boolean_t
 glusterd_is_volume_started(glusterd_volinfo_t *volinfo);
 
@@ -272,7 +255,7 @@ int
 glusterd_start_bricks(glusterd_volinfo_t *volinfo);
 
 gf_boolean_t
-glusterd_are_all_volumes_stopped();
+glusterd_are_all_volumes_stopped(void);
 int
 glusterd_stop_bricks(glusterd_volinfo_t *volinfo);
 int
@@ -287,8 +270,8 @@ glusterd_set_txn_opinfo(uuid_t *txn_id, glusterd_op_info_t *opinfo);
 
 void
 glusterd_txn_opinfo_init(glusterd_op_info_t *opinfo,
-                         glusterd_op_sm_state_t state, int *op,
-                         dict_t *op_ctx, rpcsvc_request_t *req);
+                         glusterd_op_sm_state_t state, int *op, dict_t *op_ctx,
+                         rpcsvc_request_t *req);
 
 int32_t
 glusterd_txn_opinfo_dict_init(void);
