@@ -8,10 +8,7 @@
    cases as published by the Free Software Foundation.
 */
 #include <glusterfs/common-utils.h>
-#include "cli1-xdr.h"
-#include "xdr-generic.h"
 #include <glusterfs/glusterfs.h>
-#include "glusterd.h"
 #include "glusterd-op-sm.h"
 #include "glusterd-geo-rep.h"
 #include "glusterd-store.h"
@@ -83,14 +80,14 @@ __glusterd_handle_replace_brick(rpcsvc_request_t *req)
         }
     }
 
-    ret = dict_get_strn(dict, "volname", SLEN("volname"), &volname);
+    ret = dict_get_str(dict, "volname", &volname);
     if (ret) {
         snprintf(msg, sizeof(msg), "Could not get volume name");
         gf_msg(this->name, GF_LOG_ERROR, 0, GD_MSG_DICT_GET_FAILED, "%s", msg);
         goto out;
     }
 
-    ret = dict_get_strn(dict, "operation", SLEN("operation"), &cli_op);
+    ret = dict_get_str(dict, "operation", &cli_op);
     if (ret) {
         gf_msg_debug(this->name, 0, "dict_get on operation failed");
         snprintf(msg, sizeof(msg), "Could not get operation");
@@ -110,7 +107,7 @@ __glusterd_handle_replace_brick(rpcsvc_request_t *req)
         goto out;
     }
 
-    ret = dict_get_strn(dict, "src-brick", SLEN("src-brick"), &src_brick);
+    ret = dict_get_str(dict, "src-brick", &src_brick);
 
     if (ret) {
         snprintf(msg, sizeof(msg), "Failed to get src brick");
@@ -122,7 +119,7 @@ __glusterd_handle_replace_brick(rpcsvc_request_t *req)
     if (!strcmp(cli_op, "GF_RESET_OP_COMMIT") ||
         !strcmp(cli_op, "GF_RESET_OP_COMMIT_FORCE") ||
         !strcmp(cli_op, "GF_REPLACE_OP_COMMIT_FORCE")) {
-        ret = dict_get_strn(dict, "dst-brick", SLEN("dst-brick"), &dst_brick);
+        ret = dict_get_str(dict, "dst-brick", &dst_brick);
 
         if (ret) {
             snprintf(msg, sizeof(msg),
@@ -166,6 +163,38 @@ glusterd_handle_replace_brick(rpcsvc_request_t *req)
     return glusterd_big_locked_handler(req, __glusterd_handle_replace_brick);
 }
 
+static int
+glusterd_rb_check_bricks(glusterd_volinfo_t *volinfo, glusterd_brickinfo_t *src,
+                         glusterd_brickinfo_t *dst)
+{
+    glusterd_replace_brick_t *rb = NULL;
+
+    GF_ASSERT(volinfo);
+
+    rb = &volinfo->rep_brick;
+
+    if (!rb->src_brick || !rb->dst_brick) {
+        gf_smsg("glusterd", GF_LOG_ERROR, errno, GD_MSG_INVALID_ARGUMENT, NULL);
+        return -1;
+    }
+
+    if (strcmp(rb->src_brick->hostname, src->hostname) ||
+        strcmp(rb->src_brick->path, src->path)) {
+        gf_msg("glusterd", GF_LOG_ERROR, 0, GD_MSG_RB_SRC_BRICKS_MISMATCH,
+               "Replace brick src bricks differ");
+        return -1;
+    }
+
+    if (strcmp(rb->dst_brick->hostname, dst->hostname) ||
+        strcmp(rb->dst_brick->path, dst->path)) {
+        gf_msg("glusterd", GF_LOG_ERROR, 0, GD_MSG_RB_DST_BRICKS_MISMATCH,
+               "Replace brick dst bricks differ");
+        return -1;
+    }
+
+    return 0;
+}
+
 int
 glusterd_op_stage_replace_brick(dict_t *dict, char **op_errstr,
                                 dict_t *rsp_dict)
@@ -296,7 +325,7 @@ glusterd_op_stage_replace_brick(dict_t *dict, char **op_errstr,
             *op_errstr = gf_strdup(msg);
             goto out;
 
-        } else if (GD_FRIEND_STATE_BEFRIENDED != peerinfo->state.state) {
+        } else if (GD_FRIEND_STATE_BEFRIENDED != peerinfo->state) {
             RCU_READ_UNLOCK;
             ret = -1;
             snprintf(msg, sizeof(msg),
@@ -332,7 +361,7 @@ glusterd_op_stage_replace_brick(dict_t *dict, char **op_errstr,
             }
         }
 
-        ret = dict_set_int32n(rsp_dict, "brick_count", SLEN("brick_count"), 1);
+        ret = dict_set_int32_sizen(rsp_dict, "brick_count", 1);
         if (ret) {
             gf_msg(this->name, GF_LOG_ERROR, 0, GD_MSG_DICT_SET_FAILED,
                    "Failed to set local_brick_count");
@@ -406,8 +435,7 @@ glusterd_op_perform_replace_brick(glusterd_volinfo_t *volinfo, char *old_brick,
      * introduced in gluster-3.6.0
      */
     if (conf->op_version >= GD_OP_VERSION_3_6_0) {
-        ret = dict_get_strn(dict, "brick1.mount_dir", SLEN("brick1.mount_dir"),
-                            &brick_mount_dir);
+        ret = dict_get_str(dict, "brick1.mount_dir", &brick_mount_dir);
         if (ret) {
             gf_msg(this->name, GF_LOG_ERROR, errno,
                    GD_MSG_BRICK_MOUNTDIR_GET_FAIL,
@@ -470,7 +498,7 @@ glusterd_op_replace_brick(dict_t *dict, dict_t *rsp_dict)
     priv = this->private;
     GF_ASSERT(priv);
 
-    ret = dict_get_strn(dict, "src-brick", SLEN("src-brick"), &src_brick);
+    ret = dict_get_str(dict, "src-brick", &src_brick);
     if (ret) {
         gf_msg(this->name, GF_LOG_ERROR, 0, GD_MSG_DICT_GET_FAILED,
                "Unable to get src brick");
@@ -479,7 +507,7 @@ glusterd_op_replace_brick(dict_t *dict, dict_t *rsp_dict)
 
     gf_msg_debug(this->name, 0, "src brick=%s", src_brick);
 
-    ret = dict_get_strn(dict, "dst-brick", SLEN("dst-brick"), &dst_brick);
+    ret = dict_get_str(dict, "dst-brick", &dst_brick);
     if (ret) {
         gf_msg(this->name, GF_LOG_ERROR, 0, GD_MSG_DICT_GET_FAILED,
                "Unable to get dst brick");
@@ -488,14 +516,14 @@ glusterd_op_replace_brick(dict_t *dict, dict_t *rsp_dict)
 
     gf_msg_debug(this->name, 0, "dst brick=%s", dst_brick);
 
-    ret = dict_get_strn(dict, "volname", SLEN("volname"), &volname);
+    ret = dict_get_str(dict, "volname", &volname);
     if (ret) {
         gf_msg(this->name, GF_LOG_ERROR, 0, GD_MSG_DICT_GET_FAILED,
                "Unable to get volume name");
         goto out;
     }
 
-    ret = dict_get_strn(dict, "operation", SLEN("operation"), &replace_op);
+    ret = dict_get_str(dict, "operation", &replace_op);
     if (ret) {
         gf_msg_debug(this->name, 0, "dict_get on operation failed");
         goto out;
@@ -620,8 +648,7 @@ glusterd_mgmt_v3_initiate_replace_brick_cmd_phases(rpcsvc_request_t *req,
         goto out;
     }
 
-    ret = dict_set_int32n(dict, "is_synctasked", SLEN("is_synctasked"),
-                          _gf_true);
+    ret = dict_set_int32_sizen(dict, "is_synctasked", _gf_true);
     if (ret) {
         gf_msg(this->name, GF_LOG_ERROR, 0, GD_MSG_DICT_SET_FAILED,
                "Failed to set synctasked flag to true.");
