@@ -452,7 +452,7 @@ out:
 }
 
 glusterd_svc_proc_t *
-glusterd_svcprocess_new()
+glusterd_svcprocess_new(void)
 {
     glusterd_svc_proc_t *new_svcprocess = NULL;
 
@@ -562,7 +562,7 @@ unlock:
 out:
     if (stop_daemon) {
         glusterd_proc_stop(&svc->proc, SIGTERM, PROC_STOP_FORCE);
-        glusterd_unlink_file(pidfile);
+        gf_unlink(pidfile);
     }
     return ret;
 }
@@ -761,7 +761,7 @@ __glusterd_send_svc_configure_req(glusterd_svc_t *svc, int flags,
     }
 
     conn = &rpc->conn;
-    if (!conn->connected || conn->disconnected) {
+    if (rpc_clnt_connection_status(conn) != RPC_STATUS_CONNECTED) {
         gf_msg(this->name, GF_LOG_INFO, 0, GD_MSG_CONNECT_RETURNED,
                "not connected yet");
         return -1;
@@ -880,7 +880,7 @@ err:
     }
     if (dict)
         dict_unref(dict);
-    if (ret && brick_req.dict.dict_val)
+    if (brick_req.dict.dict_val)
         GF_FREE(brick_req.dict.dict_val);
 
     GF_FREE(volfile_content);
@@ -891,6 +891,31 @@ err:
     return ret;
 }
 
+static gf_boolean_t
+glusterd_volume_exists(const char *volname)
+{
+    glusterd_volinfo_t *tmp_volinfo = NULL;
+    gf_boolean_t volume_found = _gf_false;
+    xlator_t *this = THIS;
+    glusterd_conf_t *priv = NULL;
+
+    GF_ASSERT(volname);
+
+    priv = this->private;
+    GF_ASSERT(priv);
+
+    cds_list_for_each_entry(tmp_volinfo, &priv->volumes, vol_list)
+    {
+        if (!strcmp(tmp_volinfo->volname, volname)) {
+            gf_msg_debug(this->name, 0, "Volume %s found", volname);
+            volume_found = _gf_true;
+            break;
+        }
+    }
+
+    return volume_found;
+}
+
 int
 glusterd_attach_svc(glusterd_svc_t *svc, glusterd_volinfo_t *volinfo, int flags)
 {
