@@ -12,7 +12,7 @@
 #include "glusterd-utils.h"
 #include "glusterd-tierd-svc-helper.h"
 #include "glusterd-messages.h"
-#include "syscall.h"
+#include <glusterfs/syscall.h>
 #include "glusterd-volgen.h"
 
 void
@@ -81,7 +81,8 @@ glusterd_svc_build_tierd_volfile_path(glusterd_volinfo_t *volinfo, char *path,
 void
 glusterd_svc_build_tierd_logdir(char *logdir, char *volname, size_t len)
 {
-    snprintf(logdir, len, "%s/tier/%s", DEFAULT_LOG_FILE_DIRECTORY, volname);
+    glusterd_conf_t *conf = THIS->private;
+    snprintf(logdir, len, "%s/tier/%s", priv->logdir, volname);
 }
 
 void
@@ -99,14 +100,11 @@ glusterd_svc_check_tier_volfile_identical(char *svc_name,
         0,
     };
     char *tmpvol = NULL;
-    xlator_t *this = NULL;
+    xlator_t *this = THIS;
     int ret = -1;
     int need_unlink = 0;
     int tmp_fd = -1;
 
-    this = THIS;
-
-    GF_VALIDATE_OR_GOTO(THIS->name, this, out);
     GF_VALIDATE_OR_GOTO(this->name, identical, out);
 
     glusterd_svc_build_tierd_volfile_path(volinfo, orgvol, sizeof(orgvol));
@@ -116,7 +114,7 @@ glusterd_svc_check_tier_volfile_identical(char *svc_name,
         goto out;
     }
 
-    /* coverity[secure_temp] mkstemp uses 0600 as the mode and is safe */
+    /* coverity[SECURE_TEMP] mkstemp uses 0600 as the mode and is safe */
     tmp_fd = mkstemp(tmpvol);
     if (tmp_fd < 0) {
         gf_msg(this->name, GF_LOG_WARNING, errno, GD_MSG_FILE_OP_FAILED,
@@ -164,7 +162,7 @@ glusterd_svc_check_tier_topology_identical(char *svc_name,
     int tmpclean = 0;
     int tmpfd = -1;
 
-    if ((!identical) || (!this) || (!this->private))
+    if ((!identical) || (!this->private))
         goto out;
 
     conf = this->private;
@@ -177,7 +175,7 @@ glusterd_svc_check_tier_topology_identical(char *svc_name,
         goto out;
     }
 
-    /* coverity[secure_temp] mkstemp uses 0600 as the mode and is safe */
+    /* coverity[SECURE_TEMP] mkstemp uses 0600 as the mode and is safe */
     tmpfd = mkstemp(tmpvol);
     if (tmpfd < 0) {
         gf_msg(this->name, GF_LOG_WARNING, errno, GD_MSG_FILE_OP_FAILED,
