@@ -11,14 +11,14 @@
 #ifndef _ACL3_H
 #define _ACL3_H
 
-#include "glusterfs-acl.h"
+#include <glusterfs/glusterfs-acl.h>
 
 #define ACL3_NULL 0
 #define ACL3_GETACL 1
 #define ACL3_SETACL 2
 #define ACL3_PROC_COUNT 3
 
-#define GF_ACL3_PORT 38469
+#define GF_ACL3_PORT "38469"
 #define GF_ACL GF_NFS "-ACL"
 
 /* Flags for the getacl/setacl mode */
