@@ -17,7 +17,7 @@ template_mem_acct_init(xlator_t *this)
 
     GF_VALIDATE_OR_GOTO("template", this, out);
 
-    ret = xlator_mem_acct_init(this, gf_template_mt_end + 1);
+    ret = xlator_mem_acct_init(this, gf_template_mt_end);
 
     if (ret != 0) {
         gf_msg(this->name, GF_LOG_ERROR, ENOMEM, TEMPLATE_MSG_NO_MEMORY,
