@@ -11,16 +11,12 @@
 #ifndef __CLIENT_MEM_TYPES_H__
 #define __CLIENT_MEM_TYPES_H__
 
-#include "mem-types.h"
+#include <glusterfs/mem-types.h>
 
 enum gf_client_mem_types_ {
     gf_client_mt_clnt_conf_t = gf_common_mt_end + 1,
     gf_client_mt_clnt_req_buf_t,
     gf_client_mt_clnt_fdctx_t,
-    gf_client_mt_clnt_lock_t,
-    gf_client_mt_clnt_fd_lk_local_t,
-    gf_client_mt_clnt_args_t,
-    gf_client_mt_compound_req_t,
     gf_client_mt_clnt_lock_request_t,
     gf_client_mt_end,
 };
