@@ -8,15 +8,13 @@
   cases as published by the Free Software Foundation.
 */
 
+#include "socket.h"
 #include "client.h"
-#include <glusterfs/xlator.h>
 #include <glusterfs/defaults.h>
-#include <glusterfs/glusterfs.h>
 #include <glusterfs/statedump.h>
 #include <glusterfs/compat-errno.h>
 #include <glusterfs/gf-event.h>
 
-#include "xdr-rpc.h"
 #include "glusterfs3.h"
 #include "client-messages.h"
 
@@ -251,10 +249,10 @@ client_submit_request(xlator_t *this, void *req, call_frame_t *frame,
 
     /* do not send all groups if they are resolved server-side */
     if (!conf->send_gids) {
-        if (frame->root->ngrps <= SMALL_GROUP_COUNT) {
-            frame->root->groups_small[0] = frame->root->gid;
-            frame->root->groups = frame->root->groups_small;
-        }
+        /* We don't need to send the gids of the user. Just reset the list to
+         * a single item with the primary gid. */
+        frame->root->groups_small[0] = frame->root->gid;
+        frame->root->groups = frame->root->groups_small;
         frame->root->ngrps = 1;
     }
 
@@ -796,7 +794,7 @@ client_open(call_frame_t *frame, xlator_t *this, loc_t *loc, int32_t flags,
         pthread_spin_lock(&conf->fd_lock);
         {
             fdctx = this_fd_get_ctx(fd, this);
-            if (fdctx && !fdctx_lock_lists_empty(fdctx)) {
+            if (fdctx && !fd_lk_ctx_empty(fdctx->lk_ctx)) {
                 ret = -1;
                 op_errno = EBADFD;
             }
@@ -2302,7 +2300,17 @@ client_rpc_notify(struct rpc_clnt *rpc, void *mydata, rpc_clnt_event_t event,
                 rpc_clnt_cleanup_and_start(rpc);
 
             } else {
-                rpc->conn.config.remote_port = 0;
+                int32_t remote_port = 0;
+                ret = dict_get_int32(this->options, "remote-port",
+                                     &remote_port);
+                if (IS_ERROR(ret)) {
+                    /* this is optional, so it may be error in most cases. Add a
+                     * trace log */
+                    gf_msg_trace(
+                        this->name, 0,
+                        "volfile doesn't have remote-port, resetting to 0");
+                }
+                conf->rpc_conf.remote_port = remote_port;
                 conf->connection_to_brick = _gf_false;
             }
             break;
