@@ -105,8 +105,14 @@ static int
 fini(dict_t *this, char *key, data_t *value, void *data)
 {
     auth_handle_t *handle = data_to_ptr(value);
+
     if (handle) {
         dlclose(handle->handle);
+
+        if (handle->vol_opt) {
+            list_del_init(&handle->vol_opt->list);
+            GF_FREE(handle->vol_opt);
+        }
     }
     return 0;
 }
