@@ -17,7 +17,6 @@
 #include <openssl/md5.h>
 #include <stdint.h>
 #include <sys/time.h>
-#include <sys/resource.h>
 #include <errno.h>
 #include <libgen.h>
 #include <pthread.h>
@@ -35,15 +34,13 @@
 #include <fcntl.h>
 #endif /* HAVE_LINKAT */
 
+#include "posix.h"
 #include "posix-inode-handle.h"
 #include <glusterfs/compat-errno.h>
 #include <glusterfs/compat.h>
-#include <glusterfs/byte-order.h>
 #include <glusterfs/syscall.h>
 #include <glusterfs/statedump.h>
 #include <glusterfs/locking.h>
-#include <glusterfs/timer.h>
-#include "glusterfs3-xdr.h"
 #include "posix-aio.h"
 #include "posix-io-uring.h"
 #include <glusterfs/glusterfs-acl.h>
@@ -53,7 +50,7 @@
 #include <glusterfs/compat-uuid.h>
 #include "timer-wheel.h"
 
-extern char *marker_xattrs[];
+// extern char *marker_xattrs[];
 #define ALIGN_SIZE 4096
 
 #undef HAVE_SET_FSID
@@ -130,11 +127,9 @@ posix_inode(xlator_t *this)
 }
 
 static void
-delete_posix_diskxl(xlator_t *this)
+delete_posix_diskxl(struct posix_private *priv, glusterfs_ctx_t *ctx)
 {
-    struct posix_private *priv = this->private;
     struct posix_diskxl *pxl = priv->pxl;
-    glusterfs_ctx_t *ctx = this->ctx;
     uint32_t count = 1;
 
     if (pxl) {
@@ -215,7 +210,7 @@ posix_notify(xlator_t *this, int32_t event, void *data, ...)
             }
             pthread_mutex_unlock(&ctx->fd_lock);
 
-            delete_posix_diskxl(this);
+            delete_posix_diskxl(priv, ctx);
 
             gf_log(this->name, GF_LOG_INFO, "Sending CHILD_DOWN for brick %s",
                    victim->name);
@@ -344,6 +339,7 @@ posix_reconfigure(xlator_t *this, dict_t *options)
     double old_disk_reserve = 0.0;
 
     priv = this->private;
+    glusterfs_ctx_t *ctx = this->ctx;
 
     GF_OPTION_RECONF("brick-uid", uid, options, int32, out);
     GF_OPTION_RECONF("brick-gid", gid, options, int32, out);
@@ -427,7 +423,7 @@ posix_reconfigure(xlator_t *this, dict_t *options)
        is changed for reserve option during graph reconfigure
     */
     if (old_disk_reserve != priv->disk_reserve) {
-        delete_posix_diskxl(this);
+        delete_posix_diskxl(priv, ctx);
         old_disk_reserve = 0;
     }
 
@@ -494,7 +490,7 @@ out:
     return ret;
 }
 
-int32_t
+static int32_t
 posix_delete_unlink_entry(const char *fpath, const struct stat *sb,
                           int typeflag, struct FTW *ftwbuf)
 {
@@ -531,7 +527,7 @@ out:
     return 0;
 }
 
-int32_t
+static int32_t
 posix_delete_unlink(const char *unlink_path)
 {
     int ret = -1;
@@ -547,7 +543,7 @@ posix_delete_unlink(const char *unlink_path)
     return ret;
 }
 
-int32_t
+static int32_t
 posix_create_unlink_dir(xlator_t *this)
 {
     struct posix_private *priv = NULL;
@@ -603,7 +599,7 @@ posix_create_unlink_dir(xlator_t *this)
     return 0;
 }
 
-int
+static int
 posix_create_open_directory_based_fd(xlator_t *this, int pdirfd, char *dir_name)
 {
     int ret = -1;
@@ -639,6 +635,9 @@ posix_init(xlator_t *this)
     struct stat buf = {
         0,
     };
+    struct statvfs fs = {
+        0,
+    };
     gf_boolean_t tmp_bool = 0;
     int ret = 0;
     int op_ret = -1;
@@ -714,6 +713,13 @@ posix_init(xlator_t *this)
     _private->base_path = gf_strdup(dir_data->data);
     _private->base_path_length = dir_data->len - 1;
 
+    if (sys_statvfs(_private->base_path, &fs) < 0) {
+        ret = -1;
+        goto out;
+    }
+    GF_ASSERT((fs.f_bsize & (fs.f_bsize - 1)) == 0);
+    _private->base_bsize = fs.f_bsize;
+
     _private->dirfd = -1;
     _private->mount_lock = -1;
     for (i = 0; i < 256; i++)
@@ -986,32 +992,9 @@ posix_init(xlator_t *this)
                "Could not lock brick directory (%s)", strerror(op_errno));
         goto out;
     }
-#ifndef GF_DARWIN_HOST_OS
-    {
-        struct rlimit lim;
-        lim.rlim_cur = 1048576;
-        lim.rlim_max = 1048576;
 
-        if (setrlimit(RLIMIT_NOFILE, &lim) == -1) {
-            gf_msg(this->name, GF_LOG_WARNING, errno, P_MSG_SET_ULIMIT_FAILED,
-                   "Failed to set 'ulimit -n "
-                   " 1048576'");
-            lim.rlim_cur = 65536;
-            lim.rlim_max = 65536;
+    gf_set_nofile(1048576, 65536);
 
-            if (setrlimit(RLIMIT_NOFILE, &lim) == -1) {
-                gf_msg(this->name, GF_LOG_WARNING, errno,
-                       P_MSG_SET_FILE_MAX_FAILED,
-                       "Failed to set maximum allowed open "
-                       "file descriptors to 64k");
-            } else {
-                gf_msg(this->name, GF_LOG_INFO, 0, P_MSG_MAX_FILE_OPEN,
-                       "Maximum allowed "
-                       "open file descriptors set to 65536");
-            }
-        }
-    }
-#endif
     _private->shared_brick_count = 1;
     ret = dict_get_int32(this->options, "shared-brick-count",
                          &_private->shared_brick_count);
