@@ -14,10 +14,8 @@
 #define _GNU_SOURCE
 #endif
 
-#include <openssl/md5.h>
 #include <stdint.h>
 #include <sys/time.h>
-#include <sys/resource.h>
 #include <errno.h>
 #include <libgen.h>
 #include <pthread.h>
@@ -41,12 +39,9 @@
 #include "posix-handle.h"
 #include <glusterfs/compat-errno.h>
 #include <glusterfs/compat.h>
-#include <glusterfs/byte-order.h>
 #include <glusterfs/syscall.h>
 #include <glusterfs/statedump.h>
 #include <glusterfs/locking.h>
-#include <glusterfs/timer.h>
-#include "glusterfs3-xdr.h"
 #include "posix-aio.h"
 #include <glusterfs/glusterfs-acl.h>
 #include "posix-messages.h"
@@ -56,7 +51,7 @@
 #include <glusterfs/compat-uuid.h>
 #include <glusterfs/syncop.h>
 
-extern char *marker_xattrs[];
+// extern char *marker_xattrs[];
 #define ALIGN_SIZE 4096
 
 #undef HAVE_SET_FSID
@@ -86,6 +81,9 @@ extern char *marker_xattrs[];
 
 #endif
 
+static int
+posix_unlink_stale_linkto(call_frame_t *frame, xlator_t *this,
+                          const char *real_path, int32_t *op_errno, loc_t *loc);
 static gf_boolean_t
 posix_symlinks_match(xlator_t *this, loc_t *loc, uuid_t gfid)
 {
@@ -177,6 +175,12 @@ posix_lookup(call_frame_t *frame, xlator_t *this, loc_t *loc, dict_t *xdata)
     posix_inode_ctx_t *ctx = NULL;
     int ret = 0;
     int dfd = -1;
+    uint32_t lookup_unlink_dir = 0;
+    char *unlink_path = NULL;
+    struct stat lstatbuf = {
+        0,
+    };
+    gf_boolean_t cs_obj_status, cs_obj_repair;
 
     VALIDATE_OR_GOTO(frame, out);
     VALIDATE_OR_GOTO(this, out);
@@ -198,11 +202,53 @@ posix_lookup(call_frame_t *frame, xlator_t *this, loc_t *loc, dict_t *xdata)
         goto out;
     }
 
+#ifdef __NetBSD__
+    /* Same for NetBSD's .attribute directory */
+    if (__is_root_gfid(loc->pargfid) && loc->name &&
+        (strcmp(loc->name, ".attribute") == 0)) {
+        gf_msg(this->name, GF_LOG_WARNING, EPERM, P_MSG_LOOKUP_NOT_PERMITTED,
+               "Lookup issued on .attribute,"
+               " which is not permitted");
+        op_errno = EPERM;
+        op_ret = -1;
+        goto out;
+    }
+#endif /* __NetBSD__ */
+
     op_ret = dict_get_int32_sizen(xdata, GF_GFIDLESS_LOOKUP, &gfidless);
     op_ret = -1;
     if (gf_uuid_is_null(loc->pargfid) || (loc->name == NULL)) {
         /* nameless lookup */
+        op_ret = op_errno = errno = 0;
         MAKE_INODE_HANDLE(real_path, this, loc, &buf);
+
+        /* The gfid will be renamed to ".glusterfs/unlink" in case
+         * there are any open fds on the file in posix_unlink path.
+         * So client can request server to do nameless lookup with
+         * xdata = GF_UNLINKED_LOOKUP in ".glusterfs/unlink"
+         * dir if a client wants to know the status of the all open fds
+         * on the unlinked file. If the file still present in the
+         * ".glusterfs/unlink" dir then it indicates there still
+         * open fds present on the file and the file is still under
+         * unlink process */
+        if (op_ret < 0 && errno == ENOENT) {
+            ret = dict_get_uint32(xdata, GF_UNLINKED_LOOKUP,
+                                  &lookup_unlink_dir);
+            if (!ret && lookup_unlink_dir) {
+                op_ret = op_errno = errno = 0;
+                POSIX_GET_FILE_UNLINK_PATH(priv->base_path, loc->gfid,
+                                           unlink_path);
+                ret = sys_lstat(unlink_path, &lstatbuf);
+                if (ret) {
+                    op_ret = -1;
+                    op_errno = errno;
+                } else {
+                    iatt_from_stat(&buf, &lstatbuf);
+                    buf.ia_nlink = 0;
+                }
+                goto nameless_lookup_unlink_dir_out;
+            }
+        }
     } else {
         MAKE_ENTRY_HANDLE(real_path, par_path, this, loc, &buf);
         if (!real_path || !par_path) {
@@ -245,7 +291,7 @@ posix_lookup(call_frame_t *frame, xlator_t *this, loc_t *loc, dict_t *xdata)
                        "Found stale gfid "
                        "handle %s, removing it.",
                        gfid_path);
-                posix_handle_unset(this, gfid, NULL);
+                posix_handle_unset_gfid(this, gfid);
             }
         }
         goto parent;
@@ -254,8 +300,13 @@ posix_lookup(call_frame_t *frame, xlator_t *this, loc_t *loc, dict_t *xdata)
     if (xdata && (op_ret == 0)) {
         xattr = posix_xattr_fill(this, real_path, loc, NULL, -1, xdata, &buf);
 
-        posix_cs_maintenance(this, NULL, loc, NULL, &buf, real_path, xdata,
-                             &xattr, _gf_true);
+        cs_obj_status = dict_get_sizen(xdata, GF_CS_OBJECT_STATUS);
+        cs_obj_repair = dict_get_sizen(xdata, GF_CS_OBJECT_REPAIR);
+
+        if (cs_obj_status || cs_obj_repair)
+            posix_cs_maintenance(this, NULL, loc, NULL, &buf, real_path,
+                                 cs_obj_status, cs_obj_repair, &xattr,
+                                 _gf_true);
 
         if (dict_get_sizen(xdata, GF_CLEAN_WRITE_PROTECTION)) {
             ret = sys_lremovexattr(real_path, GF_PROTECT_FROM_EXTERNAL_WRITES);
@@ -264,9 +315,11 @@ posix_lookup(call_frame_t *frame, xlator_t *this, loc_t *loc, dict_t *xdata)
                        "removexattr failed. key %s path %s",
                        GF_PROTECT_FROM_EXTERNAL_WRITES, loc->path);
         }
+
+        if (cs_obj_status)
+            posix_update_iatt_buf(&buf, -1, real_path);
     }
 
-    posix_update_iatt_buf(&buf, -1, real_path, xdata);
     if (priv->update_pgfid_nlinks) {
         if (!gf_uuid_is_null(loc->pargfid) && !IA_ISDIR(buf.ia_type)) {
             MAKE_PGFID_XATTR_KEY(pgfid_xattr_key, PGFID_XATTR_KEY_PREFIX,
@@ -292,7 +345,7 @@ posix_lookup(call_frame_t *frame, xlator_t *this, loc_t *loc, dict_t *xdata)
 parent:
     if (par_path) {
         op_ret = posix_pstat(this, loc->parent, loc->pargfid, par_path,
-                             &postparent, _gf_false);
+                             &postparent, _gf_false, _gf_true);
         if (op_ret == -1) {
             op_errno = errno;
             gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_LSTAT_FAILED,
@@ -313,15 +366,28 @@ parent:
 out:
     if (!op_ret && !gfidless && gf_uuid_is_null(buf.ia_gfid)) {
         gf_msg(this->name, GF_LOG_ERROR, ENODATA, P_MSG_NULL_GFID,
-               "buf->ia_gfid is null for "
-               "%s",
-               (real_path) ? real_path : "");
+               "buf->ia_gfid is null for %s",
+               (real_path) ? real_path : "(null)");
         op_ret = -1;
         op_errno = ENODATA;
     }
 
+    /* TODO: get the path */
+    /* In the full run of regression, I was not able to hit this case, hence
+       leaving it as TODO. Good to have logic of resolving GFID only access
+       to a path for many other features too. But initial version can just
+       be knowning that we are hitting the scenario in certain usecases */
+    if ((op_ret == 0) && (dict_get_sizen(xdata, "get-full-path"))) {
+        /* Get the path */
+        gf_log(this->name, GF_LOG_INFO,
+               "%s: inode path not completely resolved. Asking for full path",
+               loc->path);
+    }
+
     if (op_ret == 0)
         op_errno = 0;
+
+nameless_lookup_unlink_dir_out:
     STACK_UNWIND_STRICT(lookup, frame, op_ret, op_errno,
                         (loc) ? loc->inode : NULL, &buf, xattr, &postparent);
 
@@ -363,6 +429,43 @@ posix_set_gfid2path_xattr(xlator_t *this, const char *path, uuid_t pgfid,
     return ret;
 }
 
+static int
+posix_acl_xattr_from_dict(const char *path, char *key, dict_t *xattr_req)
+{
+    int ret = 0;
+    data_t *data = NULL;
+
+    data = dict_get(xattr_req, key);
+    if (data) {
+        ret = sys_lsetxattr(path, key, data->data, data->len, 0);
+#ifdef __FreeBSD__
+        if (ret != -1) {
+            ret = 0;
+        }
+#endif /* __FreeBSD__ */
+    }
+
+    return ret;
+}
+
+static int
+posix_acl_xattr_set(const char *path, dict_t *xattr_req)
+{
+    int ret = 0;
+
+    if (!xattr_req)
+        goto out;
+
+    ret = posix_acl_xattr_from_dict(path, POSIX_ACL_ACCESS_XATTR, xattr_req);
+    if (ret)
+        goto out;
+
+    ret = posix_acl_xattr_from_dict(path, POSIX_ACL_DEFAULT_XATTR, xattr_req);
+
+out:
+    return ret;
+}
+
 int
 posix_mknod(call_frame_t *frame, xlator_t *this, loc_t *loc, mode_t mode,
             dev_t dev, mode_t umask, dict_t *xdata)
@@ -420,7 +523,7 @@ posix_mknod(call_frame_t *frame, xlator_t *this, loc_t *loc, mode_t mode,
     }
 
     op_ret = posix_pstat(this, loc->parent, loc->pargfid, par_path, &preparent,
-                         _gf_false);
+                         _gf_false, _gf_true);
     if (op_ret == -1) {
         op_errno = errno;
         gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_LSTAT_FAILED,
@@ -482,9 +585,20 @@ real_op:
             }
             sys_close(tmp_fd);
         } else {
-            if (op_errno == EEXIST)
+            if (op_errno == EEXIST) {
                 level = GF_LOG_DEBUG;
-            else
+
+                if (dict_get_sizen(xdata, GF_FORCE_REPLACE_KEY)) {
+                    dict_del_sizen(xdata, GF_FORCE_REPLACE_KEY);
+                    op_ret = posix_unlink_stale_linkto(frame, this, real_path,
+                                                       &op_errno, loc);
+                    if (op_ret == 0)
+                        goto real_op;
+
+                    level = GF_LOG_ERROR;
+                }
+
+            } else
                 level = GF_LOG_ERROR;
             gf_msg(this->name, level, errno, P_MSG_MKNOD_FAILED,
                    "mknod on %s failed", real_path);
@@ -505,7 +619,7 @@ real_op:
 #endif
 
 post_op:
-    op_ret = posix_acl_xattr_set(this, real_path, xdata);
+    op_ret = posix_acl_xattr_set(real_path, xdata);
     if (op_ret) {
         gf_msg(this->name, GF_LOG_ERROR, 0, P_MSG_ACL_FAILED,
                "setting ACLs on %s failed", real_path);
@@ -555,7 +669,8 @@ post_op:
         }
     }
 
-    op_ret = posix_pstat(this, loc->inode, NULL, real_path, &stbuf, _gf_false);
+    op_ret = posix_pstat(this, loc->inode, NULL, real_path, &stbuf, _gf_false,
+                         _gf_true);
     if (op_ret == -1) {
         op_errno = errno;
         gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_MKNOD_FAILED,
@@ -566,7 +681,7 @@ post_op:
     posix_set_ctime(frame, this, real_path, -1, loc->inode, &stbuf);
 
     op_ret = posix_pstat(this, loc->parent, loc->pargfid, par_path, &postparent,
-                         _gf_false);
+                         _gf_false, _gf_true);
     if (op_ret == -1) {
         op_errno = errno;
         gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_LSTAT_FAILED,
@@ -650,6 +765,19 @@ posix_mkdir(call_frame_t *frame, xlator_t *this, loc_t *loc, mode_t mode,
         goto out;
     }
 
+#ifdef __NetBSD__
+    /* Same for NetBSD's .attribute directory */
+    if (__is_root_gfid(loc->pargfid) &&
+        (strcmp(loc->name, ".attribute") == 0)) {
+        gf_msg(this->name, GF_LOG_WARNING, EPERM, P_MSG_MKDIR_NOT_PERMITTED,
+               "mkdir issued on .attribute, which"
+               "is not permitted");
+        op_errno = EPERM;
+        op_ret = -1;
+        goto out;
+    }
+#endif
+
     priv = this->private;
     VALIDATE_OR_GOTO(priv, out);
     GFID_NULL_CHECK_AND_GOTO(frame, this, loc, xdata, op_ret, op_errno,
@@ -665,7 +793,8 @@ posix_mkdir(call_frame_t *frame, xlator_t *this, loc_t *loc, mode_t mode,
 
     gid = frame->root->gid;
 
-    op_ret = posix_pstat(this, loc->inode, NULL, real_path, &stbuf, _gf_false);
+    op_ret = posix_pstat(this, loc->inode, NULL, real_path, &stbuf, _gf_false,
+                         _gf_false);
 
     SET_FS_ID(frame->root->uid, gid);
 
@@ -682,7 +811,8 @@ posix_mkdir(call_frame_t *frame, xlator_t *this, loc_t *loc, mode_t mode,
     }
 
     if (!gf_uuid_is_null(uuid_req)) {
-        op_ret = posix_istat(this, loc->inode, uuid_req, NULL, &stbuf);
+        op_ret = posix_istat(this, loc->inode, uuid_req, NULL, &stbuf,
+                             _gf_false);
         if ((op_ret == 0) && IA_ISDIR(stbuf.ia_type)) {
             gfid_path = alloca(PATH_MAX);
             size = posix_handle_path(this, uuid_req, NULL, gfid_path, PATH_MAX);
@@ -713,7 +843,7 @@ posix_mkdir(call_frame_t *frame, xlator_t *this, loc_t *loc, mode_t mode,
                  * remove the old symlink in order for
                  * posix_gfid_set to set the symlink to the
                  * new dir.*/
-                posix_handle_unset(this, stbuf.ia_gfid, NULL);
+                posix_handle_unset_gfid(this, stbuf.ia_gfid);
         }
     } else if (frame->root->pid != GF_SERVER_PID_TRASH) {
         op_ret = -1;
@@ -726,7 +856,7 @@ posix_mkdir(call_frame_t *frame, xlator_t *this, loc_t *loc, mode_t mode,
     }
 
     op_ret = posix_pstat(this, loc->parent, loc->pargfid, par_path, &preparent,
-                         _gf_false);
+                         _gf_false, _gf_true);
     if (op_ret == -1) {
         op_errno = errno;
         gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_LSTAT_FAILED,
@@ -873,7 +1003,7 @@ posix_mkdir(call_frame_t *frame, xlator_t *this, loc_t *loc, mode_t mode,
         goto out;
     }
 #endif
-    op_ret = posix_acl_xattr_set(this, real_path, xdata);
+    op_ret = posix_acl_xattr_set(real_path, xdata);
     if (op_ret) {
         gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_ACL_FAILED,
                "setting ACLs on %s failed ", real_path);
@@ -895,7 +1025,8 @@ posix_mkdir(call_frame_t *frame, xlator_t *this, loc_t *loc, mode_t mode,
         gfid_set = _gf_true;
     }
 
-    op_ret = posix_pstat(this, loc->inode, NULL, real_path, &stbuf, _gf_false);
+    op_ret = posix_pstat(this, loc->inode, NULL, real_path, &stbuf, _gf_false,
+                         _gf_true);
     if (op_ret == -1) {
         op_errno = errno;
         gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_LSTAT_FAILED,
@@ -906,7 +1037,7 @@ posix_mkdir(call_frame_t *frame, xlator_t *this, loc_t *loc, mode_t mode,
     posix_set_ctime(frame, this, real_path, -1, loc->inode, &stbuf);
 
     op_ret = posix_pstat(this, loc->parent, loc->pargfid, par_path, &postparent,
-                         _gf_false);
+                         _gf_false, _gf_true);
     if (op_ret == -1) {
         op_errno = errno;
         gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_LSTAT_FAILED,
@@ -1019,7 +1150,7 @@ posix_unlink_gfid_handle_and_entry(call_frame_t *frame, xlator_t *this,
 
         if (loc->inode->fd_count == 0) {
             UNLOCK(&loc->inode->lock);
-            ret = posix_handle_unset(this, stbuf->ia_gfid, NULL);
+            ret = posix_handle_unset_gfid(this, stbuf->ia_gfid);
         } else {
             UNLOCK(&loc->inode->lock);
             ret = posix_move_gfid_to_unlink(this, stbuf->ia_gfid, loc);
@@ -1040,7 +1171,8 @@ posix_unlink_gfid_handle_and_entry(call_frame_t *frame, xlator_t *this,
         /* Since this stat is to get link count and not for time
          * attributes, intentionally passing inode as NULL
          */
-        ret = posix_pstat(this, NULL, loc->gfid, real_path, &prebuf, _gf_true);
+        ret = posix_pstat(this, NULL, loc->gfid, real_path, &prebuf, _gf_true,
+                          _gf_false);
         if (ret) {
             UNLOCK(&loc->inode->lock);
             locked = _gf_false;
@@ -1070,10 +1202,12 @@ posix_unlink_gfid_handle_and_entry(call_frame_t *frame, xlator_t *this,
         posix_set_ctime(frame, this, NULL, -1, loc->inode, stbuf);
     }
 
-    ret = dict_set_uint32(rsp_dict, GET_LINK_COUNT, prebuf.ia_nlink);
-    if (ret)
-        gf_msg(this->name, GF_LOG_WARNING, 0, P_MSG_SET_XDATA_FAIL,
-               "failed to set " GET_LINK_COUNT " for %s", real_path);
+    if (rsp_dict) {
+        ret = dict_set_uint32(rsp_dict, GET_LINK_COUNT, prebuf.ia_nlink);
+        if (ret)
+            gf_msg(this->name, GF_LOG_WARNING, 0, P_MSG_SET_XDATA_FAIL,
+                   "failed to set " GET_LINK_COUNT " for %s", real_path);
+    }
 
     return 0;
 
@@ -1085,6 +1219,49 @@ err:
     return -1;
 }
 
+static int
+posix_unlink_stale_linkto(call_frame_t *frame, xlator_t *this,
+                          const char *real_path, int32_t *op_errno, loc_t *loc)
+{
+    int ret = 0;
+    struct iatt stbuf = {
+        0,
+    };
+
+    /* get the stale file gfid and stat-info */
+    ret = posix_pstat(this, NULL, NULL, real_path, &stbuf, _gf_false,
+                      _gf_false);
+    if (ret) {
+        if (errno == ENOENT) {
+            ret = 0; /* retry creation if file doesn't exist */
+            gf_msg(this->name, GF_LOG_INFO, errno, P_MSG_LSTAT_FAILED,
+                   "lstat on %s failed: file unlinked by another client",
+                   real_path);
+        } else {
+            gf_msg(this->name, GF_LOG_INFO, errno, P_MSG_LSTAT_FAILED,
+                   "lstat on %s failed", real_path);
+        }
+        goto out;
+    }
+    /* unlink only if linkto file*/
+    if (IS_DHT_LINKFILE_MODE(&stbuf)) {
+        gf_msg(this->name, GF_LOG_INFO, 0, P_MSG_HANDLE_CREATE,
+               "unlinking stale linkto: %s gfid: %s", real_path,
+               uuid_utoa(stbuf.ia_gfid));
+        ret = posix_unlink_gfid_handle_and_entry(
+            frame, this, real_path, &stbuf, op_errno, loc, _gf_false, NULL);
+
+    } else {
+        gf_msg(this->name, GF_LOG_DEBUG, 0, P_MSG_HANDLE_CREATE,
+               "skip unlinking stale data-file: %s gfid: %s", real_path,
+               uuid_utoa(stbuf.ia_gfid));
+        ret = -1;
+    }
+
+out:
+    return ret;
+}
+
 static gf_boolean_t
 posix_skip_non_linkto_unlink(dict_t *xdata, loc_t *loc, char *key,
                              const int keylen, const char *linkto_xattr,
@@ -1199,7 +1376,7 @@ posix_unlink(call_frame_t *frame, xlator_t *this, loc_t *loc, int xflag,
     }
 
     op_ret = posix_pstat(this, loc->parent, loc->pargfid, par_path, &preparent,
-                         _gf_false);
+                         _gf_false, _gf_true);
     if (op_ret == -1) {
         op_errno = errno;
         gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_LSTAT_FAILED,
@@ -1324,7 +1501,7 @@ posix_unlink(call_frame_t *frame, xlator_t *this, loc_t *loc, int xflag,
     }
 
     if (fdstat_requested) {
-        op_ret = posix_fdstat(this, loc->inode, fd, &postbuf);
+        op_ret = posix_fdstat(this, loc->inode, fd, &postbuf, _gf_true);
         if (op_ret == -1) {
             op_errno = errno;
             gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_FSTAT_FAILED,
@@ -1342,7 +1519,7 @@ posix_unlink(call_frame_t *frame, xlator_t *this, loc_t *loc, int xflag,
     }
 
     op_ret = posix_pstat(this, loc->parent, loc->pargfid, par_path, &postparent,
-                         _gf_false);
+                         _gf_false, _gf_true);
     if (op_ret == -1) {
         op_errno = errno;
         gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_LSTAT_FAILED,
@@ -1416,6 +1593,19 @@ posix_rmdir(call_frame_t *frame, xlator_t *this, loc_t *loc, int flags,
         goto out;
     }
 
+#ifdef __NetBSD__
+    /* Same for NetBSD's .attribute directory */
+    if (__is_root_gfid(loc->pargfid) &&
+        (strcmp(loc->name, ".attribute") == 0)) {
+        gf_msg(this->name, GF_LOG_WARNING, EPERM, P_MSG_RMDIR_NOT_PERMITTED,
+               "rmdir issued on .attribute, which"
+               "is not permitted");
+        op_errno = EPERM;
+        op_ret = -1;
+        goto out;
+    }
+#endif
+
     priv = this->private;
 
     MAKE_ENTRY_HANDLE(real_path, par_path, this, loc, &stbuf);
@@ -1426,7 +1616,7 @@ posix_rmdir(call_frame_t *frame, xlator_t *this, loc_t *loc, int flags,
     }
 
     op_ret = posix_pstat(this, loc->parent, loc->pargfid, par_path, &preparent,
-                         _gf_false);
+                         _gf_false, _gf_true);
     if (op_ret == -1) {
         op_errno = errno;
         gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_LSTAT_FAILED,
@@ -1453,7 +1643,7 @@ posix_rmdir(call_frame_t *frame, xlator_t *this, loc_t *loc, int flags,
 
     if (op_ret == 0) {
         if (posix_symlinks_match(this, loc, stbuf.ia_gfid))
-            posix_handle_unset(this, stbuf.ia_gfid, NULL);
+            posix_handle_unset_gfid(this, stbuf.ia_gfid);
     }
 
     if (op_errno == EEXIST)
@@ -1479,7 +1669,7 @@ posix_rmdir(call_frame_t *frame, xlator_t *this, loc_t *loc, int flags,
     }
 
     op_ret = posix_pstat(this, loc->parent, loc->pargfid, par_path, &postparent,
-                         _gf_false);
+                         _gf_false, _gf_true);
     if (op_ret == -1) {
         op_errno = errno;
         gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_LSTAT_FAILED,
@@ -1549,7 +1739,7 @@ posix_symlink(call_frame_t *frame, xlator_t *this, const char *linkname,
     SET_FS_ID(frame->root->uid, gid);
 
     op_ret = posix_pstat(this, loc->parent, loc->pargfid, par_path, &preparent,
-                         _gf_false);
+                         _gf_false, _gf_true);
     if (op_ret == -1) {
         op_errno = errno;
         gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_LSTAT_FAILED,
@@ -1583,7 +1773,7 @@ posix_symlink(call_frame_t *frame, xlator_t *this, const char *linkname,
         goto out;
     }
 #endif
-    op_ret = posix_acl_xattr_set(this, real_path, xdata);
+    op_ret = posix_acl_xattr_set(real_path, xdata);
     if (op_ret) {
         gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_ACL_FAILED,
                "setting ACLs on %s failed", real_path);
@@ -1618,7 +1808,8 @@ ignore:
         gfid_set = _gf_true;
     }
 
-    op_ret = posix_pstat(this, loc->inode, NULL, real_path, &stbuf, _gf_false);
+    op_ret = posix_pstat(this, loc->inode, NULL, real_path, &stbuf, _gf_false,
+                         _gf_true);
     if (op_ret == -1) {
         op_errno = errno;
         gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_LSTAT_FAILED,
@@ -1627,7 +1818,7 @@ ignore:
     }
 
     op_ret = posix_pstat(this, loc->parent, loc->pargfid, par_path, &postparent,
-                         _gf_false);
+                         _gf_false, _gf_true);
     if (op_ret == -1) {
         op_errno = errno;
         gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_LSTAT_FAILED,
@@ -1671,7 +1862,7 @@ posix_rename(call_frame_t *frame, xlator_t *this, loc_t *oldloc, loc_t *newloc,
         0,
     };
     struct posix_private *priv = NULL;
-    char was_present = 1;
+    int was_present = 1;
     struct iatt preoldparent = {
         0,
     };
@@ -1689,6 +1880,7 @@ posix_rename(call_frame_t *frame, xlator_t *this, loc_t *oldloc, loc_t *newloc,
     uuid_t victim = {0};
     int was_dir = 0;
     int nlink = 0;
+    int64_t blocks = 0;
     char *pgfid_xattr_key = NULL;
     int32_t nlink_samepgfid = 0;
     char *gfid_path = NULL;
@@ -1731,7 +1923,7 @@ posix_rename(call_frame_t *frame, xlator_t *this, loc_t *oldloc, loc_t *newloc,
     }
 
     op_ret = posix_pstat(this, oldloc->parent, oldloc->pargfid, par_oldpath,
-                         &preoldparent, _gf_false);
+                         &preoldparent, _gf_false, _gf_true);
     if (op_ret == -1) {
         op_errno = errno;
         gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_LSTAT_FAILED,
@@ -1740,7 +1932,7 @@ posix_rename(call_frame_t *frame, xlator_t *this, loc_t *oldloc, loc_t *newloc,
     }
 
     op_ret = posix_pstat(this, newloc->parent, newloc->pargfid, par_newpath,
-                         &prenewparent, _gf_false);
+                         &prenewparent, _gf_false, _gf_false);
     if (op_ret == -1) {
         op_errno = errno;
         gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_LSTAT_FAILED,
@@ -1749,7 +1941,7 @@ posix_rename(call_frame_t *frame, xlator_t *this, loc_t *oldloc, loc_t *newloc,
     }
 
     op_ret = posix_pstat(this, newloc->inode, NULL, real_newpath, &stbuf,
-                         _gf_false);
+                         _gf_false, _gf_false);
     if ((op_ret == -1) && (errno == ENOENT)) {
         was_present = 0;
     } else {
@@ -1757,25 +1949,26 @@ posix_rename(call_frame_t *frame, xlator_t *this, loc_t *oldloc, loc_t *newloc,
         if (IA_ISDIR(stbuf.ia_type))
             was_dir = 1;
         nlink = stbuf.ia_nlink;
-    }
-
-    if (was_present && IA_ISDIR(stbuf.ia_type) && !newloc->inode) {
-        gf_msg(this->name, GF_LOG_WARNING, EEXIST, P_MSG_DIR_FOUND,
-               "found directory at %s while expecting ENOENT", real_newpath);
-        op_ret = -1;
-        op_errno = EEXIST;
-        goto out;
-    }
-
-    if (was_present && IA_ISDIR(stbuf.ia_type) &&
-        gf_uuid_compare(newloc->inode->gfid, stbuf.ia_gfid)) {
-        gf_msg(this->name, GF_LOG_WARNING, EEXIST, P_MSG_DIR_FOUND,
-               "found directory %s at %s while renaming %s",
-               uuid_utoa_r(newloc->inode->gfid, olddirid), real_newpath,
-               uuid_utoa_r(stbuf.ia_gfid, newdirid));
-        op_ret = -1;
-        op_errno = EEXIST;
-        goto out;
+        blocks = stbuf.ia_blocks;
+        if (IA_ISDIR(stbuf.ia_type)) {
+            if (!newloc->inode) {
+                gf_msg(this->name, GF_LOG_WARNING, EEXIST, P_MSG_DIR_FOUND,
+                       "found directory at %s while expecting ENOENT",
+                       real_newpath);
+                op_ret = -1;
+                op_errno = EEXIST;
+                goto out;
+            }
+            if (gf_uuid_compare(newloc->inode->gfid, stbuf.ia_gfid)) {
+                gf_msg(this->name, GF_LOG_WARNING, EEXIST, P_MSG_DIR_FOUND,
+                       "found directory %s at %s while renaming %s",
+                       uuid_utoa_r(newloc->inode->gfid, olddirid), real_newpath,
+                       uuid_utoa_r(stbuf.ia_gfid, newdirid));
+                op_ret = -1;
+                op_errno = EEXIST;
+                goto out;
+            }
+        }
     }
 
     op_ret = posix_inode_ctx_get_all(oldloc->inode, this, &ctx_old);
@@ -1794,9 +1987,6 @@ posix_rename(call_frame_t *frame, xlator_t *this, loc_t *oldloc, loc_t *newloc,
         }
     }
 
-    if (IA_ISDIR(oldloc->inode->ia_type))
-        posix_handle_unset(this, oldloc->inode->gfid, NULL);
-
     pthread_mutex_lock(&ctx_old->pgfid_lock);
     {
         if (!IA_ISDIR(oldloc->inode->ia_type) && priv->update_pgfid_nlinks) {
@@ -1812,7 +2002,7 @@ posix_rename(call_frame_t *frame, xlator_t *this, loc_t *oldloc, loc_t *newloc,
             locked = _gf_true;
             get_link_count = _gf_true;
             op_ret = posix_pstat(this, newloc->inode, newloc->gfid,
-                                 real_newpath, &stbuf, _gf_false);
+                                 real_newpath, &stbuf, _gf_false, _gf_false);
             if ((op_ret == -1) && (errno != ENOENT)) {
                 op_errno = errno;
                 gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_LSTAT_FAILED,
@@ -1825,10 +2015,10 @@ posix_rename(call_frame_t *frame, xlator_t *this, loc_t *oldloc, loc_t *newloc,
         if (op_ret == -1) {
             op_errno = errno;
             if (op_errno == ENOTEMPTY) {
-                gf_msg_debug(this->name, 0,
+                gf_msg_debug(this->name, op_errno,
                              "rename of %s to"
-                             " %s failed: %s",
-                             real_oldpath, real_newpath, strerror(op_errno));
+                             " %s failed",
+                             real_oldpath, real_newpath);
             } else {
                 gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_RENAME_FAILED,
                        "rename of %s to %s failed", real_oldpath, real_newpath);
@@ -1887,18 +2077,19 @@ unlock:
     }
 
     if (was_dir)
-        posix_handle_unset(this, victim, NULL);
+        posix_handle_unset_gfid(this, victim);
 
     if (was_present && !was_dir && nlink == 1)
-        posix_handle_unset(this, victim, NULL);
+        posix_handle_unset_gfid(this, victim);
 
     if (IA_ISDIR(oldloc->inode->ia_type)) {
+        posix_handle_unset_gfid(this, oldloc->inode->gfid);
         posix_handle_soft(this, real_newpath, newloc, oldloc->inode->gfid,
                           NULL);
     }
 
     op_ret = posix_pstat(this, newloc->inode, NULL, real_newpath, &stbuf,
-                         _gf_false);
+                         _gf_false, _gf_true);
     if (op_ret == -1) {
         op_errno = errno;
         gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_LSTAT_FAILED,
@@ -1912,7 +2103,7 @@ unlock:
     posix_set_ctime(frame, this, real_newpath, -1, oldloc->inode, &stbuf);
 
     op_ret = posix_pstat(this, oldloc->parent, oldloc->pargfid, par_oldpath,
-                         &postoldparent, _gf_false);
+                         &postoldparent, _gf_false, _gf_true);
     if (op_ret == -1) {
         op_errno = errno;
         gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_LSTAT_FAILED,
@@ -1924,7 +2115,7 @@ unlock:
                            &postoldparent);
 
     op_ret = posix_pstat(this, newloc->parent, newloc->pargfid, par_newpath,
-                         &postnewparent, _gf_false);
+                         &postnewparent, _gf_false, _gf_true);
     if (op_ret == -1) {
         op_errno = errno;
         gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_LSTAT_FAILED,
@@ -1935,8 +2126,14 @@ unlock:
     posix_set_parent_ctime(frame, this, par_newpath, -1, newloc->parent,
                            &postnewparent);
 
-    if (was_present)
+    if (was_present) {
         unwind_dict = posix_dict_set_nlink(xdata, unwind_dict, nlink);
+        int ret = dict_set_int64(unwind_dict, GF_GET_FILE_BLOCK_COUNT, blocks);
+        if (IS_ERROR(ret)) {
+            gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_DICT_SET_FAILED,
+                   "%s: dict set of Block size failed", par_newpath);
+        }
+    }
     op_ret = 0;
 out:
 
@@ -2011,7 +2208,7 @@ posix_link(call_frame_t *frame, xlator_t *this, loc_t *oldloc, loc_t *newloc,
     }
 
     op_ret = posix_pstat(this, newloc->parent, newloc->pargfid, par_newpath,
-                         &preparent, _gf_false);
+                         &preparent, _gf_false, _gf_true);
     if (op_ret == -1) {
         op_errno = errno;
         gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_LSTAT_FAILED,
@@ -2019,10 +2216,20 @@ posix_link(call_frame_t *frame, xlator_t *this, loc_t *oldloc, loc_t *newloc,
         goto out;
     }
 
+real_op:
     op_ret = sys_link(real_oldpath, real_newpath);
 
     if (op_ret == -1) {
         op_errno = errno;
+        if (op_errno == EEXIST) {
+            if (dict_get_sizen(xdata, GF_FORCE_REPLACE_KEY)) {
+                dict_del_sizen(xdata, GF_FORCE_REPLACE_KEY);
+                op_ret = posix_unlink_stale_linkto(frame, this, real_newpath,
+                                                   &op_errno, newloc);
+                if (op_ret == 0)
+                    goto real_op;
+            }
+        }
         gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_LINK_FAILED,
                "link %s to %s failed", real_oldpath, real_newpath);
         goto out;
@@ -2031,7 +2238,7 @@ posix_link(call_frame_t *frame, xlator_t *this, loc_t *oldloc, loc_t *newloc,
     entry_created = _gf_true;
 
     op_ret = posix_pstat(this, newloc->inode, NULL, real_newpath, &stbuf,
-                         _gf_false);
+                         _gf_false, _gf_true);
     if (op_ret == -1) {
         op_errno = errno;
         gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_LSTAT_FAILED,
@@ -2042,7 +2249,7 @@ posix_link(call_frame_t *frame, xlator_t *this, loc_t *oldloc, loc_t *newloc,
     posix_set_ctime(frame, this, real_newpath, -1, newloc->inode, &stbuf);
 
     op_ret = posix_pstat(this, newloc->parent, newloc->pargfid, par_newpath,
-                         &postparent, _gf_false);
+                         &postparent, _gf_false, _gf_true);
     if (op_ret == -1) {
         op_errno = errno;
         gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_LSTAT_FAILED,
@@ -2128,7 +2335,7 @@ posix_create(call_frame_t *frame, xlator_t *this, loc_t *loc, int32_t flags,
     };
     struct posix_fd *pfd = NULL;
     struct posix_private *priv = NULL;
-    char was_present = 1;
+    int was_present = 1;
 
     gid_t gid = 0;
     struct iatt preparent = {
@@ -2174,7 +2381,7 @@ posix_create(call_frame_t *frame, xlator_t *this, loc_t *loc, int32_t flags,
     }
 
     op_ret = posix_pstat(this, loc->parent, loc->pargfid, par_path, &preparent,
-                         _gf_false);
+                         _gf_false, _gf_true);
     if (op_ret == -1) {
         op_errno = errno;
         gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_LSTAT_FAILED,
@@ -2192,12 +2399,10 @@ posix_create(call_frame_t *frame, xlator_t *this, loc_t *loc, int32_t flags,
         _flags = flags | O_CREAT;
     }
 
-    op_ret = posix_pstat(this, loc->inode, NULL, real_path, &stbuf, _gf_false);
+    op_ret = posix_pstat(this, loc->inode, NULL, real_path, &stbuf, _gf_false,
+                         _gf_false);
     if ((op_ret == -1) && (errno == ENOENT)) {
         was_present = 0;
-    }
-
-    if (!was_present) {
         if (posix_is_layout_stale(xdata, par_path, this)) {
             op_ret = -1;
             op_errno = EIO;
@@ -2249,7 +2454,7 @@ posix_create(call_frame_t *frame, xlator_t *this, loc_t *loc, int32_t flags,
                "chown on %s failed", real_path);
     }
 #endif
-    op_ret = posix_acl_xattr_set(this, real_path, xdata);
+    op_ret = posix_acl_xattr_set(real_path, xdata);
     if (op_ret) {
         gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_ACL_FAILED,
                "setting ACLs on %s failed", real_path);
@@ -2284,7 +2489,7 @@ fill_stat:
         gfid_set = _gf_true;
     }
 
-    op_ret = posix_fdstat(this, loc->inode, _fd, &stbuf);
+    op_ret = posix_fdstat(this, loc->inode, _fd, &stbuf, _gf_true);
     if (op_ret == -1) {
         op_errno = errno;
         gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_FSTAT_FAILED,
@@ -2295,7 +2500,7 @@ fill_stat:
     posix_set_ctime(frame, this, real_path, -1, loc->inode, &stbuf);
 
     op_ret = posix_pstat(this, loc->parent, loc->pargfid, par_path, &postparent,
-                         _gf_false);
+                         _gf_false, _gf_true);
     if (op_ret == -1) {
         op_errno = errno;
         gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_LSTAT_FAILED,
@@ -2379,7 +2584,7 @@ posix_put(call_frame_t *frame, xlator_t *this, loc_t *loc, mode_t mode,
     }
 
     op_ret = posix_pstat(this, loc->parent, loc->pargfid, par_path, &preparent,
-                         _gf_false);
+                         _gf_false, _gf_true);
     if (op_ret < 0) {
         op_errno = errno;
         gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_LSTAT_FAILED,
@@ -2408,7 +2613,7 @@ posix_put(call_frame_t *frame, xlator_t *this, loc_t *loc, mode_t mode,
     }
 
     op_ret = posix_pstat(this, loc->parent, loc->pargfid, par_path, &postparent,
-                         _gf_false);
+                         _gf_false, _gf_true);
     if (op_ret < 0) {
         op_errno = errno;
         gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_LSTAT_FAILED,
@@ -2442,7 +2647,7 @@ posix_put(call_frame_t *frame, xlator_t *this, loc_t *loc, mode_t mode,
     }
 
     op_ret = posix_pstat(this, loc->inode, loc->gfid, real_path, &stbuf,
-                         _gf_false);
+                         _gf_false, _gf_true);
     if (op_ret < 0) {
         op_errno = errno;
         gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_LSTAT_FAILED,
